@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2007  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2007-2008  Kouhei Sutou <kou@cozmixng.org>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -18,7 +18,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+#  include "config.h"
 #endif /* HAVE_CONFIG_H */
 
 #include <stdlib.h>
@@ -33,13 +33,14 @@
 #endif
 
 #include "cut-loader.h"
+#include "cut-mach-o-loader.h"
 #include "cut-test-iterator.h"
 #include "cut-experimental.h"
 
-#define TEST_SUITE_SO_NAME_PREFIX "suite_"
+#define TEST_SUITE_SO_NAME_PREFIX "suite"
 #define TEST_NAME_PREFIX "test_"
 #define DATA_SETUP_FUNCTION_NAME_PREFIX "data_"
-#define ATTRIBUTES_FUNCTION_NAME_PREFIX "attributes_"
+#define ATTRIBUTES_SETUP_FUNCTION_NAME_PREFIX "attributes_"
 #define CUT_LOADER_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CUT_TYPE_LOADER, CutLoaderPrivate))
 
 typedef enum {
@@ -55,12 +56,17 @@ struct _CutLoaderPrivate
     GList *symbols;
     GModule *module;
     CutBinaryType binary_type;
+    CutMachOLoader *mach_o_loader;
+    gboolean keep_opening;
+    gboolean enable_convenience_attribute_definition;
+    gchar *base_directory;
 };
 
 enum
 {
     PROP_0,
-    PROP_SO_FILENAME
+    PROP_SO_FILENAME,
+    PROP_BASE_DIRECTORY
 };
 
 G_DEFINE_TYPE (CutLoader, cut_loader, G_TYPE_OBJECT)
@@ -94,6 +100,13 @@ cut_loader_class_init (CutLoaderClass *klass)
                                G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
     g_object_class_install_property(gobject_class, PROP_SO_FILENAME, spec);
 
+    spec = g_param_spec_string("base-directory",
+                               "Base directory",
+                               "The base directory of shared object",
+                               NULL,
+                               G_PARAM_READWRITE);
+    g_object_class_install_property(gobject_class, PROP_BASE_DIRECTORY, spec);
+
     g_type_class_add_private(gobject_class, sizeof(CutLoaderPrivate));
 }
 
@@ -104,6 +117,10 @@ cut_loader_init (CutLoader *loader)
 
     priv->so_filename = NULL;
     priv->binary_type = CUT_BINARY_TYPE_UNKNOWN;
+    priv->mach_o_loader = NULL;
+    priv->keep_opening = FALSE;
+    priv->enable_convenience_attribute_definition = FALSE;
+    priv->base_directory = NULL;
 }
 
 static void
@@ -126,13 +143,24 @@ dispose (GObject *object)
         priv->so_filename = NULL;
     }
 
+    if (priv->mach_o_loader) {
+        g_object_unref(priv->mach_o_loader);
+        priv->mach_o_loader = NULL;
+    }
+
     if (priv->module) {
-        g_module_close(priv->module);
+        if (!priv->keep_opening)
+            g_module_close(priv->module);
         priv->module = NULL;
     }
 
     free_symbols(priv);
 
+    if (priv->base_directory) {
+        g_free(priv->base_directory);
+        priv->base_directory = NULL;
+    }
+
     G_OBJECT_CLASS(cut_loader_parent_class)->dispose(object);
 }
 
@@ -148,6 +176,10 @@ set_property (GObject      *object,
       case PROP_SO_FILENAME:
         priv->so_filename = g_value_dup_string(value);
         break;
+      case PROP_BASE_DIRECTORY:
+        cut_loader_set_base_directory(CUT_LOADER(object),
+                                      g_value_get_string(value));
+        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
@@ -166,6 +198,9 @@ get_property (GObject    *object,
       case PROP_SO_FILENAME:
         g_value_set_string(value, priv->so_filename);
         break;
+      case PROP_BASE_DIRECTORY:
+        g_value_set_string(value, priv->base_directory);
+        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
@@ -180,6 +215,48 @@ cut_loader_new (const gchar *soname)
                         NULL);
 }
 
+gboolean
+cut_loader_get_keep_opening (CutLoader *loader)
+{
+    return CUT_LOADER_GET_PRIVATE(loader)->keep_opening;
+}
+
+void
+cut_loader_set_keep_opening (CutLoader *loader, gboolean keep_opening)
+{
+    CUT_LOADER_GET_PRIVATE(loader)->keep_opening = keep_opening;
+}
+
+gboolean
+cut_loader_get_enable_convenience_attribute_definition (CutLoader *loader)
+{
+    return CUT_LOADER_GET_PRIVATE(loader)->enable_convenience_attribute_definition;
+}
+
+void
+cut_loader_set_enable_convenience_attribute_definition (CutLoader *loader, gboolean enable_convenience_attribute_definition)
+{
+    CUT_LOADER_GET_PRIVATE(loader)->enable_convenience_attribute_definition =
+        enable_convenience_attribute_definition;
+}
+
+const gchar *
+cut_loader_get_base_directory (CutLoader *loader)
+{
+    return CUT_LOADER_GET_PRIVATE(loader)->base_directory;
+}
+
+void
+cut_loader_set_base_directory (CutLoader *loader, const gchar *base_directory)
+{
+    CutLoaderPrivate *priv;
+
+    priv = CUT_LOADER_GET_PRIVATE(loader);
+    if (priv->base_directory)
+        g_free(priv->base_directory);
+    priv->base_directory = g_strdup(base_directory);
+}
+
 static inline gboolean
 is_test_function_name (const gchar *name)
 {
@@ -188,8 +265,8 @@ is_test_function_name (const gchar *name)
 
 #ifdef HAVE_LIBBFD
 
-gboolean
-cut_loader_support_attribute (CutLoader *loader)
+static gboolean
+cut_loader_support_attribute_bfd (CutLoader *loader)
 {
     CutLoaderPrivate *priv;
 
@@ -198,7 +275,7 @@ cut_loader_support_attribute (CutLoader *loader)
 }
 
 static GList *
-collect_symbols (CutLoaderPrivate *priv)
+collect_symbols_bfd (CutLoaderPrivate *priv)
 {
     GList *symbols = NULL;
     long storage_needed;
@@ -253,8 +330,8 @@ collect_symbols (CutLoaderPrivate *priv)
 
 #else
 
-gboolean
-cut_loader_support_attribute (CutLoader *loader)
+static gboolean
+cut_loader_support_attribute_scan (CutLoader *loader)
 {
     CutLoaderPrivate *priv;
 
@@ -297,7 +374,7 @@ is_valid_symbol_name (GString *name)
 }
 
 static GList *
-collect_symbols (CutLoaderPrivate *priv)
+collect_symbols_scan (CutLoaderPrivate *priv)
 {
     FILE *input;
     GString *name;
@@ -348,6 +425,40 @@ collect_symbols (CutLoaderPrivate *priv)
 }
 #endif
 
+gboolean
+cut_loader_support_attribute (CutLoader *loader)
+{
+    CutLoaderPrivate *priv;
+
+    priv = CUT_LOADER_GET_PRIVATE(loader);
+    if (!priv->enable_convenience_attribute_definition)
+        return FALSE;
+
+    if (priv->mach_o_loader) {
+        return cut_mach_o_loader_support_attribute(priv->mach_o_loader);
+    } else {
+#ifdef HAVE_BFD_H
+        return cut_loader_support_attribute_bfd(loader);
+#else
+        return cut_loader_support_attribute_scan(loader);
+#endif
+    }
+}
+
+static GList *
+collect_symbols (CutLoaderPrivate *priv)
+{
+    if (priv->mach_o_loader) {
+        return cut_mach_o_loader_collect_symbols(priv->mach_o_loader);
+    } else {
+#ifdef HAVE_BFD_H
+        return collect_symbols_bfd(priv);
+#else
+        return collect_symbols_scan(priv);
+#endif
+    }
+}
+
 static GList *
 collect_test_functions (CutLoaderPrivate *priv)
 {
@@ -372,18 +483,12 @@ is_including_test_name (const gchar *function_name, const gchar *test_name)
 static gboolean
 is_valid_attribute_function_name (const gchar *function_name, const gchar *test_name)
 {
-    return !g_str_has_prefix(function_name, ATTRIBUTES_FUNCTION_NAME_PREFIX) &&
+    return !g_str_has_prefix(function_name,
+                             ATTRIBUTES_SETUP_FUNCTION_NAME_PREFIX) &&
         !g_str_has_prefix(function_name, DATA_SETUP_FUNCTION_NAME_PREFIX) &&
         is_including_test_name(function_name, test_name);
 }
 
-static gboolean
-is_valid_attributes_function_name (const gchar *function_name, const gchar *test_name)
-{
-    return g_str_has_prefix(function_name, ATTRIBUTES_FUNCTION_NAME_PREFIX) &&
-        is_including_test_name(function_name, test_name);
-}
-
 static gchar *
 get_attribute_name (const gchar *attribute_function_name, const gchar *test_name)
 {
@@ -396,23 +501,16 @@ get_attribute_name (const gchar *attribute_function_name, const gchar *test_name
 }
 
 typedef const gchar *(*CutAttributeItemFunction)     (void);
-typedef CutTestAttribute *(*CutAttributeFunction)     (void);
 
-static CutTestAttribute *
-cut_test_attribute_new (const gchar *name, const gchar *value)
+static void
+apply_attributes (CutLoaderPrivate *priv, CutTest *test, const gchar *test_name)
 {
-    CutTestAttribute *attribute = g_new0(CutTestAttribute, 1);
-    attribute->name = g_strdup(name);
-    attribute->value = g_strdup(value);
-    return attribute;
-}
+    GList *node;
 
-static GList *
-collect_attributes (CutLoaderPrivate *priv, const gchar *test_name)
-{
-    GList *attributes = NULL, *node;
+    if (!test)
+        return;
     if (!test_name)
-        return NULL;
+        return;
 
     for (node = priv->symbols; node; node = g_list_next(node)) {
         gchar *function_name = node->data;
@@ -422,35 +520,89 @@ collect_attributes (CutLoaderPrivate *priv, const gchar *test_name)
             CutAttributeItemFunction function = NULL;
             g_module_symbol(priv->module, function_name, (gpointer)&function);
             if (function) {
-                CutTestAttribute *attribute;
-                gchar *name = get_attribute_name(function_name, test_name);
-                const gchar *value = function();
-                attribute = cut_test_attribute_new(name, value);
+                gchar *name;
+                const gchar *value;
+
+                name = get_attribute_name(function_name, test_name);
+                value = function();
+                cut_test_set_attribute(test, name, value);
                 g_free(name);
-                attributes = g_list_append(attributes, attribute);
-            }
-        } else if (is_valid_attributes_function_name(function_name, test_name)) {
-            CutAttributeFunction function = NULL;
-            g_module_symbol(priv->module, function_name, (gpointer)&function);
-            if (function) {
-                CutTestAttribute *attribute = function();
-                while (TRUE) {
-                    CutTestAttribute *new_attribute;
-                    if (!attribute->name || !attribute->value)
-                        break;
-                    new_attribute = cut_test_attribute_new(attribute->name,
-                                                           attribute->value);
-                    attributes = g_list_append(attributes, new_attribute);
-                    attribute++;
-                }
             }
         }
     }
-    return attributes;
 }
 
 static void
-cb_complete (CutTestCase *test_case, gpointer data)
+register_test (CutLoader *loader, CutTestCase *test_case,
+               const gchar *name, CutTestFunction function)
+{
+    CutLoaderPrivate *priv;
+    CutTest *test;
+    gchar *data_setup_function_name;
+    CutDataSetupFunction data_setup_function = NULL;
+    gchar *attributes_setup_function_name;
+    CutAttributesSetupFunction attributes_setup_function = NULL;
+
+    priv = CUT_LOADER_GET_PRIVATE(loader);
+
+    data_setup_function_name =
+        g_strconcat(DATA_SETUP_FUNCTION_NAME_PREFIX,
+                    name + strlen(TEST_NAME_PREFIX),
+                    NULL);
+    g_module_symbol(priv->module, data_setup_function_name,
+                    (gpointer)&data_setup_function);
+
+    attributes_setup_function_name =
+        g_strconcat(ATTRIBUTES_SETUP_FUNCTION_NAME_PREFIX,
+                    name + strlen(TEST_NAME_PREFIX),
+                    NULL);
+    g_module_symbol(priv->module, attributes_setup_function_name,
+                    (gpointer)&attributes_setup_function);
+
+    if (data_setup_function) {
+        CutTestIterator *test_iterator;
+        CutIteratedTestFunction test_function;
+
+        test_function = (CutIteratedTestFunction)function;
+        test_iterator = cut_test_iterator_new(name,
+                                              test_function,
+                                              data_setup_function);
+        test = CUT_TEST(test_iterator);
+    } else {
+        test = cut_test_new(name, function);
+    }
+    cut_test_set_base_directory(test, priv->base_directory);
+
+    if (attributes_setup_function) {
+        CutTestContext *test_context, *original_test_context;
+
+        original_test_context = cut_test_context_current_get();
+        if (data_setup_function) {
+            test_context = cut_test_context_new(NULL, NULL, test_case,
+                                                CUT_TEST_ITERATOR(test), NULL);
+        } else {
+            test_context = cut_test_context_new(NULL, NULL, test_case,
+                                                NULL, test);
+        }
+        cut_test_context_current_set(test_context);
+        attributes_setup_function();
+        cut_test_context_current_set(original_test_context);
+        g_object_unref(test_case);
+    }
+
+    if (cut_loader_support_attribute(loader))
+        apply_attributes(priv, test, name);
+
+    cut_test_case_add_test(test_case, test);
+
+    g_object_unref(test);
+    g_free(data_setup_function_name);
+    g_free(attributes_setup_function_name);
+}
+
+static void
+cb_complete (CutTestCase *test_case, CutTestContext *test_context,
+             gboolean success, gpointer data)
 {
     CutLoader *loader = data;
     g_object_unref(loader);
@@ -462,8 +614,6 @@ create_test_case (CutLoader *loader)
     CutLoaderPrivate *priv;
     CutTestCase *test_case;
     CutSetupFunction setup = NULL;
-    CutGetCurrentTestContextFunction get_current_test_context = NULL;
-    CutSetCurrentTestContextFunction set_current_test_context = NULL;
     CutStartupFunction startup = NULL;
     CutShutdownFunction shutdown = NULL;
     CutTeardownFunction teardown = NULL;
@@ -473,10 +623,6 @@ create_test_case (CutLoader *loader)
 
     g_module_symbol(priv->module, "setup", (gpointer)&setup);
     g_module_symbol(priv->module, "teardown", (gpointer)&teardown);
-    g_module_symbol(priv->module, "get_current_test_context",
-                    (gpointer)&get_current_test_context);
-    g_module_symbol(priv->module, "set_current_test_context",
-                    (gpointer)&set_current_test_context);
     g_module_symbol(priv->module, "startup", (gpointer)&startup);
     g_module_symbol(priv->module, "shutdown", (gpointer)&shutdown);
 
@@ -492,38 +638,24 @@ create_test_case (CutLoader *loader)
     g_free(filename);
     test_case = cut_test_case_new(test_case_name,
                                   setup, teardown,
-                                  get_current_test_context,
-                                  set_current_test_context,
                                   startup, shutdown);
     g_free(test_case_name);
 
+    cut_test_set_base_directory(CUT_TEST(test_case), priv->base_directory);
+
     g_object_ref(loader);
     g_signal_connect(test_case, "complete", G_CALLBACK(cb_complete), loader);
 
     return test_case;
 }
 
-static void
-set_attributes (CutTest *test, GList *attributes)
-{
-    GList *node;
-    for (node = attributes; node; node = g_list_next(node)) {
-        CutTestAttribute *attribute = (CutTestAttribute *)node->data;
-        cut_test_set_attribute(test, attribute->name, attribute->value);
-        g_free((gchar *)attribute->name);
-        g_free((gchar *)attribute->value);
-        g_free(attribute);
-    }
-    g_list_free(attributes);
-}
-
 CutTestCase *
 cut_loader_load_test_case (CutLoader *loader)
 {
+    CutLoaderPrivate *priv;
     GList *node;
     GList *test_names;
     CutTestCase *test_case;
-    CutLoaderPrivate *priv;
 
     priv = CUT_LOADER_GET_PRIVATE(loader);
     if (!priv->so_filename)
@@ -531,8 +663,17 @@ cut_loader_load_test_case (CutLoader *loader)
 
     priv->module = g_module_open(priv->so_filename,
                                  G_MODULE_BIND_LAZY | G_MODULE_BIND_LOCAL);
-    if (!priv->module)
+    if (!priv->module) {
+        g_warning("can't load a shared object for test case: %s: %s",
+                  priv->so_filename, g_module_error());
         return NULL;
+    }
+
+    priv->mach_o_loader = cut_mach_o_loader_new(priv->so_filename);
+    if (!cut_mach_o_loader_is_mach_o(priv->mach_o_loader)) {
+        g_object_unref(priv->mach_o_loader);
+        priv->mach_o_loader = NULL;
+    }
 
     priv->symbols = collect_symbols(priv);
     if (!priv->symbols)
@@ -545,43 +686,12 @@ cut_loader_load_test_case (CutLoader *loader)
     test_case = create_test_case(loader);
     for (node = test_names; node; node = g_list_next(node)) {
         gchar *name;
-        CutTest *test;
         CutTestFunction function = NULL;
 
         name = node->data;
         g_module_symbol(priv->module, name, (gpointer)&function);
-        if (function) {
-            GList *attributes = NULL;
-            gchar *data_setup_function_name;
-            CutDataSetupFunction data_setup_function = NULL;
-
-            data_setup_function_name =
-                g_strconcat(DATA_SETUP_FUNCTION_NAME_PREFIX,
-                            name + strlen(TEST_NAME_PREFIX),
-                            NULL);
-            g_module_symbol(priv->module, data_setup_function_name,
-                            (gpointer)&data_setup_function);
-
-            if (data_setup_function) {
-                CutTestIterator *test_iterator;
-                CutIteratedTestFunction test_function;
-
-                test_function = (CutIteratedTestFunction)function;
-                test_iterator = cut_test_iterator_new(name,
-                                                      test_function,
-                                                      data_setup_function);
-                test = CUT_TEST(test_iterator);
-            } else {
-                test = cut_test_new(name, function);
-            }
-            if (cut_loader_support_attribute(loader))
-                attributes = collect_attributes(priv, name);
-            if (attributes) {
-                set_attributes(test, attributes);
-            }
-            cut_test_case_add_test(test_case, test);
-            g_object_unref(test);
-        }
+        if (function)
+            register_test(loader, test_case, name, function);
     }
     g_list_free(test_names);
 
@@ -592,13 +702,21 @@ static gchar *
 get_suite_prefix (CutLoaderPrivate *priv)
 {
     gchar *prefix, *base_name, *original_base_name;
+    gchar *character;
 
     original_base_name = base_name = g_path_get_basename(priv->so_filename);
-    if (g_str_has_prefix(base_name, TEST_SUITE_SO_NAME_PREFIX))
-        base_name += strlen(TEST_SUITE_SO_NAME_PREFIX);
+    if (g_str_has_prefix(base_name, TEST_SUITE_SO_NAME_PREFIX "_") ||
+        g_str_has_prefix(base_name, TEST_SUITE_SO_NAME_PREFIX "-"))
+        base_name += strlen(TEST_SUITE_SO_NAME_PREFIX "-");
     prefix = g_strndup(base_name,
                        strlen(base_name) - strlen("." G_MODULE_SUFFIX));
     g_free(original_base_name);
+
+    for (character = prefix; *character; character++) {
+        if (*character == '-')
+            *character = '_';
+    }
+
     return prefix;
 }
 
@@ -607,6 +725,7 @@ cut_loader_load_test_suite (CutLoader *loader)
 {
     CutLoaderPrivate *priv;
     gchar *prefix, *warmup_function_name, *cooldown_function_name;
+    CutTestSuite *test_suite;
     CutWarmupFunction warmup = NULL;
     CutCooldownFunction cooldown = NULL;
 
@@ -616,8 +735,11 @@ cut_loader_load_test_suite (CutLoader *loader)
         return NULL;
 
     priv->module = g_module_open(priv->so_filename, G_MODULE_BIND_LAZY);
-    if (!priv->module)
+    if (!priv->module) {
+        g_warning("can't load a shared object for test suite: %s: %s",
+                  priv->so_filename, g_module_error());
         return NULL;
+    }
 
     prefix = get_suite_prefix(priv);
 
@@ -631,7 +753,9 @@ cut_loader_load_test_suite (CutLoader *loader)
 
     g_free(prefix);
 
-    return cut_test_suite_new(NULL, warmup, cooldown);
+    test_suite = cut_test_suite_new(NULL, warmup, cooldown);
+    cut_test_set_base_directory(CUT_TEST(test_suite), priv->base_directory);
+    return test_suite;
 }
 
 /*
