@@ -41,6 +41,9 @@ struct _CutRepositoryPrivate
 
     gint deep;
     CutLoader *test_suite_loader;
+
+    gboolean keep_opening_modules;
+    gboolean enable_convenience_attribute_definition;
 };
 
 enum
@@ -94,6 +97,8 @@ cut_repository_init (CutRepository *repository)
     priv->exclude_dirs_regexs = NULL;
     priv->deep = 0;
     priv->test_suite_loader = NULL;
+    priv->keep_opening_modules = FALSE;
+    priv->enable_convenience_attribute_definition = FALSE;
 }
 
 static void
@@ -181,14 +186,42 @@ cut_repository_new (const gchar *directory)
                         NULL);
 }
 
+gboolean
+cut_repository_get_keep_opening_modules (CutRepository *repository)
+{
+    return CUT_REPOSITORY_GET_PRIVATE(repository)->keep_opening_modules;
+}
+
+void
+cut_repository_set_keep_opening_modules (CutRepository *repository,
+                                         gboolean keep_opening)
+{
+    CUT_REPOSITORY_GET_PRIVATE(repository)->keep_opening_modules = keep_opening;
+}
+
+gboolean
+cut_repository_get_enable_convenience_attribute_definition (CutRepository *repository)
+{
+    return CUT_REPOSITORY_GET_PRIVATE(repository)->enable_convenience_attribute_definition;
+}
+
+void
+cut_repository_set_enable_convenience_attribute_definition (CutRepository *repository,
+                                                            gboolean enable_convenience_attribute_definition)
+{
+    CUT_REPOSITORY_GET_PRIVATE(repository)->enable_convenience_attribute_definition = enable_convenience_attribute_definition;
+}
+
 static gboolean
 is_test_suite_so_path_name (const gchar *path_name)
 {
-    gboolean is_test_suite_so;
+    gboolean is_test_suite_so = FALSE;
     gchar *base_name;
 
     base_name = g_path_get_basename(path_name);
-    is_test_suite_so = g_str_has_prefix(base_name, "suite_");
+    if (g_str_has_prefix(base_name, "suite_") ||
+        g_str_has_prefix(base_name, "suite-"))
+        is_test_suite_so = TRUE;
     g_free(base_name);
     return is_test_suite_so;
 }
@@ -209,14 +242,46 @@ update_test_suite_loader (CutRepositoryPrivate *priv, CutLoader *loader,
     }
 }
 
+static inline gboolean
+is_ignore_directory (const gchar *dir_name)
+{
+    return g_str_equal(dir_name, ".svn") ||
+        g_str_equal(dir_name, ".git") ||
+        g_str_equal(dir_name, "CVS") ||
+        g_str_has_suffix(dir_name, ".dSYM");
+}
+
+static inline gchar *
+compute_relative_path (GArray *paths)
+{
+    gchar *last_component, *relative_path;
+    gboolean is_ignore_library_directory = FALSE;
+
+    last_component = g_array_index(paths, gchar *, paths->len - 1);
+    if (g_str_equal(last_component, ".libs") ||
+        g_str_equal(last_component, "_libs"))
+        is_ignore_library_directory = TRUE;
+
+    if (is_ignore_library_directory)
+        g_array_index(paths, gchar *, paths->len - 1) = NULL;
+    relative_path = g_build_filenamev((gchar **)(paths->data));
+    if (is_ignore_library_directory)
+        g_array_index(paths, gchar *, paths->len - 1) = last_component;
+
+    return relative_path;
+}
+
 static void
 cut_repository_collect_loader (CutRepository *repository, const gchar *dir_name,
-                               gint deep)
+                               GArray *paths)
 {
     GDir *dir;
     const gchar *entry;
     CutRepositoryPrivate *priv = CUT_REPOSITORY_GET_PRIVATE(repository);
 
+    if (is_ignore_directory(dir_name))
+        return;
+
     dir = g_dir_open(dir_name, 0, NULL);
     if (!dir)
         return;
@@ -230,9 +295,12 @@ cut_repository_collect_loader (CutRepository *repository, const gchar *dir_name,
                 g_free(path_name);
                 continue;
             }
-            cut_repository_collect_loader(repository, path_name, deep + 1);
+            g_array_append_val(paths, entry);
+            cut_repository_collect_loader(repository, path_name, paths);
+            g_array_remove_index(paths, paths->len - 1);
         } else {
             CutLoader *loader;
+            gchar *relative_path;
 
             if (cut_utils_filter_match(priv->exclude_files_regexs, entry) ||
                 !g_str_has_suffix(entry, "."G_MODULE_SUFFIX)) {
@@ -241,8 +309,13 @@ cut_repository_collect_loader (CutRepository *repository, const gchar *dir_name,
             }
 
             loader = cut_loader_new(path_name);
+            relative_path = compute_relative_path(paths);
+            cut_loader_set_base_directory(loader, relative_path);
+            cut_loader_set_keep_opening(loader, priv->keep_opening_modules);
+            cut_loader_set_enable_convenience_attribute_definition(loader,
+                                                                   priv->enable_convenience_attribute_definition);
             if (is_test_suite_so_path_name(path_name)) {
-                update_test_suite_loader(priv, loader, deep);
+                update_test_suite_loader(priv, loader, paths->len);
             } else {
                 priv->loaders = g_list_prepend(priv->loaders, loader);
             }
@@ -263,8 +336,12 @@ cut_repository_create_test_suite (CutRepository *repository)
         return NULL;
 
     if (!priv->loaders) {
+        GArray *paths;
+
         priv->deep = 0;
-        cut_repository_collect_loader(repository, priv->directory, priv->deep);
+        paths = g_array_new(TRUE, TRUE, sizeof(gchar *));
+        cut_repository_collect_loader(repository, priv->directory, paths);
+        g_array_free(paths, TRUE);
     }
 
     if (priv->test_suite_loader)
