@@ -26,6 +26,7 @@
 #include <glib.h>
 
 #include "cut-stream-parser.h"
+#include "cut-backtrace-entry.h"
 
 typedef enum {
     IN_TOP_LEVEL,
@@ -44,6 +45,7 @@ typedef enum {
     IN_TEST_NAME,
     IN_TEST_DESCRIPTION,
     IN_TEST_OPTION,
+    IN_TEST_START_TIME,
     IN_TEST_ELAPSED,
 
     IN_TEST_DATA_NAME,
@@ -86,12 +88,15 @@ typedef enum {
     IN_RESULT_BACKTRACE_ENTRY_FILE,
     IN_RESULT_BACKTRACE_ENTRY_LINE,
     IN_RESULT_BACKTRACE_ENTRY_INFO,
+    IN_RESULT_START_TIME,
     IN_RESULT_ELAPSED,
 
     IN_COMPLETE_ITERATED_TEST,
 
     IN_COMPLETE_TEST,
 
+    IN_TEST_ITERATOR_RESULT,
+
     IN_TEST_CASE_RESULT,
 
     IN_COMPLETE_TEST_ITERATOR,
@@ -100,6 +105,8 @@ typedef enum {
 
     IN_COMPLETE_TEST_SUITE,
 
+    IN_COMPLETE_SUCCESS,
+
     IN_CRASHED,
     IN_CRASHED_BACKTRACE
 } ParseState;
@@ -173,6 +180,13 @@ struct _CompleteTest
     CutTestContext *test_context;
 };
 
+typedef struct _TestIteratorResult TestIteratorResult;
+struct _TestIteratorResult
+{
+    CutTestIterator *test_iterator;
+    CutTestResult *result;
+};
+
 typedef struct _TestCaseResult TestCaseResult;
 struct _TestCaseResult
 {
@@ -204,12 +218,16 @@ struct _CutStreamParserPrivate
     TestResult *test_result;
     CompleteIteratedTest *complete_iterated_test;
     CompleteTest *complete_test;
+    TestIteratorResult *test_iterator_result;
     TestCaseResult *test_case_result;
 
     CutTestResult *result;
+    GList *backtrace;
+    CutBacktraceEntry *backtrace_entry;
     gchar *option_name;
     gchar *option_value;
-    gboolean success;
+    gboolean complete_success;
+    gboolean stream_success;
     gchar *crashed_backtrace;
 };
 
@@ -391,13 +409,17 @@ cut_stream_parser_init (CutStreamParser *stream_parser)
     priv->complete_iterated_test = NULL;
     priv->complete_test = NULL;
     priv->test_result = NULL;
+    priv->test_iterator_result = NULL;
     priv->test_case_result = NULL;
     priv->crashed_backtrace = NULL;
 
     priv->result = NULL;
+    priv->backtrace = NULL;
+    priv->backtrace_entry = NULL;
     priv->option_name = NULL;
     priv->option_value = NULL;
-    priv->success = TRUE;
+    priv->complete_success = TRUE;
+    priv->stream_success = TRUE;
 }
 
 static ReadyTestSuite *
@@ -540,6 +562,22 @@ complete_test_free (CompleteTest *complete_test)
     g_slice_free(CompleteTest, complete_test);
 }
 
+static TestIteratorResult *
+test_iterator_result_new (void)
+{
+    return g_slice_new0(TestIteratorResult);
+}
+
+static void
+test_iterator_result_free (TestIteratorResult *test_iterator_result)
+{
+    if (test_iterator_result->test_iterator)
+        g_object_unref(test_iterator_result->test_iterator);
+    if (test_iterator_result->result)
+        g_object_unref(test_iterator_result->result);
+    g_slice_free(TestIteratorResult, test_iterator_result);
+}
+
 static TestCaseResult *
 test_case_result_new (void)
 {
@@ -556,6 +594,16 @@ test_case_result_free (TestCaseResult *test_case_result)
     g_slice_free(TestCaseResult, test_case_result);
 }
 
+static void
+run_context_weak_notify (gpointer data, GObject *where_the_object_was)
+{
+    CutStreamParser *parser = data;
+    CutStreamParserPrivate *priv;
+
+    priv = CUT_STREAM_PARSER_GET_PRIVATE(parser);
+    priv->run_context = NULL;
+}
+
 static void
 dispose (GObject *object)
 {
@@ -567,11 +615,9 @@ dispose (GObject *object)
     }
 
     if (priv->run_context) {
-        CutRunContext *run_context;
-
-        run_context = priv->run_context;
+        g_object_weak_unref(G_OBJECT(priv->run_context),
+                            run_context_weak_notify, object);
         priv->run_context = NULL;
-        g_object_unref(run_context);
     }
 
     if (priv->states) {
@@ -665,6 +711,17 @@ dispose (GObject *object)
         priv->result = NULL;
     }
 
+    if (priv->backtrace) {
+        g_list_foreach(priv->backtrace, (GFunc)g_object_unref, NULL);
+        g_list_free(priv->backtrace);
+        priv->backtrace = NULL;
+    }
+
+    if (priv->backtrace_entry) {
+        g_object_unref(priv->backtrace_entry);
+        priv->backtrace_entry = NULL;
+    }
+
     if (priv->option_name) {
         g_free(priv->option_name);
         priv->option_name = NULL;
@@ -693,13 +750,10 @@ set_property (GObject      *object,
 
     switch (prop_id) {
       case PROP_RUN_CONTEXT:
-        if (priv->run_context) {
-            g_object_unref(priv->run_context);
-            priv->run_context = NULL;
-        }
         priv->run_context = g_value_get_object(value);
         if (priv->run_context)
-            g_object_ref(priv->run_context);
+            g_object_weak_ref(G_OBJECT(priv->run_context),
+                              run_context_weak_notify, object);
         break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
@@ -878,6 +932,9 @@ start_stream (CutStreamParserPrivate *priv,
     } else if (g_str_equal("complete-test", element_name)) {
         PUSH_STATE(priv, IN_COMPLETE_TEST);
         priv->complete_test = complete_test_new();
+    } else if (g_str_equal("test-iterator-result", element_name)) {
+        PUSH_STATE(priv, IN_TEST_ITERATOR_RESULT);
+        priv->test_iterator_result = test_iterator_result_new();
     } else if (g_str_equal("complete-test-iterator", element_name)) {
         PUSH_STATE(priv, IN_COMPLETE_TEST_ITERATOR);
     } else if (g_str_equal("test-case-result", element_name)) {
@@ -1030,6 +1087,10 @@ start_pass_assertion (CutStreamParserPrivate *priv,
         PUSH_STATE(priv, IN_TEST);
         priv->pass_assertion->test = cut_test_new_empty();
         PUSH_TEST(priv, priv->pass_assertion->test);
+    } else if (g_str_equal("iterated-test", element_name)) {
+        PUSH_STATE(priv, IN_ITERATED_TEST);
+        priv->pass_assertion->test = CUT_TEST(cut_iterated_test_new_empty());
+        PUSH_TEST(priv, priv->pass_assertion->test);
     } else if (g_str_equal("test-context", element_name)) {
         PUSH_STATE(priv, IN_TEST_CONTEXT);
         priv->pass_assertion->test_context = cut_test_context_new_empty();
@@ -1048,6 +1109,10 @@ start_test_result (CutStreamParserPrivate *priv,
         PUSH_STATE(priv, IN_TEST);
         priv->test_result->test = cut_test_new_empty();
         PUSH_TEST(priv, priv->test_result->test);
+    } else if (g_str_equal("iterated-test", element_name)) {
+        PUSH_STATE(priv, IN_ITERATED_TEST);
+        priv->test_result->test = CUT_TEST(cut_iterated_test_new_empty());
+        PUSH_TEST(priv, priv->test_result->test);
     } else if (g_str_equal("test-context", element_name)) {
         PUSH_STATE(priv, IN_TEST_CONTEXT);
         priv->test_result->test_context = cut_test_context_new_empty();
@@ -1055,7 +1120,7 @@ start_test_result (CutStreamParserPrivate *priv,
     } else if (g_str_equal("result", element_name)) {
         PUSH_STATE(priv, IN_RESULT);
         priv->test_result->result = cut_test_result_new_empty();
-        priv->result = priv->test_result->result;
+        priv->result = g_object_ref(priv->test_result->result);
     } else {
         invalid_element(context, error);
     }
@@ -1086,20 +1151,42 @@ start_result (CutStreamParserPrivate *priv, GMarkupParseContext *context,
         cut_test_result_set_test_case(priv->result, test_case);
         PUSH_TEST_CASE(priv, test_case);
         g_object_unref(test_case);
-    } else if (g_str_equal("test", element_name)) {
+    } else if (g_str_equal("test-iterator", element_name)) {
+        CutTestIterator *test_iterator;
+
+        PUSH_STATE(priv, IN_TEST_ITERATOR);
+        test_iterator = cut_test_iterator_new_empty();
+        cut_test_result_set_test_iterator(priv->result, test_iterator);
+        PUSH_TEST_ITERATOR(priv, test_iterator);
+        g_object_unref(test_iterator);
+    } else if (g_str_equal("test", element_name) ||
+               g_str_equal("iterated-test", element_name)) {
         CutTest *test;
 
         PUSH_STATE(priv, IN_TEST);
-        test = cut_test_new_empty();
+        if (g_str_equal("iterated-test", element_name))
+            test = CUT_TEST(cut_iterated_test_new_empty());
+        else
+            test = cut_test_new_empty();
         cut_test_result_set_test(priv->result, test);
         PUSH_TEST(priv, test);
         g_object_unref(test);
+    } else if (g_str_equal("test-data", element_name)) {
+        CutTestData *test_data;
+
+        PUSH_STATE(priv, IN_TEST_DATA);
+        test_data = cut_test_data_new_empty();
+        cut_test_result_set_test_data(priv->result, test_data);
+        PUSH_TEST_DATA(priv, test_data);
+        g_object_unref(test_data);
     } else if (g_str_equal("status", element_name)) {
         PUSH_STATE(priv, IN_RESULT_STATUS);
     } else if (g_str_equal("detail", element_name)) {
         PUSH_STATE(priv, IN_RESULT_DETAIL);
     } else if (g_str_equal("backtrace", element_name)) {
         PUSH_STATE(priv, IN_RESULT_BACKTRACE);
+    } else if (g_str_equal("start-time", element_name)) {
+        PUSH_STATE(priv, IN_RESULT_START_TIME);
     } else if (g_str_equal("elapsed", element_name)) {
         PUSH_STATE(priv, IN_RESULT_ELAPSED);
     } else {
@@ -1122,6 +1209,8 @@ start_complete_iterated_test (CutStreamParserPrivate *priv,
         priv->complete_iterated_test->test_context =
             cut_test_context_new_empty();
         PUSH_TEST_CONTEXT(priv, priv->complete_iterated_test->test_context);
+    } else if (g_str_equal("success", element_name)) {
+        PUSH_STATE(priv, IN_COMPLETE_SUCCESS);
     } else {
         invalid_element(context, error);
     }
@@ -1139,6 +1228,27 @@ start_complete_test (CutStreamParserPrivate *priv, GMarkupParseContext *context,
         PUSH_STATE(priv, IN_TEST_CONTEXT);
         priv->complete_test->test_context = cut_test_context_new_empty();
         PUSH_TEST_CONTEXT(priv, priv->complete_test->test_context);
+    } else if (g_str_equal("success", element_name)) {
+        PUSH_STATE(priv, IN_COMPLETE_SUCCESS);
+    } else {
+        invalid_element(context, error);
+    }
+}
+
+static void
+start_test_iterator_result (CutStreamParserPrivate *priv,
+                        GMarkupParseContext *context,
+                        const gchar *element_name, GError **error)
+{
+    if (g_str_equal("test-iterator", element_name)) {
+        PUSH_STATE(priv, IN_TEST_ITERATOR);
+        priv->test_iterator_result->test_iterator =
+            cut_test_iterator_new_empty();
+        PUSH_TEST_ITERATOR(priv, priv->test_iterator_result->test_iterator);
+    } else if (g_str_equal("result", element_name)) {
+        PUSH_STATE(priv, IN_RESULT);
+        priv->test_iterator_result->result = cut_test_result_new_empty();
+        priv->result = g_object_ref(priv->test_iterator_result->result);
     } else {
         invalid_element(context, error);
     }
@@ -1152,6 +1262,8 @@ start_complete_test_iterator (CutStreamParserPrivate *priv,
     if (g_str_equal("test-iterator", element_name)) {
         PUSH_STATE(priv, IN_TEST_ITERATOR);
         PUSH_TEST_ITERATOR(priv, cut_test_iterator_new_empty());
+    } else if (g_str_equal("success", element_name)) {
+        PUSH_STATE(priv, IN_COMPLETE_SUCCESS);
     } else {
         invalid_element(context, error);
     }
@@ -1169,7 +1281,7 @@ start_test_case_result (CutStreamParserPrivate *priv,
     } else if (g_str_equal("result", element_name)) {
         PUSH_STATE(priv, IN_RESULT);
         priv->test_case_result->result = cut_test_result_new_empty();
-        priv->result = priv->test_case_result->result;
+        priv->result = g_object_ref(priv->test_case_result->result);
     } else {
         invalid_element(context, error);
     }
@@ -1183,6 +1295,8 @@ start_complete_test_case (CutStreamParserPrivate *priv,
     if (g_str_equal("test-case", element_name)) {
         PUSH_STATE(priv, IN_TEST_CASE);
         PUSH_TEST_CASE(priv, cut_test_case_new_empty());
+    } else if (g_str_equal("success", element_name)) {
+        PUSH_STATE(priv, IN_COMPLETE_SUCCESS);
     } else {
         invalid_element(context, error);
     }
@@ -1196,6 +1310,8 @@ start_complete_test_suite (CutStreamParserPrivate *priv,
     if (g_str_equal("test-suite", element_name)) {
         PUSH_STATE(priv, IN_TEST_SUITE);
         PUSH_TEST_SUITE(priv, cut_test_suite_new_empty());
+    } else if (g_str_equal("success", element_name)) {
+        PUSH_STATE(priv, IN_COMPLETE_SUCCESS);
     } else {
         invalid_element(context, error);
     }
@@ -1222,6 +1338,8 @@ start_test_object (CutStreamParserPrivate *priv, GMarkupParseContext *context,
         PUSH_STATE(priv, IN_TEST_DESCRIPTION);
     } else if (g_str_equal("option", element_name)) {
         PUSH_STATE(priv, IN_TEST_OPTION);
+    } else if (g_str_equal("start-time", element_name)) {
+        PUSH_STATE(priv, IN_TEST_START_TIME);
     } else if (g_str_equal("elapsed", element_name)) {
         PUSH_STATE(priv, IN_TEST_ELAPSED);
     } else {
@@ -1247,11 +1365,12 @@ start_result_backtrace (CutStreamParserPrivate *priv,
                         GMarkupParseContext *context,
                         const gchar *element_name, GError **error)
 {
-        if (g_str_equal("entry", element_name)) {
-            PUSH_STATE(priv, IN_RESULT_BACKTRACE_ENTRY);
-        } else {
-            invalid_element(context, error);
-        }
+    if (g_str_equal("entry", element_name)) {
+        PUSH_STATE(priv, IN_RESULT_BACKTRACE_ENTRY);
+        priv->backtrace_entry = cut_backtrace_entry_new_empty();
+    } else {
+        invalid_element(context, error);
+    }
 }
 
 static void
@@ -1406,6 +1525,9 @@ start_element_handler (GMarkupParseContext *context,
       case IN_COMPLETE_TEST:
         start_complete_test(priv, context, element_name, error);
         break;
+      case IN_TEST_ITERATOR_RESULT:
+        start_test_iterator_result(priv, context, element_name, error);
+        break;
       case IN_COMPLETE_TEST_ITERATOR:
         start_complete_test_iterator(priv, context, element_name, error);
         break;
@@ -1466,8 +1588,8 @@ end_stream (CutStreamParser *parser, CutStreamParserPrivate *priv,
             const gchar *element_name, GError **error)
 {
     if (priv->run_context)
-        g_signal_emit_by_name(priv->run_context, "complete-run", priv->success);
-
+        cut_run_context_emit_complete_run(priv->run_context,
+                                          priv->stream_success);
 }
 
 static void
@@ -1526,6 +1648,36 @@ end_test_option (CutStreamParser *parser, CutStreamParserPrivate *priv,
     }
 }
 
+static void
+end_result_backtrace (CutStreamParser *parser, CutStreamParserPrivate *priv,
+                      GMarkupParseContext *context,
+                      const gchar *element_name, GError **error)
+{
+    if (!priv->backtrace)
+        return;
+
+    if (priv->result)
+        cut_test_result_set_backtrace(priv->result,
+                                      g_list_reverse(priv->backtrace));
+
+    g_list_foreach(priv->backtrace, (GFunc)g_object_unref, NULL);
+    g_list_free(priv->backtrace);
+    priv->backtrace = NULL;
+}
+
+static void
+end_result_backtrace_entry (CutStreamParser *parser,
+                            CutStreamParserPrivate *priv,
+                            GMarkupParseContext *context,
+                            const gchar *element_name, GError **error)
+{
+    if (!priv->backtrace_entry)
+        return; /* should check file name, line, info? */
+
+    priv->backtrace = g_list_prepend(priv->backtrace, priv->backtrace_entry);
+    priv->backtrace_entry = NULL;
+}
+
 static void
 end_ready_test_suite (CutStreamParser *parser, CutStreamParserPrivate *priv,
                       GMarkupParseContext *context,
@@ -1535,8 +1687,6 @@ end_ready_test_suite (CutStreamParser *parser, CutStreamParserPrivate *priv,
         return;
 
     if (priv->run_context) {
-        cut_run_context_prepare_test_suite(priv->run_context,
-                                           priv->ready_test_suite->test_suite);
         g_signal_emit_by_name(priv->run_context, "ready-test-suite",
                               priv->ready_test_suite->test_suite,
                               priv->ready_test_suite->n_test_cases,
@@ -1574,8 +1724,6 @@ end_ready_test_case (CutStreamParser *parser, CutStreamParserPrivate *priv,
         return;
 
     if (priv->run_context) {
-        cut_run_context_prepare_test_case(priv->run_context,
-                                          priv->ready_test_case->test_case);
         g_signal_emit_by_name(priv->run_context, "ready-test-case",
                               priv->ready_test_case->test_case,
                               priv->ready_test_case->n_tests);
@@ -1613,9 +1761,6 @@ end_ready_test_iterator (CutStreamParser *parser, CutStreamParserPrivate *priv,
         return;
 
     if (priv->run_context) {
-        cut_run_context_prepare_test_iterator(
-            priv->run_context,
-            priv->ready_test_iterator->test_iterator);
         g_signal_emit_by_name(priv->run_context, "ready-test-iterator",
                               priv->ready_test_iterator->test_iterator,
                               priv->ready_test_iterator->n_tests);
@@ -1655,7 +1800,6 @@ end_start_test (CutStreamParser *parser, CutStreamParserPrivate *priv,
         return;
 
     if (priv->run_context) {
-        cut_run_context_prepare_test(priv->run_context, priv->start_test->test);
         g_signal_emit_by_name(priv->run_context, "start-test",
                               priv->start_test->test,
                               priv->start_test->test_context);
@@ -1678,12 +1822,8 @@ end_start_iterated_test (CutStreamParser *parser, CutStreamParserPrivate *priv,
         return;
 
     if (priv->run_context) {
-        CutIteratedTest *iterated_test;
-
-        iterated_test = priv->start_iterated_test->iterated_test;
-        cut_run_context_prepare_iterated_test(priv->run_context, iterated_test);
         g_signal_emit_by_name(priv->run_context, "start-iterated-test",
-                              iterated_test,
+                              priv->start_iterated_test->iterated_test,
                               priv->start_iterated_test->test_context);
     }
 
@@ -1762,7 +1902,8 @@ end_complete_iterated_test (CutStreamParser *parser,
     if (priv->run_context)
         g_signal_emit_by_name(priv->run_context, "complete-iterated-test",
                               priv->complete_iterated_test->iterated_test,
-                              priv->complete_iterated_test->test_context);
+                              priv->complete_iterated_test->test_context,
+                              priv->complete_success);
 
     if (priv->complete_iterated_test->iterated_test)
         DROP_TEST(priv);
@@ -1783,7 +1924,8 @@ end_complete_test (CutStreamParser *parser, CutStreamParserPrivate *priv,
     if (priv->run_context)
         g_signal_emit_by_name(priv->run_context, "complete-test",
                               priv->complete_test->test,
-                              priv->complete_test->test_context);
+                              priv->complete_test->test_context,
+                              priv->complete_success);
 
     if (priv->complete_test->test)
         DROP_TEST(priv);
@@ -1791,6 +1933,39 @@ end_complete_test (CutStreamParser *parser, CutStreamParserPrivate *priv,
         DROP_TEST_CONTEXT(priv);
     complete_test_free(priv->complete_test);
     priv->complete_test = NULL;
+    priv->complete_success = TRUE;
+}
+
+static void
+end_test_iterator_result (CutStreamParser *parser, CutStreamParserPrivate *priv,
+                          GMarkupParseContext *context,
+                          const gchar *element_name, GError **error)
+{
+    if (!priv->test_iterator_result)
+        return;
+
+    if (priv->run_context) {
+        CutTestResult *result;
+        CutTestResultStatus status;
+        const gchar *signal_name;
+        gchar *full_signal_name;
+
+        result = priv->test_iterator_result->result;
+        status = cut_test_result_get_status(result);
+        signal_name = cut_test_result_status_to_signal_name(status);
+        full_signal_name = g_strdup_printf("%s-test-iterator", signal_name);
+        g_signal_emit_by_name(priv->run_context, full_signal_name,
+                              priv->test_iterator_result->test_iterator,
+                              priv->test_iterator_result->result);
+        g_free(full_signal_name);
+    }
+
+    if (priv->test_iterator_result->test_iterator)
+        DROP_TEST_ITERATOR(priv);
+    test_iterator_result_free(priv->test_iterator_result);
+    priv->test_iterator_result = NULL;
+    g_object_unref(priv->result);
+    priv->result = NULL;
 }
 
 static void
@@ -1807,8 +1982,10 @@ end_complete_test_iterator (CutStreamParser *parser,
 
     if (priv->run_context)
         g_signal_emit_by_name(priv->run_context,
-                              "complete-test-iterator", test_iterator);
+                              "complete-test-iterator", test_iterator,
+                              priv->complete_success);
     g_object_unref(test_iterator);
+    priv->complete_success = TRUE;
 }
 
 static void
@@ -1839,6 +2016,7 @@ end_test_case_result (CutStreamParser *parser, CutStreamParserPrivate *priv,
         DROP_TEST_CASE(priv);
     test_case_result_free(priv->test_case_result);
     priv->test_case_result = NULL;
+    g_object_unref(priv->result);
     priv->result = NULL;
 }
 
@@ -1855,8 +2033,10 @@ end_complete_test_case (CutStreamParser *parser, CutStreamParserPrivate *priv,
 
     if (priv->run_context)
         g_signal_emit_by_name(priv->run_context,
-                              "complete-test-case", test_case);
+                              "complete-test-case", test_case,
+                              priv->complete_success);
     g_object_unref(test_case);
+    priv->complete_success = TRUE;
 }
 
 static void
@@ -1872,8 +2052,10 @@ end_complete_test_suite (CutStreamParser *parser, CutStreamParserPrivate *priv,
 
     if (priv->run_context)
         g_signal_emit_by_name(priv->run_context,
-                              "complete-test-suite", test_suite);
+                              "complete-test-suite", test_suite,
+                              priv->complete_success);
     g_object_unref(test_suite);
+    priv->complete_success = TRUE;
 }
 
 static void
@@ -1923,6 +2105,12 @@ end_element_handler (GMarkupParseContext *context,
       case IN_TEST_OPTION:
         end_test_option(parser, priv, context, element_name, error);
         break;
+      case IN_RESULT_BACKTRACE:
+        end_result_backtrace(parser, priv, context, element_name, error);
+        break;
+      case IN_RESULT_BACKTRACE_ENTRY:
+        end_result_backtrace_entry(parser, priv, context, element_name, error);
+        break;
       case IN_READY_TEST_SUITE:
         end_ready_test_suite(parser, priv, context, element_name, error);
         break;
@@ -1959,6 +2147,9 @@ end_element_handler (GMarkupParseContext *context,
       case IN_COMPLETE_TEST:
         end_complete_test(parser, priv, context, element_name, error);
         break;
+      case IN_TEST_ITERATOR_RESULT:
+        end_test_iterator_result(parser, priv, context, element_name, error);
+        break;
       case IN_COMPLETE_TEST_ITERATOR:
         end_complete_test_iterator(parser, priv, context, element_name, error);
         break;
@@ -2109,6 +2300,29 @@ text_test_option_value (CutStreamParserPrivate *priv,
     }
 }
 
+static void
+text_test_start_time (CutStreamParserPrivate *priv,
+                      GMarkupParseContext *context,
+                      const gchar *text, gsize text_len, GError **error)
+{
+    CutTest *target;
+
+    target = target_test_object(priv, PEEK_NTH_STATE(priv, 1));
+    if (target) {
+        GTimeVal start_time;
+
+        if (g_time_val_from_iso8601(text, &start_time)) {
+            cut_test_set_start_time(target, &start_time);
+        } else {
+            set_parse_error(context, error,
+                            "invalid start-time value (not ISO 8601 format): %s",
+                            text);
+        }
+    } else {
+        set_parse_error(context, error, "can't find test start time target");
+    }
+}
+
 static void
 text_test_elapsed (CutStreamParserPrivate *priv,
                    GMarkupParseContext *context,
@@ -2159,7 +2373,7 @@ text_result_backtrace_entry_file (CutStreamParserPrivate *priv,
                                   const gchar *text, gsize text_len,
                                   GError **error)
 {
-    cut_test_result_set_filename(priv->result, text);
+    cut_backtrace_entry_set_file(priv->backtrace_entry, text);
 }
 
 static void
@@ -2169,7 +2383,7 @@ text_result_backtrace_entry_line (CutStreamParserPrivate *priv,
                                   GError **error)
 {
     if (is_integer(text)) {
-        cut_test_result_set_line(priv->result, atoi(text));
+        cut_backtrace_entry_set_line(priv->backtrace_entry, atoi(text));
     } else {
         set_parse_error(context, error, "invalid line number: %s", text);
     }
@@ -2182,13 +2396,45 @@ text_result_backtrace_entry_info (CutStreamParserPrivate *priv,
                                   GError **error)
 {
     if (g_str_has_suffix(text, "()")) {
-        gchar *function_name;
+        gchar *function;
 
-        function_name = g_strndup(text, strlen(text) - 2);
-        cut_test_result_set_function_name(priv->result, function_name);
-        g_free(function_name);
+        function = g_strndup(text, strlen(text) - 2);
+        cut_backtrace_entry_set_function(priv->backtrace_entry, function);
+        g_free(function);
     } else {
-        set_parse_error(context, error, "invalid function name: %s", text);
+        const gchar *info_start;
+
+        info_start = strstr(text, "():");
+        if (info_start) {
+            gchar *function;
+
+            function = g_strndup(text, info_start - text);
+            info_start += strlen("():");
+            while (info_start[0] && info_start[0] == ' ')
+                info_start++;
+            cut_backtrace_entry_set_function(priv->backtrace_entry, function);
+            cut_backtrace_entry_set_info(priv->backtrace_entry, info_start);
+            g_free(function);
+        } else {
+            cut_backtrace_entry_set_info(priv->backtrace_entry, text);
+        }
+    }
+}
+
+
+static void
+text_result_start_time (CutStreamParserPrivate *priv,
+                        GMarkupParseContext *context,
+                        const gchar *text, gsize text_len, GError **error)
+{
+    GTimeVal start_time;
+
+    if (g_time_val_from_iso8601(text, &start_time)) {
+        cut_test_result_set_start_time(priv->result, &start_time);
+    } else {
+        set_parse_error(context, error,
+                        "invalid start-time value (not ISO 8601 format): %s",
+                        text);
     }
 }
 
@@ -2282,12 +2528,24 @@ text_test_context_failed (CutStreamParserPrivate *priv,
     }
 }
 
+static void
+text_complete_success (CutStreamParserPrivate *priv,
+                       GMarkupParseContext *context,
+                       const gchar *text, gsize text_len, GError **error)
+{
+    if (is_boolean(text)) {
+        priv->complete_success = string_to_boolean(text);
+    } else {
+        set_parse_error(context, error, "invalid boolean value: %s", text);
+    }
+}
+
 static void
 text_stream_success (CutStreamParserPrivate *priv, GMarkupParseContext *context,
                      const gchar *text, gsize text_len, GError **error)
 {
     if (is_boolean(text)) {
-        priv->success = string_to_boolean(text);
+        priv->stream_success = string_to_boolean(text);
     } else {
         set_parse_error(context, error, "invalid boolean value: %s", text);
     }
@@ -2330,6 +2588,9 @@ text_handler (GMarkupParseContext *context,
       case IN_TEST_OPTION_VALUE:
         text_test_option_value(priv, context, text, text_len, error);
         break;
+      case IN_TEST_START_TIME:
+        text_test_start_time(priv, context, text, text_len, error);
+        break;
       case IN_TEST_ELAPSED:
         text_test_elapsed(priv, context, text, text_len, error);
         break;
@@ -2348,6 +2609,9 @@ text_handler (GMarkupParseContext *context,
       case IN_RESULT_BACKTRACE_ENTRY_INFO:
         text_result_backtrace_entry_info(priv, context, text, text_len, error);
         break;
+      case IN_RESULT_START_TIME:
+        text_result_start_time(priv, context, text, text_len, error);
+        break;
       case IN_RESULT_ELAPSED:
         text_result_elapsed(priv, context, text, text_len, error);
         break;
@@ -2369,6 +2633,8 @@ text_handler (GMarkupParseContext *context,
       case IN_TEST_CONTEXT_FAILED:
         text_test_context_failed(priv, context, text, text_len, error);
         break;
+      case IN_COMPLETE_SUCCESS:
+        text_complete_success(priv, context, text, text_len, error);
       case IN_STREAM_SUCCESS:
         text_stream_success(priv, context, text, text_len, error);
         break;
