@@ -40,8 +40,6 @@ struct _CutTestCasePrivate
 {
     CutSetupFunction setup;
     CutTeardownFunction teardown;
-    CutGetCurrentTestContextFunction get_current_test_context;
-    CutSetCurrentTestContextFunction set_current_test_context;
     CutStartupFunction startup;
     CutShutdownFunction shutdown;
 };
@@ -51,17 +49,17 @@ enum
     PROP_0,
     PROP_SETUP_FUNCTION,
     PROP_TEARDOWN_FUNCTION,
-    PROP_GET_CURRENT_TEST_CONTEXT_FUNCTION,
-    PROP_SET_CURRENT_TEST_CONTEXT_FUNCTION,
     PROP_STARTUP_FUNCTION,
     PROP_SHUTDOWN_FUNCTION
 };
 
 enum
 {
-    READY_SIGNAL,
-    START_TEST_SIGNAL,
-    COMPLETE_TEST_SIGNAL,
+    READY,
+    START_TEST,
+    COMPLETE_TEST,
+    START_TEST_ITERATOR,
+    COMPLETE_TEST_ITERATOR,
     LAST_SIGNAL
 };
 
@@ -105,22 +103,6 @@ cut_test_case_class_init (CutTestCaseClass *klass)
                                 G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
     g_object_class_install_property(gobject_class, PROP_TEARDOWN_FUNCTION, spec);
 
-    spec = g_param_spec_pointer("get-current-test-context-function",
-                                "Get current test context function",
-                                "The function for getting current test context",
-                                G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
-    g_object_class_install_property(gobject_class,
-                                    PROP_GET_CURRENT_TEST_CONTEXT_FUNCTION,
-                                    spec);
-
-    spec = g_param_spec_pointer("set-current-test-context-function",
-                                "Set current test context function",
-                                "The function for setting current test context",
-                                G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
-    g_object_class_install_property(gobject_class,
-                                    PROP_SET_CURRENT_TEST_CONTEXT_FUNCTION,
-                                    spec);
-
     spec = g_param_spec_pointer("startup-function",
                                 "Startup Function",
                                 "The function for initialization of TestCase",
@@ -138,7 +120,7 @@ cut_test_case_class_init (CutTestCaseClass *klass)
                                     spec);
 
 
-    cut_test_case_signals[READY_SIGNAL]
+    cut_test_case_signals[READY]
         = g_signal_new("ready",
                        G_TYPE_FROM_CLASS(klass),
                        G_SIGNAL_RUN_LAST,
@@ -147,7 +129,7 @@ cut_test_case_class_init (CutTestCaseClass *klass)
                        g_cclosure_marshal_VOID__UINT,
                        G_TYPE_NONE, 1, G_TYPE_UINT);
 
-    cut_test_case_signals[START_TEST_SIGNAL]
+    cut_test_case_signals[START_TEST]
         = g_signal_new("start-test",
                 G_TYPE_FROM_CLASS(klass),
                 G_SIGNAL_RUN_LAST,
@@ -156,14 +138,36 @@ cut_test_case_class_init (CutTestCaseClass *klass)
                 _cut_marshal_VOID__OBJECT_OBJECT,
                 G_TYPE_NONE, 2, CUT_TYPE_TEST, CUT_TYPE_TEST_CONTEXT);
 
-    cut_test_case_signals[COMPLETE_TEST_SIGNAL]
+    cut_test_case_signals[COMPLETE_TEST]
         = g_signal_new("complete-test",
-                G_TYPE_FROM_CLASS(klass),
-                G_SIGNAL_RUN_LAST,
-                G_STRUCT_OFFSET(CutTestCaseClass, complete_test),
-                NULL, NULL,
-                _cut_marshal_VOID__OBJECT_OBJECT,
-                G_TYPE_NONE, 2, CUT_TYPE_TEST, CUT_TYPE_TEST_CONTEXT);
+                       G_TYPE_FROM_CLASS(klass),
+                       G_SIGNAL_RUN_LAST,
+                       G_STRUCT_OFFSET(CutTestCaseClass, complete_test),
+                       NULL, NULL,
+                       _cut_marshal_VOID__OBJECT_OBJECT_BOOLEAN,
+                       G_TYPE_NONE, 3,
+                       CUT_TYPE_TEST, CUT_TYPE_TEST_CONTEXT, G_TYPE_BOOLEAN);
+
+    cut_test_case_signals[START_TEST_ITERATOR]
+        = g_signal_new("start-test-iterator",
+                       G_TYPE_FROM_CLASS(klass),
+                       G_SIGNAL_RUN_LAST,
+                       G_STRUCT_OFFSET(CutTestCaseClass, start_test_iterator),
+                       NULL, NULL,
+                       _cut_marshal_VOID__OBJECT_OBJECT,
+                       G_TYPE_NONE,
+                       2, CUT_TYPE_TEST_ITERATOR, CUT_TYPE_TEST_CONTEXT);
+
+    cut_test_case_signals[COMPLETE_TEST_ITERATOR]
+        = g_signal_new("complete-test-iterator",
+                       G_TYPE_FROM_CLASS(klass),
+                       G_SIGNAL_RUN_LAST,
+                       G_STRUCT_OFFSET(CutTestCaseClass, complete_test_iterator),
+                       NULL, NULL,
+                       _cut_marshal_VOID__OBJECT_OBJECT_BOOLEAN,
+                       G_TYPE_NONE,
+                       3, CUT_TYPE_TEST_ITERATOR, CUT_TYPE_TEST_CONTEXT,
+                       G_TYPE_BOOLEAN);
 
     g_type_class_add_private(gobject_class, sizeof(CutTestCasePrivate));
 }
@@ -175,8 +179,6 @@ cut_test_case_init (CutTestCase *test_case)
 
     priv->setup = NULL;
     priv->teardown = NULL;
-    priv->get_current_test_context = NULL;
-    priv->set_current_test_context = NULL;
     priv->startup = NULL;
     priv->shutdown = NULL;
 }
@@ -202,12 +204,6 @@ set_property (GObject      *object,
       case PROP_TEARDOWN_FUNCTION:
         priv->teardown = g_value_get_pointer(value);
         break;
-      case PROP_GET_CURRENT_TEST_CONTEXT_FUNCTION:
-        priv->get_current_test_context = g_value_get_pointer(value);
-        break;
-      case PROP_SET_CURRENT_TEST_CONTEXT_FUNCTION:
-        priv->set_current_test_context = g_value_get_pointer(value);
-        break;
       case PROP_STARTUP_FUNCTION:
         priv->startup = g_value_get_pointer(value);
         break;
@@ -251,8 +247,6 @@ CutTestCase *
 cut_test_case_new (const gchar *name,
                    CutSetupFunction setup,
                    CutTeardownFunction teardown,
-                   CutGetCurrentTestContextFunction get_current_test_context,
-                   CutSetCurrentTestContextFunction set_current_test_context,
                    CutStartupFunction startup,
                    CutShutdownFunction shutdown)
 {
@@ -261,10 +255,6 @@ cut_test_case_new (const gchar *name,
                         "element-name", "test-case",
                         "setup-function", setup,
                         "teardown-function", teardown,
-                        "get-current-test-context-function",
-                        get_current_test_context,
-                        "set-current-test-context-function",
-                        set_current_test_context,
                         "startup-function", startup,
                         "shutdown-function", shutdown,
                         NULL);
@@ -274,7 +264,6 @@ CutTestCase *
 cut_test_case_new_empty (void)
 {
     return cut_test_case_new(NULL,
-                             NULL, NULL,
                              NULL, NULL,
                              NULL, NULL);
 }
@@ -292,19 +281,6 @@ get_filtered_tests (CutTestCase *test_case, const gchar **test_names)
     }
 }
 
-guint
-cut_test_case_get_n_tests (CutTestCase *test_case,  const gchar **test_names)
-{
-    GList *filtered_tests;
-    guint n_tests;
-
-    filtered_tests = get_filtered_tests(test_case, test_names);
-    n_tests = g_list_length(filtered_tests);
-    g_list_free(filtered_tests);
-
-    return n_tests;
-}
-
 void
 cut_test_case_add_test (CutTestCase *test_case, CutTest *test)
 {
@@ -343,10 +319,41 @@ cut_test_case_run_teardown (CutTestCase *test_case, CutTestContext *test_context
     }
 }
 
+static gboolean
+run_test (CutTestCase *test_case, CutTest *test,
+          CutTestContext *test_context, CutRunContext *run_context)
+{
+    gboolean success = TRUE;
+
+    if (CUT_IS_TEST_ITERATOR(test)) {
+        g_signal_emit_by_name(test_case, "start-test-iterator",
+                              test, test_context);
+    } else {
+        g_signal_emit_by_name(test_case, "start-test", test, test_context);
+        cut_test_case_run_setup(test_case, test_context);
+    }
+
+    if (cut_test_context_is_failed(test_context)) {
+        success = FALSE;
+    } else {
+        success = cut_test_run(test, test_context, run_context);
+    }
+
+    if (CUT_IS_TEST_ITERATOR(test)) {
+        g_signal_emit_by_name(test_case, "complete-test-iterator",
+                              test, test_context, success);
+    } else {
+        cut_test_case_run_teardown(test_case, test_context);
+        g_signal_emit_by_name(test_case, "complete-test",
+                              test, test_context, success);
+    }
+
+    return success;
+}
+
 static gboolean
 run (CutTestCase *test_case, CutTest *test, CutRunContext *run_context)
 {
-    CutTestCasePrivate *priv;
     CutTestContext *original_test_context, *test_context;
     gboolean success = TRUE;
     gboolean is_multi_thread;
@@ -356,38 +363,17 @@ run (CutTestCase *test_case, CutTest *test, CutRunContext *run_context)
 
     is_multi_thread = cut_run_context_is_multi_thread(run_context);
 
-    priv = CUT_TEST_CASE_GET_PRIVATE(test_case);
-    if (!priv->get_current_test_context ||
-        !priv->set_current_test_context) {
-        g_warning("You should include <cutter.h>");
-        return FALSE;
-    }
-
-    test_context = cut_test_context_new(NULL, test_case, NULL, NULL);
+    test_context = cut_test_context_new(run_context,
+                                        NULL, test_case, NULL, NULL);
     cut_test_context_set_multi_thread(test_context, is_multi_thread);
-    original_test_context = cut_test_case_get_current_test_context(test_case);
-    cut_test_case_set_current_test_context(test_case, test_context);
+    original_test_context = cut_test_context_current_get();
+    cut_test_context_current_set(test_context);
 
+    cut_test_context_set_test(test_context, test);
+    success = run_test(test_case, test, test_context, run_context);
+    cut_test_context_set_test(test_context, NULL);
 
-    if (!CUT_IS_TEST_CONTAINER(test)) {
-        g_signal_emit_by_name(test_case, "start-test", test, test_context);
-        cut_test_case_run_setup(test_case, test_context);
-    }
-
-    if (cut_test_context_is_failed(test_context)) {
-        success = FALSE;
-    } else {
-        cut_test_context_set_test(test_context, test);
-        success = cut_test_run(test, test_context, run_context);
-        cut_test_context_set_test(test_context, NULL);
-    }
-
-    if (!CUT_IS_TEST_CONTAINER(test)) {
-        cut_test_case_run_teardown(test_case, test_context);
-        g_signal_emit_by_name(test_case, "complete-test", test, test_context);
-    }
-
-    cut_test_case_set_current_test_context(test_case, original_test_context);
+    cut_test_context_current_set(original_test_context);
     g_object_unref(test_context);
 
     return success;
@@ -412,9 +398,8 @@ cut_test_case_run_tests (CutTestCase *test_case, CutRunContext *run_context,
     CutTestResult *result;
     gboolean all_success = TRUE;
 
-    cut_run_context_prepare_test_case(run_context, test_case);
     g_signal_emit_by_name(test_case, "ready", g_list_length((GList *)tests));
-    g_signal_emit_by_name(CUT_TEST(test_case), "start");
+    g_signal_emit_by_name(CUT_TEST(test_case), "start", NULL);
 
     priv = CUT_TEST_CASE_GET_PRIVATE(test_case);
 
@@ -453,17 +438,14 @@ cut_test_case_run_tests (CutTestCase *test_case, CutRunContext *run_context,
 
     result = cut_test_result_new(status,
                                  NULL, NULL, test_case, NULL, NULL,
-                                 NULL, NULL,
-                                 NULL, NULL, 0);
-    g_signal_emit_by_name(CUT_TEST(test_case),
-                          cut_test_result_status_to_signal_name(status),
-                          NULL, result);
+                                 NULL, NULL, NULL);
+    cut_test_emit_result_signal(CUT_TEST(test_case), NULL, result);
     g_object_unref(result);
 
     if (priv->shutdown) {
         priv->shutdown();
     }
-    g_signal_emit_by_name(CUT_TEST(test_case), "complete");
+    g_signal_emit_by_name(CUT_TEST(test_case), "complete", NULL, all_success);
 
     return all_success;
 }
@@ -480,8 +462,8 @@ cut_test_case_run_test (CutTestCase *test_case, CutRunContext *run_context, cons
 }
 
 gboolean
-cut_test_case_run_with_filter (CutTestCase  *test_case,
-                               CutRunContext   *run_context,
+cut_test_case_run_with_filter (CutTestCase *test_case,
+                               CutRunContext *run_context,
                                const gchar **test_names)
 {
     GList *filtered_tests;
@@ -498,27 +480,10 @@ cut_test_case_run_with_filter (CutTestCase  *test_case,
 gboolean
 cut_test_case_run (CutTestCase *test_case, CutRunContext *run_context)
 {
-    gchar **test_names;
+    const gchar **test_names;
 
     test_names = cut_run_context_get_target_test_names(run_context);
-    return cut_test_case_run_with_filter(test_case, run_context,
-                                         (const gchar **)test_names);
-}
-
-CutTestContext *
-cut_test_case_get_current_test_context (CutTestCase *test_case)
-{
-    return CUT_TEST_CASE_GET_PRIVATE(test_case)->get_current_test_context();
-}
-
-void
-cut_test_case_set_current_test_context (CutTestCase *test_case,
-                                        CutTestContext *test_context)
-{
-    CutTestCasePrivate *priv;
-
-    priv = CUT_TEST_CASE_GET_PRIVATE(test_case);
-    priv->set_current_test_context(test_context);
+    return cut_test_case_run_with_filter(test_case, run_context, test_names);
 }
 
 /*
