@@ -27,7 +27,8 @@
 
 #include "cut-test-container.h"
 
-#include "cut-test.h"
+#include "cut-iterated-test.h"
+#include "cut-run-context.h"
 #include "cut-utils.h"
 
 #define CUT_TEST_CONTAINER_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CUT_TYPE_TEST_CONTAINER, CutTestContainerPrivate))
@@ -93,7 +94,7 @@ cut_test_container_add_test (CutTestContainer *container, CutTest *test)
 
     if (CUT_IS_TEST(test)) {
         g_object_ref(test);
-        priv->tests = g_list_prepend(priv->tests, test);
+        priv->tests = g_list_append(priv->tests, test);
     }
 }
 
@@ -153,8 +154,11 @@ cut_test_container_filter_children (CutTestContainer *container,
     for (node = original; node; node = g_list_next(node)) {
         CutTest *test = node->data;
 
-        if (cut_utils_filter_match(regexs, cut_test_get_name(test)))
-            matched_tests = g_list_prepend(matched_tests, test);
+        if (CUT_IS_TEST_ITERATOR(test) ||
+            cut_utils_filter_match(regexs, cut_test_get_name(test)) ||
+            cut_utils_filter_match(regexs, cut_test_get_full_name(test))) {
+            matched_tests = g_list_append(matched_tests, test);
+        }
     }
 
     g_list_foreach(regexs, (GFunc)g_regex_unref, NULL);
@@ -163,6 +167,32 @@ cut_test_container_filter_children (CutTestContainer *container,
     return matched_tests;
 }
 
+guint
+cut_test_container_get_n_tests (CutTestContainer *container,
+                                CutRunContext *run_context)
+{
+    const gchar **test_names = NULL;
+
+    if (run_context)
+        test_names = cut_run_context_get_target_test_names(run_context);
+
+    if (test_names) {
+        GList *filtered_tests;
+        guint n_tests;
+
+        filtered_tests = cut_test_container_filter_children(container,
+                                                            test_names);
+        n_tests = g_list_length(filtered_tests);
+        g_list_free(filtered_tests);
+        return n_tests;
+    } else {
+        CutTestContainerPrivate *priv;
+
+        priv = CUT_TEST_CONTAINER_GET_PRIVATE(container);
+        return g_list_length(priv->tests);
+    }
+}
+
 /*
 vi:nowrap:ai:expandtab:sw=4
 */
