@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2007  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2007-2008  Kouhei Sutou <kou@cozmixng.org>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -32,21 +32,31 @@
 #  include <unistd.h>
 #endif
 #include <glib.h>
+#include <glib/gi18n-lib.h>
 
+#ifdef G_OS_WIN32
+#  include <process.h>		/* For getpid() */
+#  include <io.h>
+#  define STRICT		/* Strict typing, please */
+#  define _WIN32_WINDOWS 0x0401 /* to get IsDebuggerPresent */
+#  include <windows.h>
+#  undef STRICT
+#endif
+
+#include "cut-run-context.h"
 #include "cut-test-context.h"
 #include "cut-test-suite.h"
 #include "cut-test-iterator.h"
 #include "cut-test-result.h"
 #include "cut-test-data.h"
 #include "cut-process.h"
+#include "cut-backtrace-entry.h"
 #include "cut-utils.h"
 
 #define cut_omit(context, message, ...) do                      \
 {                                                               \
     cut_test_context_register_result(context,                   \
                                      CUT_TEST_RESULT_OMISSION,  \
-                                     __PRETTY_FUNCTION__,       \
-                                     __FILE__, __LINE__,        \
                                      message, ## __VA_ARGS__,   \
                                      NULL);                     \
     cut_test_context_long_jump(context);                        \
@@ -57,6 +67,7 @@
 typedef struct _CutTestContextPrivate	CutTestContextPrivate;
 struct _CutTestContextPrivate
 {
+    CutRunContext *run_context;
     CutTestSuite *test_suite;
     CutTestCase *test_case;
     CutTestIterator *test_iterator;
@@ -64,21 +75,25 @@ struct _CutTestContextPrivate
     gboolean failed;
     gboolean is_multi_thread;
     jmp_buf *jump_buffer;
-    GList *taken_strings;
-    GList *taken_string_arrays;
     GList *taken_objects;
+    GList *taken_string_arrays;
+    GList *taken_g_objects;
     GList *taken_errors;
     GList *taken_lists;
+    GList *taken_hash_tables;
     GList *current_data;
     GList *data_list;
     GList *processes;
     gchar *fixture_data_dir;
     GHashTable *cached_fixture_data;
+    GList *backtrace;
+    GMutex *mutex;
 };
 
 enum
 {
     PROP_0,
+    PROP_RUN_CONTEXT,
     PROP_TEST_SUITE,
     PROP_TEST_CASE,
     PROP_TEST_ITERATOR,
@@ -87,6 +102,32 @@ enum
 
 G_DEFINE_TYPE (CutTestContext, cut_test_context, G_TYPE_OBJECT)
 
+typedef struct _TakenObject TakenObject;
+struct _TakenObject
+{
+    gpointer object;
+    CutDestroyFunction destroy_function;
+};
+
+static TakenObject *
+taken_object_new (gpointer object, CutDestroyFunction destroy_function)
+{
+    TakenObject *taken_object;
+
+    taken_object = g_slice_new(TakenObject);
+    taken_object->object = object;
+    taken_object->destroy_function = destroy_function;
+    return taken_object;
+}
+
+static void
+taken_object_free (TakenObject *taken_object)
+{
+    if (taken_object->object)
+        taken_object->destroy_function(taken_object->object);
+    g_slice_free(TakenObject, taken_object);
+}
+
 typedef struct _TakenList TakenList;
 struct _TakenList
 {
@@ -100,7 +141,7 @@ taken_list_new (GList *list, CutDestroyFunction destroy_function)
     TakenList *taken_list;
 
     taken_list = g_slice_new(TakenList);
-    taken_list->list = g_list_copy(list);
+    taken_list->list = list;
     taken_list->destroy_function = destroy_function;
     return taken_list;
 }
@@ -117,6 +158,7 @@ taken_list_free (TakenList *taken_list)
     g_slice_free(TakenList, taken_list);
 }
 
+static GStaticPrivate current_context_private;
 
 static void dispose        (GObject         *object);
 static void set_property   (GObject         *object,
@@ -141,32 +183,39 @@ cut_test_context_class_init (CutTestContextClass *klass)
     gobject_class->set_property = set_property;
     gobject_class->get_property = get_property;
 
+    spec = g_param_spec_object("run-context",
+                               "Run context",
+                               "The run context of the test context",
+                               CUT_TYPE_RUN_CONTEXT,
+                               G_PARAM_READWRITE);
+    g_object_class_install_property(gobject_class, PROP_RUN_CONTEXT, spec);
+
     spec = g_param_spec_object("test-suite",
                                "Test suite",
                                "The test suite of the test context",
                                CUT_TYPE_TEST_SUITE,
-                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
+                               G_PARAM_READWRITE);
     g_object_class_install_property(gobject_class, PROP_TEST_SUITE, spec);
 
     spec = g_param_spec_object("test-case",
                                "Test case",
                                "The test case of the test context",
                                CUT_TYPE_TEST_CASE,
-                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
+                               G_PARAM_READWRITE);
     g_object_class_install_property(gobject_class, PROP_TEST_CASE, spec);
 
     spec = g_param_spec_object("test-iterator",
                                "Test iterator",
                                "The test iterator of the test context",
                                CUT_TYPE_TEST_ITERATOR,
-                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
+                               G_PARAM_READWRITE);
     g_object_class_install_property(gobject_class, PROP_TEST_ITERATOR, spec);
 
     spec = g_param_spec_object("test",
                                "Test",
                                "The test of the test context",
                                CUT_TYPE_TEST,
-                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
+                               G_PARAM_READWRITE);
     g_object_class_install_property(gobject_class, PROP_TEST, spec);
 
     g_type_class_add_private(gobject_class, sizeof(CutTestContextPrivate));
@@ -177,6 +226,7 @@ cut_test_context_init (CutTestContext *context)
 {
     CutTestContextPrivate *priv = CUT_TEST_CONTEXT_GET_PRIVATE(context);
 
+    priv->run_context = NULL;
     priv->test_suite = NULL;
     priv->test_case = NULL;
     priv->test_iterator = NULL;
@@ -185,12 +235,7 @@ cut_test_context_init (CutTestContext *context)
     priv->failed = FALSE;
     priv->is_multi_thread = FALSE;
 
-    priv->taken_strings = NULL;
-    priv->taken_string_arrays = NULL;
-
     priv->taken_objects = NULL;
-    priv->taken_errors = NULL;
-    priv->taken_lists = NULL;
 
     priv->data_list = NULL;
     priv->current_data = NULL;
@@ -200,6 +245,10 @@ cut_test_context_init (CutTestContext *context)
     priv->fixture_data_dir = NULL;
     priv->cached_fixture_data = g_hash_table_new_full(g_str_hash, g_str_equal,
                                                       g_free, g_free);
+
+    priv->backtrace = NULL;
+
+    priv->mutex = g_mutex_new();
 }
 
 static void
@@ -218,6 +267,11 @@ dispose (GObject *object)
 {
     CutTestContextPrivate *priv = CUT_TEST_CONTEXT_GET_PRIVATE(object);
 
+    if (priv->run_context) {
+        g_object_unref(priv->run_context);
+        priv->run_context = NULL;
+    }
+
     if (priv->test_suite) {
         g_object_unref(priv->test_suite);
         priv->test_suite = NULL;
@@ -244,36 +298,12 @@ dispose (GObject *object)
         priv->processes = NULL;
     }
 
-    if (priv->taken_strings) {
-        g_list_foreach(priv->taken_strings, (GFunc)g_free, NULL);
-        g_list_free(priv->taken_strings);
-        priv->taken_strings = NULL;
-    }
-
-    if (priv->taken_string_arrays) {
-        g_list_foreach(priv->taken_string_arrays, (GFunc)g_strfreev, NULL);
-        g_list_free(priv->taken_string_arrays);
-        priv->taken_string_arrays = NULL;
-    }
-
     if (priv->taken_objects) {
-        g_list_foreach(priv->taken_objects, (GFunc)g_object_unref, NULL);
+        g_list_foreach(priv->taken_objects, (GFunc)taken_object_free, NULL);
         g_list_free(priv->taken_objects);
         priv->taken_objects = NULL;
     }
 
-    if (priv->taken_errors) {
-        g_list_foreach(priv->taken_errors, (GFunc)g_error_free, NULL);
-        g_list_free(priv->taken_errors);
-        priv->taken_errors = NULL;
-    }
-
-    if (priv->taken_lists) {
-        g_list_foreach(priv->taken_lists, (GFunc)taken_list_free, NULL);
-        g_list_free(priv->taken_lists);
-        priv->taken_lists = NULL;
-    }
-
     free_data_list(priv);
 
     if (priv->fixture_data_dir) {
@@ -286,6 +316,17 @@ dispose (GObject *object)
         priv->cached_fixture_data = NULL;
     }
 
+    if (priv->backtrace) {
+        g_list_foreach(priv->backtrace, (GFunc)g_object_unref, NULL);
+        g_list_free(priv->backtrace);
+        priv->backtrace = NULL;
+    }
+
+    if (priv->mutex) {
+        g_mutex_free(priv->mutex);
+        priv->mutex = NULL;
+    }
+
     G_OBJECT_CLASS(cut_test_context_parent_class)->dispose(object);
 }
 
@@ -298,6 +339,9 @@ set_property (GObject      *object,
     CutTestContext *context = CUT_TEST_CONTEXT(object);
 
     switch (prop_id) {
+      case PROP_RUN_CONTEXT:
+        cut_test_context_set_run_context(context, g_value_get_object(value));
+        break;
       case PROP_TEST_SUITE:
         cut_test_context_set_test_suite(context, g_value_get_object(value));
         break;
@@ -325,6 +369,9 @@ get_property (GObject    *object,
     CutTestContextPrivate *priv = CUT_TEST_CONTEXT_GET_PRIVATE(object);
 
     switch (prop_id) {
+      case PROP_RUN_CONTEXT:
+        g_value_set_object(value, priv->run_context);
+        break;
       case PROP_TEST_SUITE:
         g_value_set_object(value, priv->test_suite);
         break;
@@ -343,11 +390,43 @@ get_property (GObject    *object,
     }
 }
 
+GQuark
+cut_test_context_error_quark (void)
+{
+    return g_quark_from_static_string("cut-test-context-error-quark");
+}
+
+void
+cut_test_context_current_init (void)
+{
+    g_static_private_init(&current_context_private);
+}
+
+void
+cut_test_context_current_quit (void)
+{
+    g_static_private_free(&current_context_private);
+}
+
+void
+cut_test_context_current_set (CutTestContext *context)
+{
+    g_static_private_set(&current_context_private, context, NULL);
+}
+
 CutTestContext *
-cut_test_context_new (CutTestSuite *test_suite, CutTestCase *test_case,
+cut_test_context_current_get (void)
+{
+    return g_static_private_get(&current_context_private);
+}
+
+CutTestContext *
+cut_test_context_new (CutRunContext *run_context,
+                      CutTestSuite *test_suite, CutTestCase *test_case,
                       CutTestIterator *test_iterator, CutTest *test)
 {
     return g_object_new(CUT_TYPE_TEST_CONTEXT,
+                        "run-context", run_context,
                         "test-suite", test_suite,
                         "test-case", test_case,
                         "test-iterator", test_iterator,
@@ -358,7 +437,25 @@ cut_test_context_new (CutTestSuite *test_suite, CutTestCase *test_case,
 CutTestContext *
 cut_test_context_new_empty (void)
 {
-    return cut_test_context_new(NULL, NULL, NULL, NULL);
+    return cut_test_context_new(NULL, NULL, NULL, NULL, NULL);
+}
+
+CutRunContext *
+cut_test_context_get_run_context (CutTestContext *context)
+{
+    return CUT_TEST_CONTEXT_GET_PRIVATE(context)->run_context;
+}
+
+void
+cut_test_context_set_run_context (CutTestContext *context, CutRunContext *run_context)
+{
+    CutTestContextPrivate *priv = CUT_TEST_CONTEXT_GET_PRIVATE(context);
+
+    if (priv->run_context)
+        g_object_unref(priv->run_context);
+    if (run_context)
+        g_object_ref(run_context);
+    priv->run_context = run_context;
 }
 
 CutTestSuite *
@@ -463,6 +560,46 @@ cut_test_context_add_data (CutTestContext *context, const gchar *first_data_name
     va_end(args);
 }
 
+void
+cut_test_context_set_attributes (CutTestContext *context,
+                                 const gchar *first_attribute_name,
+                                 ...)
+{
+    CutTestContextPrivate *priv;
+    CutTest *target = NULL;
+    const gchar *name;
+    va_list args;
+
+    priv = CUT_TEST_CONTEXT_GET_PRIVATE(context);
+
+    if (priv->test) {
+        target = priv->test;
+    } else if (priv->test_iterator) {
+        target = CUT_TEST(priv->test_iterator);
+    } else if (priv->test_case) {
+        target = CUT_TEST(priv->test_case);
+    } else if (priv->test_suite) {
+        target = CUT_TEST(priv->test_suite);
+    }
+
+    if (!target) {
+        g_warning("can't find a target to set attributes.");
+        return;
+    }
+
+    name = first_attribute_name;
+    va_start(args, first_attribute_name);
+    while (name) {
+        const gchar *value;
+
+        value = va_arg(args, gchar *);
+        cut_test_set_attribute(target, name, value);
+
+        name = va_arg(args, gchar *);
+    }
+    va_end(args);
+}
+
 void
 cut_test_context_set_data (CutTestContext *context, CutTestData *test_data)
 {
@@ -479,6 +616,16 @@ cut_test_context_set_data (CutTestContext *context, CutTestData *test_data)
     }
 }
 
+CutTestData *
+cut_test_context_get_data (CutTestContext *context, guint index)
+{
+    CutTestContextPrivate *priv;
+
+    priv = CUT_TEST_CONTEXT_GET_PRIVATE(context);
+
+    return g_list_nth_data(priv->data_list, index);
+}
+
 void
 cut_test_context_shift_data (CutTestContext *context)
 {
@@ -538,8 +685,11 @@ cut_test_context_long_jump (CutTestContext *context)
 void
 cut_test_context_pass_assertion (CutTestContext *context)
 {
-    CutTestContextPrivate *priv = CUT_TEST_CONTEXT_GET_PRIVATE(context);
+    CutTestContextPrivate *priv;
 
+    g_return_if_fail(CUT_IS_TEST_CONTEXT(context));
+
+    priv = CUT_TEST_CONTEXT_GET_PRIVATE(context);
     g_return_if_fail(priv->test);
 
     g_signal_emit_by_name(priv->test, "pass-assertion", context);
@@ -564,54 +714,41 @@ void
 cut_test_context_emit_signal (CutTestContext *context,
                               CutTestResult *result)
 {
-    CutTestContextPrivate *priv = CUT_TEST_CONTEXT_GET_PRIVATE(context);
-    const gchar *status_signal_name = NULL;
-    CutTestResultStatus status;
-
-    status = cut_test_result_get_status(result);
-    status_signal_name = cut_test_result_status_to_signal_name(status);
+    CutTestContextPrivate *priv;
+    CutTest *target = NULL;
 
+    priv = CUT_TEST_CONTEXT_GET_PRIVATE(context);
     if (priv->test) {
-        cut_test_stop_timer(priv->test);
-        cut_test_result_set_elapsed(result, cut_test_get_elapsed(priv->test));
-        g_signal_emit_by_name(priv->test, status_signal_name,
-                              context, result);
+        target = priv->test;
     } else if (priv->test_iterator) {
-        g_signal_emit_by_name(priv->test_iterator, status_signal_name,
-                              context, result);
+        target = CUT_TEST(priv->test_iterator);
     } else if (priv->test_case) {
-        g_signal_emit_by_name(priv->test_case, status_signal_name,
-                              context, result);
+        target = CUT_TEST(priv->test_case);
     }
+
+    if (target)
+        cut_test_emit_result_signal(target, context, result);
 }
 
 void
-cut_test_context_register_result (CutTestContext *context,
-                                  CutTestResultStatus status,
-                                  const gchar *function_name,
-                                  const gchar *filename,
-                                  guint line,
-                                  const gchar *message,
-                                  ...)
+cut_test_context_register_result_va_list (CutTestContext *context,
+                                          CutTestResultStatus status,
+                                          const char *system_message,
+                                          const char *user_message_format,
+                                          va_list args)
 {
     CutTestContextPrivate *priv;
     CutTestResult *result;
     CutTestData *test_data = NULL;
-    const gchar *system_message;
-    gchar *user_message = NULL, *user_message_format;
-    va_list args;
+    gchar *user_message = NULL;
 
     priv = CUT_TEST_CONTEXT_GET_PRIVATE(context);
     if (cut_test_result_status_is_critical(status))
         priv->failed = TRUE;
 
-    system_message = message;
-    va_start(args, message);
-    user_message_format = va_arg(args, gchar *);
     if (user_message_format) {
         user_message = g_strdup_vprintf(user_message_format, args);
     }
-    va_end(args);
 
     if (priv->current_data)
         test_data = priv->current_data->data;
@@ -620,7 +757,7 @@ cut_test_context_register_result (CutTestContext *context,
                                  priv->test_case, priv->test_suite,
                                  test_data,
                                  user_message, system_message,
-                                 function_name, filename, line);
+                                 priv->backtrace);
     if (user_message)
         g_free(user_message);
 
@@ -639,9 +776,50 @@ cut_test_context_register_result (CutTestContext *context,
 
     cut_test_context_emit_signal(context, result);
 
+    if (status == CUT_TEST_RESULT_FAILURE &&
+        priv->run_context &&
+        cut_run_context_get_fatal_failures(priv->run_context)) {
+        const gchar *message;
+
+        message = _("Treat a failure as a fatal problem, aborting.");
+        cut_run_context_emit_error(priv->run_context,
+                                   CUT_TEST_CONTEXT_ERROR,
+                                   CUT_TEST_CONTEXT_ERROR_FATAL,
+                                   NULL, "%s", message);
+        cut_run_context_emit_complete_run(priv->run_context, FALSE);
+#ifdef G_OS_WIN32
+        if (IsDebuggerPresent())
+            G_BREAKPOINT();
+        else
+            abort();
+#else
+#  ifdef SIGTRAP
+        G_BREAKPOINT();
+#  else
+        abort();
+#  endif
+#endif
+    }
+
     g_object_unref(result);
 }
 
+void
+cut_test_context_register_result (CutTestContext *context,
+                                  CutTestResultStatus status,
+                                  const gchar *message,
+                                  ...)
+{
+    va_list args;
+    const gchar *user_message_format;
+
+    va_start(args, message);
+    user_message_format = va_arg(args, gchar *);
+    cut_test_context_register_result_va_list(context, status, message,
+                                             user_message_format, args);
+    va_end(args);
+}
+
 void
 cut_test_context_set_failed (CutTestContext *context, gboolean failed)
 {
@@ -654,18 +832,56 @@ cut_test_context_is_failed (CutTestContext *context)
     return CUT_TEST_CONTEXT_GET_PRIVATE(context)->failed;
 }
 
+const void *
+cut_test_context_take (CutTestContext *context,
+                       void           *object,
+                       CutDestroyFunction destroy_function)
+{
+    CutTestContextPrivate *priv;
+    TakenObject *taken_object;
+
+    priv = CUT_TEST_CONTEXT_GET_PRIVATE(context);
+    taken_object = taken_object_new(object, destroy_function);
+    priv->taken_objects = g_list_prepend(priv->taken_objects, taken_object);
+
+    return object;
+}
+
+const void *
+cut_test_context_take_memory (CutTestContext *context,
+                              void           *memory)
+{
+    return cut_test_context_take(context, memory, g_free);
+}
+
 const char *
 cut_test_context_take_string (CutTestContext *context,
                               char           *string)
 {
-    CutTestContextPrivate *priv = CUT_TEST_CONTEXT_GET_PRIVATE(context);
-    char *taken_string;
+    return cut_test_context_take(context, string, g_free);
+}
 
-    taken_string = g_strdup(string);
-    priv->taken_strings = g_list_prepend(priv->taken_strings, taken_string);
-    g_free(string);
+const char *
+cut_test_context_take_strdup (CutTestContext *context,
+                              const char     *string)
+{
+    return cut_test_context_take_string(context, g_strdup(string));
+}
 
-    return taken_string;
+const char *
+cut_test_context_take_strndup (CutTestContext *context,
+                               const char     *string,
+                               size_t          size)
+{
+    return cut_test_context_take_string(context, g_strndup(string, size));
+}
+
+const void *
+cut_test_context_take_memdup (CutTestContext *context,
+                              const void     *memory,
+                              size_t          size)
+{
+    return cut_test_context_take_memory(context, g_memdup(memory, size));
 }
 
 const char *
@@ -689,61 +905,49 @@ const char **
 cut_test_context_take_string_array (CutTestContext *context,
                                     char          **strings)
 {
-    CutTestContextPrivate *priv = CUT_TEST_CONTEXT_GET_PRIVATE(context);
-    char **taken_strings;
-
-    taken_strings = g_strdupv(strings);
-    priv->taken_string_arrays = g_list_prepend(priv->taken_string_arrays, taken_strings);
-    g_strfreev(strings);
-
-    return (const char **)taken_strings;
+    return (const char **)cut_test_context_take(context,
+                                                (gpointer)strings,
+                                                (CutDestroyFunction)g_strfreev);
 }
 
 GObject *
 cut_test_context_take_g_object (CutTestContext *context,
                                 GObject        *object)
 {
-    CutTestContextPrivate *priv = CUT_TEST_CONTEXT_GET_PRIVATE(context);
-
-    priv->taken_objects = g_list_prepend(priv->taken_objects, object);
-
-    return object;
+    return (GObject *)cut_test_context_take(context, object, g_object_unref);
 }
 
 const GError *
 cut_test_context_take_g_error (CutTestContext *context, GError *error)
 {
-    CutTestContextPrivate *priv;
-    GError *taken_error;
-
-    priv = CUT_TEST_CONTEXT_GET_PRIVATE(context);
-    taken_error = g_error_copy(error);
-    g_error_free(error);
-    priv->taken_errors = g_list_prepend(priv->taken_errors, taken_error);
-
-    return taken_error;
+    return cut_test_context_take(context, error,
+                                 (CutDestroyFunction) g_error_free);
 }
 
 const GList *
 cut_test_context_take_g_list (CutTestContext *context, GList *list,
                               CutDestroyFunction destroy_function)
 {
-    CutTestContextPrivate *priv;
     TakenList *taken_list;
 
-    priv = CUT_TEST_CONTEXT_GET_PRIVATE(context);
     taken_list = taken_list_new(list, destroy_function);
-    priv->taken_lists = g_list_prepend(priv->taken_lists, taken_list);
-    g_list_free(list);
+    cut_test_context_take(context, taken_list,
+                          (CutDestroyFunction)taken_list_free);
 
-    return taken_list->list;
+    return list;
+}
+
+GHashTable *
+cut_test_context_take_g_hash_table (CutTestContext *context,
+                                    GHashTable     *hash_table)
+{
+    return (GHashTable *)cut_test_context_take(context,
+                                               hash_table,
+                                               (CutDestroyFunction)g_hash_table_unref);
 }
 
 int
-cut_test_context_trap_fork (CutTestContext *context,
-                            const gchar *function_name,
-                            const gchar *filename,
-                            guint line)
+cut_test_context_trap_fork (CutTestContext *context)
 {
 #ifdef G_OS_WIN32
     cut_omit(context,
@@ -757,7 +961,6 @@ cut_test_context_trap_fork (CutTestContext *context,
     if (cut_test_context_is_multi_thread(context)) {
         cut_test_context_register_result(context,
                                          CUT_TEST_RESULT_OMISSION,
-                                         function_name, filename, line,
                                          "can't use cut_fork() "
                                          "in multi thread mode",
                                          NULL);
@@ -1008,6 +1211,111 @@ cut_test_context_get_fixture_data_string (CutTestContext *context,
     return value;
 }
 
+gchar *
+cut_test_context_build_source_filename (CutTestContext *context,
+                                        const gchar    *filename)
+{
+    CutTestContextPrivate *priv;
+    const gchar *base_directory = NULL;
+
+    priv = CUT_TEST_CONTEXT_GET_PRIVATE(context);
+    if (priv->test)
+        base_directory = cut_test_get_base_directory(priv->test);
+    if (!base_directory && priv->test_case)
+        base_directory = cut_test_get_base_directory(CUT_TEST(priv->test_case));
+    if (!base_directory && priv->test_suite)
+        base_directory = cut_test_get_base_directory(CUT_TEST(priv->test_suite));
+
+    if (base_directory) {
+        gchar *relative_filename, *full_filename;
+
+        relative_filename = g_build_filename(base_directory, filename, NULL);
+        if (priv->run_context) {
+            full_filename =
+                cut_run_context_build_source_filename(priv->run_context,
+                                                      relative_filename);
+            g_free(relative_filename);
+            return full_filename;
+        } else {
+            return relative_filename;
+        }
+    }
+
+
+    if (priv->run_context)
+        return cut_run_context_build_source_filename(priv->run_context,
+                                                     filename);
+    else
+        return g_strdup(filename);
+}
+
+void
+cut_test_context_push_backtrace (CutTestContext *context,
+                                 const char     *filename,
+                                 unsigned int    line,
+                                 const char     *function_name,
+                                 const char     *info)
+{
+    CutTestContextPrivate *priv;
+    CutBacktraceEntry *entry;
+    gchar *full_filename = NULL;
+
+    g_return_if_fail(context != NULL);
+    g_return_if_fail(CUT_IS_TEST_CONTEXT(context));
+
+    priv = CUT_TEST_CONTEXT_GET_PRIVATE(context);
+
+    g_mutex_lock(priv->mutex);
+    full_filename = cut_test_context_build_source_filename(context, filename);
+    entry = cut_backtrace_entry_new(full_filename, line, function_name, info);
+    g_free(full_filename);
+    priv->backtrace = g_list_prepend(priv->backtrace, entry);
+    g_mutex_unlock(priv->mutex);
+}
+
+void
+cut_test_context_pop_backtrace (CutTestContext *context)
+{
+    CutTestContextPrivate *priv;
+    CutBacktraceEntry *entry;
+
+    g_return_if_fail(context != NULL);
+    g_return_if_fail(CUT_IS_TEST_CONTEXT(context));
+
+    priv = CUT_TEST_CONTEXT_GET_PRIVATE(context);
+    g_return_if_fail(priv->backtrace != NULL);
+
+    g_mutex_lock(priv->mutex);
+    entry = priv->backtrace->data;
+    g_object_unref(entry);
+    priv->backtrace = g_list_delete_link(priv->backtrace, priv->backtrace);
+    g_mutex_unlock(priv->mutex);
+}
+
+void
+cut_test_context_get_last_backtrace (CutTestContext *context,
+                                     const char    **filename,
+                                     unsigned int   *line,
+                                     const char    **function_name,
+                                     const char    **info)
+{
+    CutTestContextPrivate *priv;
+    CutBacktraceEntry *entry;
+
+    priv = CUT_TEST_CONTEXT_GET_PRIVATE(context);
+    g_return_if_fail(priv->backtrace != NULL);
+
+    entry = priv->backtrace->data;
+    if (filename)
+        *filename = cut_backtrace_entry_get_file(entry);
+    if (line)
+        *line = cut_backtrace_entry_get_line(entry);
+    if (function_name)
+        *function_name = cut_backtrace_entry_get_function(entry);
+    if (info)
+        *info = cut_backtrace_entry_get_info(entry);
+}
+
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
 */
