@@ -40,32 +40,11 @@ static CutRunnerIface *parent_runner_iface;
 G_DEFINE_TYPE_WITH_CODE(CutTestRunner, cut_test_runner, CUT_TYPE_RUN_CONTEXT,
                         G_IMPLEMENT_INTERFACE(CUT_TYPE_RUNNER, runner_init))
 
-static void prepare_test_suite  (CutRunContext  *context,
-                                 CutTestSuite   *test_suite);
-static void prepare_test_case   (CutRunContext  *context,
-                                 CutTestCase    *test_case);
-static void prepare_test_iterator
-                                (CutRunContext  *context,
-                                 CutTestIterator *test_iterator);
-static void prepare_test        (CutRunContext  *context,
-                                 CutTest        *test);
-static void prepare_iterated_test
-                                (CutRunContext  *context,
-                                 CutIteratedTest *iterated_test);
 static gboolean runner_run (CutRunner *runner);
 
 static void
 cut_test_runner_class_init (CutTestRunnerClass *klass)
 {
-    CutRunContextClass *run_context_class;
-
-    run_context_class = CUT_RUN_CONTEXT_CLASS(klass);
-
-    run_context_class->prepare_test_suite = prepare_test_suite;
-    run_context_class->prepare_test_case = prepare_test_case;
-    run_context_class->prepare_test_iterator = prepare_test_iterator;
-    run_context_class->prepare_test = prepare_test;
-    run_context_class->prepare_iterated_test = prepare_iterated_test;
 }
 
 static void
@@ -88,8 +67,7 @@ cut_test_runner_new (void)
 }
 
 static void
-cb_start_test (gpointer test_case_or_test_iterator, CutTest *test,
-               CutTestContext *test_context, gpointer data)
+cb_start_test (CutTest *test, CutTestContext *test_context, gpointer data)
 {
     CutRunContext *context = data;
 
@@ -97,12 +75,128 @@ cb_start_test (gpointer test_case_or_test_iterator, CutTest *test,
 }
 
 static void
-cb_complete_test (gpointer test_case_or_test_iterator, CutTest *test,
-                  CutTestContext *test_context, gpointer data)
+cb_pass_assertion_test (CutTest *test, CutTestContext *test_context,
+                        gpointer data)
 {
     CutRunContext *context = data;
 
-    g_signal_emit_by_name(context, "complete-test", test, test_context);
+    g_signal_emit_by_name(context, "pass-assertion", test, test_context);
+}
+
+static void
+cb_success_test (CutTest *test, CutTestContext *test_context,
+                 CutTestResult *result, gpointer data)
+{
+    CutRunContext *context = data;
+
+    g_signal_emit_by_name(context, "success-test", test, test_context, result);
+}
+
+static void
+cb_failure_test (CutTest *test, CutTestContext *test_context,
+                 CutTestResult *result, gpointer data)
+{
+    CutRunContext *context = data;
+
+    g_signal_emit_by_name(context, "failure-test", test, test_context, result);
+}
+
+static void
+cb_error_test (CutTest *test, CutTestContext *test_context,
+               CutTestResult *result, gpointer data)
+{
+    CutRunContext *context = data;
+
+    g_signal_emit_by_name(context, "error-test", test, test_context, result);
+}
+
+static void
+cb_pending_test (CutTest *test, CutTestContext *test_context,
+                 CutTestResult *result, gpointer data)
+{
+    CutRunContext *context = data;
+
+    g_signal_emit_by_name(context, "pending-test", test, test_context, result);
+}
+
+static void
+cb_notification_test (CutTest *test, CutTestContext *test_context,
+                      CutTestResult *result, gpointer data)
+{
+    CutRunContext *context = data;
+
+    g_signal_emit_by_name(context, "notification-test",
+                          test, test_context, result);
+}
+
+static void
+cb_omission_test (CutTest *test, CutTestContext *test_context,
+                  CutTestResult *result, gpointer data)
+{
+    CutRunContext *context = data;
+
+    g_signal_emit_by_name(context, "omission-test", test, test_context, result);
+}
+
+static void
+cb_complete_test (CutTest *test, CutTestContext *test_context,
+                  gboolean success, gpointer data)
+{
+    CutRunContext *context = data;
+
+    g_signal_emit_by_name(context, "complete-test", test, test_context, success);
+}
+
+static void
+connect_to_test (CutRunContext *context, CutTest *test)
+{
+#define CONNECT(name)                                                   \
+    g_signal_connect(test, #name,                                       \
+                     G_CALLBACK(cb_ ## name ## _test), context)
+
+    CONNECT(start);
+    CONNECT(pass_assertion);
+    CONNECT(success);
+    CONNECT(failure);
+    CONNECT(error);
+    CONNECT(pending);
+    CONNECT(notification);
+    CONNECT(omission);
+    CONNECT(complete);
+#undef CONNECT
+}
+
+static void
+cb_start_test_test_case (CutTestCase *test_case, CutTest *test,
+                         CutTestContext *test_context, gpointer data)
+{
+    CutRunContext *context = data;
+
+    connect_to_test(context, test);
+}
+
+static void
+cb_complete_test_test_case (CutTestCase *test_case, CutTest *test,
+                            CutTestContext *test_context, gboolean success,
+                            gpointer data)
+{
+#define DISCONNECT(name)                                                \
+    g_signal_handlers_disconnect_by_func(test,                          \
+                                         G_CALLBACK(cb_ ##              \
+                                                    name ##             \
+                                                    _test),             \
+                                         data)
+
+    DISCONNECT(start);
+    DISCONNECT(pass_assertion);
+    DISCONNECT(success);
+    DISCONNECT(failure);
+    DISCONNECT(error);
+    DISCONNECT(pending);
+    DISCONNECT(notification);
+    DISCONNECT(omission);
+    DISCONNECT(complete);
+#undef DISCONNECT
 }
 
 static void
@@ -160,7 +254,8 @@ cb_omission_test_case (CutTestCase *test_case, CutTestContext *test_context,
 }
 
 static void
-cb_start_test_case(CutTestCase *test_case, gpointer data)
+cb_start_test_case (CutTestCase *test_case, CutTestContext *test_context,
+                    gpointer data)
 {
     CutRunContext *context = data;
 
@@ -168,7 +263,7 @@ cb_start_test_case(CutTestCase *test_case, gpointer data)
 }
 
 static void
-cb_ready_test_case(CutTestCase *test_case, guint n_tests, gpointer data)
+cb_ready_test_case (CutTestCase *test_case, guint n_tests, gpointer data)
 {
     CutRunContext *context = data;
 
@@ -176,23 +271,321 @@ cb_ready_test_case(CutTestCase *test_case, guint n_tests, gpointer data)
 }
 
 static void
-cb_complete_test_case(CutTestCase *test_case, gpointer data)
+cb_complete_test_case (CutTestCase *test_case, CutTestContext *test_context,
+                       gboolean success, gpointer data)
+{
+    CutRunContext *context = data;
+
+    g_signal_emit_by_name(context, "complete-test-case", test_case, success);
+}
+
+static void
+cb_ready_test_suite (CutTestSuite *test_suite, guint n_test_cases,
+                     guint n_tests, gpointer data)
+{
+    CutRunContext *context = data;
+
+    g_signal_emit_by_name(context, "ready-test-suite", test_suite,
+                          n_test_cases, n_tests);
+}
+
+static void
+cb_start_test_suite (CutTestSuite *test_suite, CutTestContext *test_context,
+                     gpointer data)
+{
+    CutRunContext *context = data;
+
+    g_signal_emit_by_name(context, "start-test-suite", test_suite);
+}
+
+static void
+cb_crashed_test_suite (CutTestSuite *test_suite, const gchar *backtrace,
+                       gpointer data)
+{
+    CutRunContext *context = data;
+
+    cut_run_context_crash(context, backtrace);
+    g_signal_emit_by_name(context, "crashed", backtrace);
+}
+
+static void
+cb_start_iterated_test (CutIteratedTest *iterated_test,
+                        CutTestContext *test_context, gpointer data)
+{
+    CutRunContext *context = data;
+
+    g_signal_emit_by_name(context, "start-iterated-test",
+                          iterated_test, test_context);
+}
+
+static void
+cb_pass_assertion_iterated_test (CutIteratedTest *iterated_test,
+                                 CutTestContext *test_context, gpointer data)
+{
+    cb_pass_assertion_test(CUT_TEST(iterated_test), test_context, data);
+}
+
+static void
+cb_success_iterated_test (CutIteratedTest *iterated_test,
+                          CutTestContext *test_context,
+                          CutTestResult *result, gpointer data)
+{
+    cb_success_test(CUT_TEST(iterated_test), test_context, result, data);
+}
+
+static void
+cb_failure_iterated_test (CutIteratedTest *iterated_test,
+                          CutTestContext *test_context,
+                          CutTestResult *result, gpointer data)
+{
+    cb_failure_test(CUT_TEST(iterated_test), test_context, result, data);
+}
+
+static void
+cb_error_iterated_test (CutIteratedTest *iterated_test,
+                        CutTestContext *test_context,
+                        CutTestResult *result, gpointer data)
+{
+    cb_error_test(CUT_TEST(iterated_test), test_context, result, data);
+}
+
+static void
+cb_pending_iterated_test (CutIteratedTest *iterated_test,
+                          CutTestContext *test_context,
+                          CutTestResult *result, gpointer data)
+{
+    cb_pending_test(CUT_TEST(iterated_test), test_context, result, data);
+}
+
+static void
+cb_notification_iterated_test (CutIteratedTest *iterated_test,
+                               CutTestContext *test_context,
+                               CutTestResult *result, gpointer data)
+{
+    cb_notification_test(CUT_TEST(iterated_test), test_context, result, data);
+}
+
+static void
+cb_omission_iterated_test (CutIteratedTest *iterated_test,
+                           CutTestContext *test_context,
+                           CutTestResult *result, gpointer data)
+{
+    cb_omission_test(CUT_TEST(iterated_test), test_context, result, data);
+}
+
+static void
+cb_complete_iterated_test (CutIteratedTest *iterated_test,
+                           CutTestContext *test_context, gboolean success,
+                           gpointer data)
+{
+    CutRunContext *context = data;
+
+    g_signal_emit_by_name(context, "complete-iterated-test",
+                          iterated_test, test_context, success);
+}
+
+static void
+cb_start_iterated_test_test_iterator (CutTestIterator *test_iterator,
+                                      CutIteratedTest *iterated_test,
+                                      CutTestContext *test_context,
+                                      gpointer data)
+{
+    CutRunContext *context = data;
+
+#define CONNECT(name)                                                   \
+    g_signal_connect(iterated_test, #name,                              \
+                     G_CALLBACK(cb_ ## name ## _iterated_test),         \
+                     context)
+
+    CONNECT(start);
+    CONNECT(complete);
+
+    CONNECT(pass_assertion);
+    CONNECT(success);
+    CONNECT(failure);
+    CONNECT(error);
+    CONNECT(pending);
+    CONNECT(notification);
+    CONNECT(omission);
+#undef CONNECT
+}
+
+static void
+cb_complete_iterated_test_test_iterator (CutTestIterator *test_iterator,
+                                         CutIteratedTest *iterated_test,
+                                         CutTestContext *test_context,
+                                         gboolean success,
+                                         gpointer data)
+{
+#define DISCONNECT(name)                                                \
+    g_signal_handlers_disconnect_by_func(iterated_test,                 \
+                                         G_CALLBACK(cb_ ##              \
+                                                    name ##             \
+                                                    _iterated_test),    \
+                                         data)
+
+    DISCONNECT(start);
+    DISCONNECT(complete);
+
+    DISCONNECT(pass_assertion);
+    DISCONNECT(success);
+    DISCONNECT(failure);
+    DISCONNECT(error);
+    DISCONNECT(pending);
+    DISCONNECT(notification);
+    DISCONNECT(omission);
+#undef DISCONNECT
+}
+
+static void
+cb_success_test_iterator (CutTestIterator *test_iterator,
+                          CutTestContext *test_context,
+                          CutTestResult *result, gpointer data)
+{
+    CutRunContext *context = data;
+
+    g_signal_emit_by_name(context, "success-test-iterator",
+                          test_iterator, result);
+}
+
+static void
+cb_failure_test_iterator (CutTestIterator *test_iterator,
+                          CutTestContext *test_context,
+                          CutTestResult *result, gpointer data)
+{
+    CutRunContext *context = data;
+
+    g_signal_emit_by_name(context, "failure-test-iterator",
+                          test_iterator, result);
+}
+
+static void
+cb_error_test_iterator (CutTestIterator *test_iterator,
+                        CutTestContext *test_context,
+                        CutTestResult *result, gpointer data)
+{
+    CutRunContext *context = data;
+
+    g_signal_emit_by_name(context, "error-test-iterator",
+                          test_iterator, result);
+}
+
+static void
+cb_pending_test_iterator (CutTestIterator *test_iterator,
+                          CutTestContext *test_context,
+                          CutTestResult *result, gpointer data)
+{
+    CutRunContext *context = data;
+
+    g_signal_emit_by_name(context, "pending-test-iterator",
+                          test_iterator, result);
+}
+
+static void
+cb_notification_test_iterator (CutTestIterator *test_iterator,
+                               CutTestContext *test_context,
+                               CutTestResult *result, gpointer data)
+{
+    CutRunContext *context = data;
+
+    g_signal_emit_by_name(context, "notification-test-iterator",
+                          test_iterator, result);
+}
+
+static void
+cb_omission_test_iterator (CutTestIterator *test_iterator,
+                           CutTestContext *test_context,
+                           CutTestResult *result, gpointer data)
+{
+    CutRunContext *context = data;
+
+    g_signal_emit_by_name(context, "omission-test-iterator",
+                          test_iterator, result);
+}
+
+static void
+cb_start_test_iterator (CutTestIterator *test_iterator,
+                        CutTestContext *test_context, gpointer data)
+{
+    CutRunContext *context = data;
+
+    g_signal_emit_by_name(context, "start-test-iterator", test_iterator);
+}
+
+static void
+cb_ready_test_iterator (CutTestIterator *test_iterator, guint n_tests,
+                        gpointer data)
+{
+    CutRunContext *context = data;
+
+    g_signal_emit_by_name(context, "ready-test-iterator",
+                          test_iterator, n_tests);
+}
+
+static void
+cb_complete_test_iterator (CutTestIterator *test_iterator,
+                           CutTestContext *test_context,
+                           gboolean success, gpointer data)
+{
+    CutRunContext *context = data;
+
+    g_signal_emit_by_name(context, "complete-test-iterator",
+                          test_iterator, success);
+}
+
+static void
+connect_to_test_iterator (CutRunContext *context, CutTestIterator *test_iterator)
+{
+
+#define CONNECT(name)                                                   \
+    g_signal_connect(test_iterator, #name,                              \
+                     G_CALLBACK(cb_ ## name ## _test_iterator), context)
+
+    CONNECT(start_iterated_test);
+    CONNECT(complete_iterated_test);
+
+    CONNECT(success);
+    CONNECT(failure);
+    CONNECT(error);
+    CONNECT(pending);
+    CONNECT(notification);
+    CONNECT(omission);
+
+    CONNECT(ready);
+    CONNECT(start);
+    CONNECT(complete);
+#undef CONNECT
+}
+
+static void
+cb_start_test_iterator_test_case (CutTestCase *test_case,
+                                  CutTestIterator *test_iterator,
+                                  CutTestContext *test_context,
+                                  gpointer data)
+{
+    CutRunContext *context = data;
+
+    connect_to_test_iterator(context, test_iterator);
+}
+
+static void
+cb_complete_test_iterator_test_case (CutTestCase *test_case,
+                                     CutTestIterator *test_iterator,
+                                     CutTestContext *test_context,
+                                     gboolean success,
+                                     gpointer data)
 {
     CutRunContext *context = data;
 
 #define DISCONNECT(name)                                                \
-    g_signal_handlers_disconnect_by_func(test_case,                     \
-                                         G_CALLBACK(cb_ ## name),       \
-                                         data)
+    g_signal_handlers_disconnect_by_func(test_iterator,                 \
+                                         G_CALLBACK(cb_ ##              \
+                                                    name ##             \
+                                                    _test_iterator),    \
+                                         context)
 
-    DISCONNECT(start_test);
-    DISCONNECT(complete_test);
-#undef DISCONNECT
-
-#define DISCONNECT(name)                                                \
-    g_signal_handlers_disconnect_by_func(test_case,                     \
-                                         G_CALLBACK(cb_ ## name ## _test_case), \
-                                         data)
+    DISCONNECT(start_iterated_test);
+    DISCONNECT(complete_iterated_test);
 
     DISCONNECT(success);
     DISCONNECT(failure);
@@ -205,40 +598,78 @@ cb_complete_test_case(CutTestCase *test_case, gpointer data)
     DISCONNECT(start);
     DISCONNECT(complete);
 #undef DISCONNECT
-
-    g_signal_emit_by_name(context, "complete-test-case", test_case);
 }
 
 static void
-cb_ready_test_suite(CutTestSuite *test_suite, guint n_test_cases,
-                    guint n_tests, gpointer data)
+connect_to_test_case (CutRunContext *context, CutTestCase *test_case)
+{
+#define CONNECT(name)                                                   \
+    g_signal_connect(test_case, #name,                                  \
+                     G_CALLBACK(cb_ ## name ## _test_case), context)
+
+    CONNECT(start_test);
+    CONNECT(complete_test);
+    CONNECT(start_test_iterator);
+    CONNECT(complete_test_iterator);
+
+    CONNECT(success);
+    CONNECT(failure);
+    CONNECT(error);
+    CONNECT(pending);
+    CONNECT(notification);
+    CONNECT(omission);
+
+    CONNECT(ready);
+    CONNECT(start);
+    CONNECT(complete);
+#undef CONNECT
+}
+
+static void
+cb_start_test_case_test_suite (CutTestSuite *test_suite, CutTestCase *test_case,
+                               gpointer data)
 {
     CutRunContext *context = data;
 
-    g_signal_emit_by_name(context, "ready-test-suite", test_suite,
-                          n_test_cases, n_tests);
+    connect_to_test_case(context, test_case);
 }
 
 static void
-cb_start_test_suite(CutTestSuite *test_suite, gpointer data)
+cb_complete_test_case_test_suite (CutTestSuite *test_suite,
+                                  CutTestCase *test_case,
+                                  gboolean success,
+                                  gpointer data)
 {
     CutRunContext *context = data;
 
-    g_signal_emit_by_name(context, "start-test-suite", test_suite);
+#define DISCONNECT(name)                                                \
+    g_signal_handlers_disconnect_by_func(test_case,                     \
+                                         G_CALLBACK(cb_ ##              \
+                                                    name ##             \
+                                                    _test_case),        \
+                                         context)
+
+    DISCONNECT(start_test);
+    DISCONNECT(complete_test);
+    DISCONNECT(start_test_iterator);
+    DISCONNECT(complete_test_iterator);
+
+    DISCONNECT(success);
+    DISCONNECT(failure);
+    DISCONNECT(error);
+    DISCONNECT(pending);
+    DISCONNECT(notification);
+    DISCONNECT(omission);
+
+    DISCONNECT(ready);
+    DISCONNECT(start);
+    DISCONNECT(complete);
+#undef DISCONNECT
 }
 
 static void
-cb_crashed_test_suite(CutTestSuite *test_suite, const gchar *backtrace,
-                      gpointer data)
-{
-    CutRunContext *context = data;
-
-    cut_run_context_crash(context, backtrace);
-    g_signal_emit_by_name(context, "crashed", backtrace);
-}
-
-static void
-cb_complete_test_suite(CutTestSuite *test_suite, gpointer data)
+cb_complete_test_suite (CutTestSuite *test_suite, CutTestContext *test_context,
+                        gboolean success, gpointer data)
 {
     CutRunContext *context = data;
 
@@ -253,20 +684,17 @@ cb_complete_test_suite(CutTestSuite *test_suite, gpointer data)
     DISCONNECT(start);
     DISCONNECT(complete);
     DISCONNECT(crashed);
+
+    DISCONNECT(start_test_case);
+    DISCONNECT(complete_test_case);
 #undef DISCONNECT
 
-    g_signal_emit_by_name(context, "complete-test-suite", test_suite);
+    g_signal_emit_by_name(context, "complete-test-suite", test_suite, success);
 }
 
 static void
-prepare_test_suite (CutRunContext *context, CutTestSuite *test_suite)
+connect_to_test_suite (CutRunContext *context, CutTestSuite *test_suite)
 {
-    CutRunContextClass *klass;
-
-    klass = CUT_RUN_CONTEXT_CLASS(cut_test_runner_parent_class);
-    if (klass->prepare_test_suite)
-        klass->prepare_test_suite(context, test_suite);
-
 #define CONNECT(name)                                                \
     g_signal_connect(test_suite, #name,                              \
                      G_CALLBACK(cb_ ## name ## _test_suite),         \
@@ -276,331 +704,12 @@ prepare_test_suite (CutRunContext *context, CutTestSuite *test_suite)
     CONNECT(start);
     CONNECT(complete);
     CONNECT(crashed);
+
+    CONNECT(start_test_case);
+    CONNECT(complete_test_case);
 #undef CONNECT
 }
 
-static void
-prepare_test_case (CutRunContext *context, CutTestCase *test_case)
-{
-    CutRunContextClass *klass;
-
-    klass = CUT_RUN_CONTEXT_CLASS(cut_test_runner_parent_class);
-    if (klass->prepare_test_case)
-        klass->prepare_test_case(context, test_case);
-
-#define CONNECT(name) \
-    g_signal_connect(test_case, #name, G_CALLBACK(cb_ ## name), context)
-
-    CONNECT(start_test);
-    CONNECT(complete_test);
-#undef CONNECT
-
-#define CONNECT(name)                                                   \
-    g_signal_connect(test_case, #name,                                  \
-                     G_CALLBACK(cb_ ## name ## _test_case), context)
-
-    CONNECT(success);
-    CONNECT(failure);
-    CONNECT(error);
-    CONNECT(pending);
-    CONNECT(notification);
-    CONNECT(omission);
-
-    CONNECT(ready);
-    CONNECT(start);
-    CONNECT(complete);
-#undef CONNECT
-}
-
-static void
-cb_start_iterated_test (gpointer test_case_or_test_iterator, CutTest *test,
-                        CutTestContext *test_context, gpointer data)
-{
-    CutRunContext *context = data;
-
-    g_signal_emit_by_name(context, "start-iterated-test", test, test_context);
-}
-
-static void
-cb_complete_iterated_test (gpointer test_case_or_test_iterator, CutTest *test,
-                           CutTestContext *test_context, gpointer data)
-{
-    CutRunContext *context = data;
-
-    g_signal_emit_by_name(context, "complete-iterated-test", test, test_context);
-}
-
-static void
-cb_success_test_iterator (CutTestIterator *test_iterator,
-                          CutTestContext *test_context,
-                          CutTestResult *result, gpointer data)
-{
-    CutRunContext *context = data;
-
-    g_signal_emit_by_name(context, "success-test-iterator", test_iterator, result);
-}
-
-static void
-cb_failure_test_iterator (CutTestIterator *test_iterator,
-                          CutTestContext *test_context,
-                          CutTestResult *result, gpointer data)
-{
-    CutRunContext *context = data;
-
-    g_signal_emit_by_name(context, "failure-test-iterator", test_iterator, result);
-}
-
-static void
-cb_error_test_iterator (CutTestIterator *test_iterator,
-                        CutTestContext *test_context,
-                        CutTestResult *result, gpointer data)
-{
-    CutRunContext *context = data;
-
-    g_signal_emit_by_name(context, "error-test-iterator", test_iterator, result);
-}
-
-static void
-cb_pending_test_iterator (CutTestIterator *test_iterator,
-                          CutTestContext *test_context,
-                          CutTestResult *result, gpointer data)
-{
-    CutRunContext *context = data;
-
-    g_signal_emit_by_name(context, "pending-test-iterator", test_iterator, result);
-}
-
-static void
-cb_notification_test_iterator (CutTestIterator *test_iterator,
-                               CutTestContext *test_context,
-                               CutTestResult *result, gpointer data)
-{
-    CutRunContext *context = data;
-
-    g_signal_emit_by_name(context, "notification-test-iterator", test_iterator, result);
-}
-
-static void
-cb_omission_test_iterator (CutTestIterator *test_iterator,
-                           CutTestContext *test_context,
-                           CutTestResult *result, gpointer data)
-{
-    CutRunContext *context = data;
-
-    g_signal_emit_by_name(context, "omission-test-iterator", test_iterator, result);
-}
-
-static void
-cb_start_test_iterator(CutTestIterator *test_iterator, gpointer data)
-{
-    CutRunContext *context = data;
-
-    g_signal_emit_by_name(context, "start-test-iterator", test_iterator);
-}
-
-static void
-cb_ready_test_iterator(CutTestIterator *test_iterator, guint n_tests,
-                       gpointer data)
-{
-    CutRunContext *context = data;
-
-    g_signal_emit_by_name(context, "ready-test-iterator", test_iterator, n_tests);
-}
-
-static void
-cb_complete_test_iterator(CutTestIterator *test_iterator, gpointer data)
-{
-    CutRunContext *context = data;
-
-#define DISCONNECT(name)                                                \
-    g_signal_handlers_disconnect_by_func(test_iterator,                 \
-                                         G_CALLBACK(cb_ ## name),       \
-                                         data)
-
-    DISCONNECT(start_test);
-    DISCONNECT(complete_test);
-#undef DISCONNECT
-
-#define DISCONNECT(name)                                                \
-    g_signal_handlers_disconnect_by_func(test_iterator,                 \
-                                         G_CALLBACK(cb_ ## name ## _test_iterator), \
-                                         data)
-
-    DISCONNECT(success);
-    DISCONNECT(failure);
-    DISCONNECT(error);
-    DISCONNECT(pending);
-    DISCONNECT(notification);
-    DISCONNECT(omission);
-
-    DISCONNECT(ready);
-    DISCONNECT(start);
-    DISCONNECT(complete);
-#undef DISCONNECT
-
-    g_signal_emit_by_name(context, "complete-test-iterator", test_iterator);
-}
-
-static void
-prepare_test_iterator (CutRunContext *context, CutTestIterator *test_iterator)
-{
-    CutRunContextClass *klass;
-
-    klass = CUT_RUN_CONTEXT_CLASS(cut_test_runner_parent_class);
-    if (klass->prepare_test_iterator)
-        klass->prepare_test_iterator(context, test_iterator);
-
-#define CONNECT(name)                                                   \
-    g_signal_connect(test_iterator, #name "-test",                      \
-                     G_CALLBACK(cb_ ## name ## _iterated_test), context)
-
-    CONNECT(start);
-    CONNECT(complete);
-#undef CONNECT
-
-#define CONNECT(name)                                                   \
-    g_signal_connect(test_iterator, #name,                              \
-                     G_CALLBACK(cb_ ## name ## _test_iterator), context)
-
-    CONNECT(success);
-    CONNECT(failure);
-    CONNECT(error);
-    CONNECT(pending);
-    CONNECT(notification);
-    CONNECT(omission);
-
-    CONNECT(ready);
-    CONNECT(start);
-    CONNECT(complete);
-#undef CONNECT
-}
-
-static void
-cb_pass_assertion (CutTest *test, CutTestContext *test_context, gpointer data)
-{
-    CutRunContext *context = data;
-
-    g_signal_emit_by_name(context, "pass-assertion", test, test_context);
-}
-
-static void
-cb_success (CutTest *test, CutTestContext *test_context, CutTestResult *result,
-            gpointer data)
-{
-    CutRunContext *context = data;
-
-    g_signal_emit_by_name(context, "success-test", test, test_context, result);
-}
-
-static void
-cb_failure (CutTest *test, CutTestContext *test_context, CutTestResult *result,
-            gpointer data)
-{
-    CutRunContext *context = data;
-
-    g_signal_emit_by_name(context, "failure-test", test, test_context, result);
-}
-
-static void
-cb_error (CutTest *test, CutTestContext *test_context, CutTestResult *result,
-          gpointer data)
-{
-    CutRunContext *context = data;
-
-    g_signal_emit_by_name(context, "error-test", test, test_context, result);
-}
-
-static void
-cb_pending (CutTest *test, CutTestContext *test_context, CutTestResult *result,
-            gpointer data)
-{
-    CutRunContext *context = data;
-
-    g_signal_emit_by_name(context, "pending-test", test, test_context, result);
-}
-
-static void
-cb_notification (CutTest *test, CutTestContext *test_context,
-                 CutTestResult *result, gpointer data)
-{
-    CutRunContext *context = data;
-
-    g_signal_emit_by_name(context, "notification-test",
-                          test, test_context, result);
-}
-
-static void
-cb_omission (CutTest *test, CutTestContext *test_context,
-             CutTestResult *result, gpointer data)
-{
-    CutRunContext *context = data;
-
-    g_signal_emit_by_name(context, "omission-test", test, test_context, result);
-}
-
-static void
-cb_complete (CutTest *test, gpointer data)
-{
-#define DISCONNECT(name)                                                \
-    g_signal_handlers_disconnect_by_func(test,                          \
-                                         G_CALLBACK(cb_ ## name),       \
-                                         data)
-
-    DISCONNECT(pass_assertion);
-    DISCONNECT(success);
-    DISCONNECT(failure);
-    DISCONNECT(error);
-    DISCONNECT(pending);
-    DISCONNECT(notification);
-    DISCONNECT(omission);
-    DISCONNECT(complete);
-#undef DISCONNECT
-
-}
-
-static void
-prepare_test (CutRunContext *context, CutTest *test)
-{
-    CutRunContextClass *klass;
-
-    klass = CUT_RUN_CONTEXT_CLASS(cut_test_runner_parent_class);
-    if (klass->prepare_test)
-        klass->prepare_test(context, test);
-
-    g_signal_connect(test, "pass_assertion",
-                     G_CALLBACK(cb_pass_assertion), context);
-    g_signal_connect(test, "success", G_CALLBACK(cb_success), context);
-    g_signal_connect(test, "failure", G_CALLBACK(cb_failure), context);
-    g_signal_connect(test, "error", G_CALLBACK(cb_error), context);
-    g_signal_connect(test, "pending", G_CALLBACK(cb_pending), context);
-    g_signal_connect(test, "notification", G_CALLBACK(cb_notification), context);
-    g_signal_connect(test, "omission", G_CALLBACK(cb_omission), context);
-    g_signal_connect(test, "complete", G_CALLBACK(cb_complete), context);
-}
-
-static void
-prepare_iterated_test (CutRunContext *context, CutIteratedTest *iterated_test)
-{
-    CutRunContextClass *klass;
-
-    klass = CUT_RUN_CONTEXT_CLASS(cut_test_runner_parent_class);
-    if (klass->prepare_iterated_test)
-        klass->prepare_iterated_test(context, iterated_test);
-
-    g_signal_connect(iterated_test, "pass_assertion",
-                     G_CALLBACK(cb_pass_assertion), context);
-    g_signal_connect(iterated_test, "success", G_CALLBACK(cb_success), context);
-    g_signal_connect(iterated_test, "failure", G_CALLBACK(cb_failure), context);
-    g_signal_connect(iterated_test, "error", G_CALLBACK(cb_error), context);
-    g_signal_connect(iterated_test, "pending", G_CALLBACK(cb_pending), context);
-    g_signal_connect(iterated_test, "notification",
-                     G_CALLBACK(cb_notification), context);
-    g_signal_connect(iterated_test, "omission",
-                     G_CALLBACK(cb_omission), context);
-    g_signal_connect(iterated_test, "complete",
-                     G_CALLBACK(cb_complete), context);
-}
-
 static gboolean
 runner_run (CutRunner *runner)
 {
@@ -611,11 +720,54 @@ runner_run (CutRunner *runner)
     context = CUT_RUN_CONTEXT(runner);
     suite = cut_run_context_get_test_suite(context);
     g_signal_emit_by_name(context, "start-run");
-    success = cut_test_suite_run(suite, context);
-    g_signal_emit_by_name(context, "complete-run", success);
+    success = cut_test_runner_run_test_suite(CUT_TEST_RUNNER(runner), suite);
+    success = cut_run_context_emit_complete_run(context, success);
     return success;
 }
 
+gboolean
+cut_test_runner_run_test (CutTestRunner *runner, CutTest *test,
+                          CutTestContext *test_context)
+{
+    CutRunContext *context;
+
+    context = CUT_RUN_CONTEXT(runner);
+    connect_to_test(context, test);
+    return cut_test_run(test, test_context, context);
+}
+
+gboolean
+cut_test_runner_run_test_case (CutTestRunner *runner, CutTestCase *test_case)
+{
+    CutRunContext *context;
+
+    context = CUT_RUN_CONTEXT(runner);
+    connect_to_test_case(context, test_case);
+    return cut_test_case_run(test_case, context);
+}
+
+gboolean
+cut_test_runner_run_test_iterator (CutTestRunner *runner,
+                                   CutTestIterator *test_iterator,
+                                   CutTestContext *test_context)
+{
+    CutRunContext *context;
+
+    context = CUT_RUN_CONTEXT(runner);
+    connect_to_test_iterator(context, test_iterator);
+    return cut_test_run(CUT_TEST(test_iterator), test_context, context);
+}
+
+gboolean
+cut_test_runner_run_test_suite (CutTestRunner *runner, CutTestSuite *test_suite)
+{
+    CutRunContext *context;
+
+    context = CUT_RUN_CONTEXT(runner);
+    connect_to_test_suite(context, test_suite);
+    return cut_test_suite_run(test_suite, context);
+}
+
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
 */
