@@ -43,6 +43,8 @@
 #include "cut-marshalers.h"
 #include "cut-test-result.h"
 
+#include "../gcutter/gcut-error.h"
+
 #define CUT_TEST_SUITE_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), CUT_TYPE_TEST_SUITE, CutTestSuitePrivate))
 
 typedef struct _CutTestSuitePrivate	CutTestSuitePrivate;
@@ -61,9 +63,9 @@ enum
 
 enum
 {
-    READY_SIGNAL,
-    START_TEST_CASE_SIGNAL,
-    COMPLETE_TEST_CASE_SIGNAL,
+    READY,
+    START_TEST_CASE,
+    COMPLETE_TEST_CASE,
     LAST_SIGNAL
 };
 
@@ -110,7 +112,7 @@ cut_test_suite_class_init (CutTestSuiteClass *klass)
                                 G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
     g_object_class_install_property(gobject_class, PROP_COOLDOWN_FUNCTION, spec);
 
-    cut_test_suite_signals[READY_SIGNAL]
+    cut_test_suite_signals[READY]
         = g_signal_new("ready",
                        G_TYPE_FROM_CLASS(klass),
                        G_SIGNAL_RUN_LAST,
@@ -119,7 +121,7 @@ cut_test_suite_class_init (CutTestSuiteClass *klass)
                        _cut_marshal_VOID__UINT_UINT,
                        G_TYPE_NONE, 2, G_TYPE_UINT, G_TYPE_UINT);
 
-    cut_test_suite_signals[START_TEST_CASE_SIGNAL]
+    cut_test_suite_signals[START_TEST_CASE]
         = g_signal_new("start-test-case",
                        G_TYPE_FROM_CLASS(klass),
                        G_SIGNAL_RUN_LAST,
@@ -128,14 +130,14 @@ cut_test_suite_class_init (CutTestSuiteClass *klass)
                        g_cclosure_marshal_VOID__OBJECT,
                        G_TYPE_NONE, 1, CUT_TYPE_TEST_CASE);
 
-    cut_test_suite_signals[COMPLETE_TEST_CASE_SIGNAL]
+    cut_test_suite_signals[COMPLETE_TEST_CASE]
         = g_signal_new("complete-test-case",
                        G_TYPE_FROM_CLASS(klass),
                        G_SIGNAL_RUN_LAST,
                        G_STRUCT_OFFSET(CutTestSuiteClass, complete_test_case),
                        NULL, NULL,
-                       g_cclosure_marshal_VOID__OBJECT,
-                       G_TYPE_NONE, 1, CUT_TYPE_TEST_CASE);
+                       _cut_marshal_VOID__OBJECT_BOOLEAN,
+                       G_TYPE_NONE, 2, CUT_TYPE_TEST_CASE, G_TYPE_BOOLEAN);
 
     g_type_class_add_private(gobject_class, sizeof(CutTestSuitePrivate));
 }
@@ -223,15 +225,15 @@ typedef struct _RunTestInfo
     gchar **test_names;
 } RunTestInfo;
 
-static gpointer
-run (gpointer data)
+static void
+run (gpointer data, gpointer user_data)
 {
     RunTestInfo *info = data;
     CutTestSuite *test_suite;
     CutTestCase *test_case;
     CutRunContext *run_context;
     gchar **test_names;
-    gboolean success;
+    gboolean *success = user_data;
 
     test_suite = info->test_suite;
     test_case = info->test_case;
@@ -239,23 +241,22 @@ run (gpointer data)
     test_names = info->test_names;
 
     g_signal_emit_by_name(test_suite, "start-test-case", test_case);
-    success = cut_test_case_run_with_filter(test_case, run_context,
-                                            (const gchar**)test_names);
-    g_signal_emit_by_name(test_suite, "complete-test-case", test_case);
+    if (!cut_test_case_run_with_filter(test_case, run_context,
+                                       (const gchar**)test_names))
+        *success =  FALSE;
+    g_signal_emit_by_name(test_suite, "complete-test-case", test_case, *success);
 
     g_object_unref(test_suite);
     g_object_unref(test_case);
     g_object_unref(run_context);
     g_strfreev(test_names);
     g_free(info);
-
-    return GINT_TO_POINTER(success);
 }
 
 static void
-run_with_thread (CutTestSuite *test_suite, CutTestCase *test_case,
-                 CutRunContext *run_context, const gchar **test_names,
-                 gboolean try_thread, GList **threads, gboolean *success)
+run_with_thread_support (CutTestSuite *test_suite, CutTestCase *test_case,
+                         CutRunContext *run_context, const gchar **test_names,
+                         GThreadPool *thread_pool, gboolean *success)
 {
     RunTestInfo *info;
     gboolean need_no_thread_run = TRUE;
@@ -269,24 +270,24 @@ run_with_thread (CutTestSuite *test_suite, CutTestCase *test_case,
     info->run_context = g_object_ref(run_context);
     info->test_names = g_strdupv((gchar **)test_names);
 
-    if (try_thread) {
-        GThread *thread;
+    if (thread_pool) {
         GError *error = NULL;
 
-        thread = g_thread_create(run, info, TRUE, &error);
+        g_thread_pool_push(thread_pool, info, &error);
         if (error) {
-            g_warning("%s(%d)", error->message, error->code);
+            gchar *inspected;
+
+            inspected = gcut_error_inspect(error);
+            g_warning("%s", inspected);
+            g_free(inspected);
             g_error_free(error);
         } else {
             need_no_thread_run = FALSE;
-            *threads = g_list_append(*threads, thread);
         }
     }
 
-    if (need_no_thread_run) {
-        if (!run(info))
-            *success = FALSE;
-    }
+    if (need_no_thread_run)
+        run(info, success);
 }
 
 #ifndef G_OS_WIN32
@@ -358,7 +359,7 @@ i_will_be_back_handler (int signum)
 
 static void
 emit_ready_signal (CutTestSuite *test_suite, GList *test_cases,
-                   const gchar **test_names)
+                   CutRunContext *run_context)
 {
     GList *node;
     guint n_test_cases, n_tests;
@@ -370,7 +371,8 @@ emit_ready_signal (CutTestSuite *test_suite, GList *test_cases,
         CutTestCase *test_case = node->data;
 
         n_test_cases++;
-        n_tests += cut_test_case_get_n_tests(test_case, test_names);
+        n_tests += cut_test_container_get_n_tests(CUT_TEST_CONTAINER(test_case),
+                                                  run_context);
     }
 
     g_signal_emit_by_name(test_suite, "ready", n_test_cases, n_tests);
@@ -382,15 +384,18 @@ cut_test_suite_run_test_cases (CutTestSuite *test_suite,
                                GList *test_cases, const gchar **test_names)
 {
     CutTestSuitePrivate *priv;
-    GList *node, *threads = NULL;
+    GList *node;
+    GThreadPool *thread_pool = NULL;
     GList *sorted_test_cases;
     gboolean try_thread;
     gboolean all_success = TRUE;
     gint signum;
 #ifndef G_OS_WIN32
     struct sigaction i_will_be_back_action;
-    struct sigaction previous_segv_action, previous_int_action;
+    struct sigaction previous_segv_action, previous_abort_action;
+    struct sigaction previous_int_action;
     gboolean set_segv_action = TRUE;
+    gboolean set_abort_action = TRUE;
     gboolean set_int_action = TRUE;
 #endif
 
@@ -406,53 +411,62 @@ cut_test_suite_run_test_cases (CutTestSuite *test_suite,
     i_will_be_back_action.sa_flags = 0;
     if (sigaction(SIGSEGV, &i_will_be_back_action, &previous_segv_action) == -1)
         set_segv_action = FALSE;
+    if (sigaction(SIGABRT, &i_will_be_back_action, &previous_abort_action) == -1)
+        set_abort_action = FALSE;
     if (sigaction(SIGINT, &i_will_be_back_action, &previous_int_action) == -1)
         set_int_action = FALSE;
 #endif
 
+    try_thread = cut_run_context_get_multi_thread(run_context);
+    if (try_thread) {
+        GError *error = NULL;
+        gint max_threads;
+
+        max_threads = cut_run_context_get_max_threads(run_context);
+        thread_pool = g_thread_pool_new(run, &all_success,
+                                        max_threads, FALSE, &error);
+        if (error) {
+            cut_utils_report_error(error);
+        }
+    }
+
     signum = setjmp(jump_buffer);
     switch (signum) {
       case 0:
-        cut_run_context_prepare_test_suite(run_context, test_suite);
-        emit_ready_signal(test_suite, sorted_test_cases, test_names);
-        g_signal_emit_by_name(CUT_TEST(test_suite), "start");
+        emit_ready_signal(test_suite, sorted_test_cases, run_context);
+        g_signal_emit_by_name(test_suite, "start", NULL);
 
         if (priv->warmup)
             priv->warmup();
 
-        try_thread = cut_run_context_get_multi_thread(run_context);
         for (node = sorted_test_cases; node; node = g_list_next(node)) {
             CutTestCase *test_case = node->data;
 
             if (!test_case)
                 continue;
             if (CUT_IS_TEST_CASE(test_case)) {
-                run_with_thread(test_suite, test_case, run_context, test_names,
-                                try_thread, &threads, &all_success);
+                run_with_thread_support(test_suite, test_case, run_context,
+                                        test_names, thread_pool, &all_success);
             } else {
                 g_warning("This object is not test case!");
             }
         }
 
-        for (node = threads; node; node = g_list_next(node)) {
-            GThread *thread = node->data;
-
-            if (!GPOINTER_TO_INT(g_thread_join(thread)))
-                all_success = FALSE;
-        }
+        if (thread_pool)
+            g_thread_pool_free(thread_pool, FALSE, TRUE);
 
         if (all_success) {
             CutTestResult *result;
             result = cut_test_result_new(CUT_TEST_RESULT_SUCCESS,
                                          NULL, NULL, NULL, test_suite, NULL,
-                                         NULL, NULL,
-                                         NULL, NULL, 0);
-            g_signal_emit_by_name(CUT_TEST(test_suite), "success", NULL, result);
+                                         NULL, NULL, NULL);
+            cut_test_emit_result_signal(CUT_TEST(test_suite), NULL, result);
             g_object_unref(result);
         }
         break;
 #ifndef G_OS_WIN32
       case SIGSEGV:
+      case SIGABRT:
         all_success = FALSE;
         g_signal_emit_by_name(CUT_TEST(test_suite), "crashed", backtrace);
         g_free(backtrace);
@@ -464,6 +478,8 @@ cut_test_suite_run_test_cases (CutTestSuite *test_suite,
 #ifndef G_OS_WIN32
     if (set_int_action)
         sigaction(SIGINT, &previous_int_action, NULL);
+    if (set_abort_action)
+        sigaction(SIGABRT, &previous_abort_action, NULL);
     if (set_segv_action)
         sigaction(SIGSEGV, &previous_segv_action, NULL);
 #endif
@@ -471,7 +487,7 @@ cut_test_suite_run_test_cases (CutTestSuite *test_suite,
     if (priv->cooldown)
         priv->cooldown();
 
-    g_signal_emit_by_name(CUT_TEST(test_suite), "complete");
+    g_signal_emit_by_name(CUT_TEST(test_suite), "complete", NULL, all_success);
 
     g_list_free(sorted_test_cases);
 
@@ -481,14 +497,14 @@ cut_test_suite_run_test_cases (CutTestSuite *test_suite,
 gboolean
 cut_test_suite_run (CutTestSuite *suite, CutRunContext *run_context)
 {
-    gchar **test_case_names;
-    gchar **test_names;
+    const gchar **test_case_names;
+    const gchar **test_names;
 
     test_case_names = cut_run_context_get_target_test_case_names(run_context);
     test_names = cut_run_context_get_target_test_names(run_context);
     return cut_test_suite_run_with_filter(suite, run_context,
-                                          (const gchar **)test_case_names,
-                                          (const gchar **)test_names);
+                                          test_case_names,
+                                          test_names);
 }
 
 gboolean
@@ -521,7 +537,7 @@ cut_test_suite_run_test (CutTestSuite *suite, CutRunContext *run_context,
 
 gboolean
 cut_test_suite_run_test_in_test_case (CutTestSuite *suite,
-                                      CutRunContext    *run_context,
+                                      CutRunContext *run_context,
                                       gchar        *test_name,
                                       gchar        *test_case_name)
 {
@@ -540,7 +556,7 @@ cut_test_suite_run_test_in_test_case (CutTestSuite *suite,
 
 gboolean
 cut_test_suite_run_with_filter (CutTestSuite *test_suite,
-                                CutRunContext   *run_context,
+                                CutRunContext *run_context,
                                 const gchar **test_case_names,
                                 const gchar **test_names)
 {
