@@ -20,8 +20,6 @@
 #ifndef __CUT_TEST_UTILS_H__
 #define __CUT_TEST_UTILS_H__
 
-#include <cutter/cut-public.h>
-
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -39,17 +37,100 @@ extern "C" {
  * The utilities work without GLib support.
  */
 
+/**
+ * cut_take:
+ * @object: the object to be owned by Cutter.
+ * @destroy_function: the destroy function for the object.
+ *
+ * Passes ownership of the object to Cutter and returns the
+ * object itself. @object is destroyed by @destroy_func.
+ *
+ * Returns: the object owned by Cutter. Don't free it.
+ *
+ * Since: 1.0.5
+ */
+#define cut_take(object, destroy_function)                      \
+    cut_test_context_take(cut_get_current_test_context(),       \
+                          (object), destroy_function)
+
+/**
+ * cut_take_memory:
+ * @memory: the memory to be owned by Cutter. (void *)
+ *
+ * Passes ownership of the memory to Cutter and returns the
+ * memory itself. @memory is destroyed by free().
+ *
+ * Returns: the memory owned by Cutter. Don't free it.
+ *
+ * Since: 1.0.5
+ */
+#define cut_take_memory(memory)                                         \
+    cut_test_context_take_memory(cut_get_current_test_context(),        \
+                                 (memory))
+
 /**
  * cut_take_string:
  * @string: the string to be owned by Cutter.
  *
- * Passes ownership of the string to Cutter and returns a
- * string that has same contents of @string.
+ * Passes ownership of the string to Cutter and returns the
+ * string itself.
  *
  * Returns: a string owned by Cutter. Don't free it.
  */
 #define cut_take_string(string)                                         \
-    cut_test_context_take_string(get_current_test_context(), (string))
+    cut_test_context_take_string(cut_get_current_test_context(),        \
+                                 (string))
+
+/**
+ * cut_take_strdup:
+ * @string: the string to be duplicated. (const char *)
+ *
+ * Duplicates the string, passes ownership of the
+ * duplicated string to Cutter and returns the duplicated
+ * string.
+ *
+ * Returns: a duplicated string owned by Cutter. Don't free it.
+ *
+ * Since: 1.0.5
+ */
+#define cut_take_strdup(string)                                         \
+    cut_test_context_take_strdup(cut_get_current_test_context(),        \
+                                 (string))
+
+/**
+ * cut_take_strndup:
+ * @string: the string to be duplicated. (const char *)
+ * @size: the number of bytes to duplicate. (size_t)
+ *
+ * Duplicates the first @size bytes of the string, passes
+ * ownership of the duplicated string to Cutter and returns
+ * the duplicated string. The duplicated string is always
+ * nul-terminated.
+ *
+ * Returns: the duplicated string owned by Cutter. Don't free it.
+ *
+ * Since: 1.0.5
+ */
+#define cut_take_strndup(string, size)                                  \
+    cut_test_context_take_strndup(cut_get_current_test_context(),       \
+                                  (string), (size))
+
+/**
+ * cut_take_memdup:
+ * @memory: the memory to be duplicated. (void *)
+ * @size: the number of bytes to duplicate. (size_t)
+ *
+ * Duplicates @size bytes of the memory, passes ownership of
+ * the duplicated memory to Cutter and returns the
+ * duplicated memory.
+ *
+ * Returns: the duplicated memory owned by Cutter. Don't free it.
+ *
+ * Since: 1.0.5
+ */
+#define cut_take_memdup(memory, size)                                   \
+    cut_test_context_take_memdup(cut_get_current_test_context(),        \
+                                 (memory), (size))
 
 /**
  * cut_take_printf:
@@ -62,8 +143,8 @@ extern "C" {
  * Returns: a formatted string owned by Cutter. Don't free it.
  */
 #define cut_take_printf(format, ...)                                    \
-    cut_test_context_take_printf(get_current_test_context(),            \
-                                 (format), __VA_ARGS__)
+    cut_test_context_take_printf(cut_get_current_test_context(),        \
+                                 format, __VA_ARGS__)
 
 /**
  * cut_take_string_array:
@@ -76,7 +157,8 @@ extern "C" {
  * Returns: an array of strings owned by Cutter. Don't free it.
  */
 #define cut_take_string_array(strings)                                  \
-    cut_test_context_take_string_array(get_current_test_context(), (strings))
+    cut_test_context_take_string_array(cut_get_current_test_context(),  \
+                                       (strings))
 
 /**
  * cut_take_diff:
@@ -118,7 +200,6 @@ extern "C" {
 #define cut_inspect_string_array(strings)                               \
     cut_take_string(cut_utils_inspect_string_array(strings))
 
-
 /**
  * cut_set_fixture_data_dir:
  * @path: a first element of the path to the fixture data directory.
@@ -132,7 +213,7 @@ extern "C" {
 #define cut_set_fixture_data_dir(path, ...) do  \
 {                                               \
     cut_test_context_set_fixture_data_dir(      \
-        get_current_test_context(),             \
+        cut_get_current_test_context(),         \
         path, ## __VA_ARGS__, NULL);            \
 } while (0)
 
@@ -146,13 +227,15 @@ extern "C" {
  * directory that is specified by cut_set_fixture_data_dir()
  * or the current directory.
  *
- * Returns: a path to the fixture data.
+ * Returns: a path to the fixture data. It should be freed
+ * when no longer needed.
  *
  * Since: 1.0.2
  */
-#define cut_build_fixture_data_path(path, ...)                          \
-    cut_test_context_build_fixture_data_path(get_current_test_context(), \
-                                             path, ## __VA_ARGS__, NULL)
+#define cut_build_fixture_data_path(path, ...)  \
+    cut_test_context_build_fixture_data_path(   \
+        cut_get_current_test_context(),         \
+        path, ## __VA_ARGS__, NULL)
 
 /**
  * cut_get_fixture_data_string:
@@ -164,15 +247,12 @@ extern "C" {
  * cut_build_fixture_data_path() shows how the fixture data
  * path is determined.
  *
- * Returns: a content of the fixture data as string.
+ * Returns: a content of the fixture data as string. (char *)
  *
  * Since: 1.0.2
  */
 #define cut_get_fixture_data_string(path, ...)                          \
-    cut_utils_get_fixture_data_string(get_current_test_context(),       \
-                                      __PRETTY_FUNCTION__,              \
-                                      __FILE__,                         \
-                                      __LINE__,                         \
+    cut_utils_get_fixture_data_string(cut_get_current_test_context(),   \
                                       path, ## __VA_ARGS__, NULL)
 
 /**
@@ -190,6 +270,36 @@ extern "C" {
         cut_take_string(                                                \
             cut_utils_build_path(path, ## __VA_ARGS__, NULL)))
 
+/**
+ * cut_equal_string:
+ * @string1: a string. (char *)
+ * @string2: a string. (char *)
+ *
+ * Compare @string1 to @string2. @string1 and/or @string2
+ * maybe NULL.
+ *
+ * Returns: TRUE if both @string1 and @string2 are NULL or
+ *          have the same contents; FALSE otherwise.
+ *
+ * Since: 1.0.5
+ */
+#define cut_equal_string(string1, string2)                              \
+    cut_utils_equal_string(string1, string2)
+
+/**
+ * cut_equal_double:
+ * @double1: a double value.
+ * @double2: a double value.
+ * @error: a double value that specifies error range.
+ *
+ * Compare @double1 to @double2 with @error range.
+ *
+ * Returns: TRUE if |@double1 - @double2| <= @error; FALSE otherwise.
+ *
+ * Since: 1.0.5
+ */
+#define cut_equal_double(double1, double2, error)       \
+    cut_utils_equal_double(double1, double2, error)
 
 #ifdef __cplusplus
 }
