@@ -40,11 +40,12 @@ struct _CutTestPrivate
     gchar *name;
     gchar *full_name;
     gchar *element_name;
-    CutTestData *test_data;
     CutTestFunction test_function;
     GTimer *timer;
+    GTimeVal start_time;
     gdouble elapsed;
     GHashTable *attributes;
+    gchar *base_directory;
 };
 
 enum
@@ -52,7 +53,8 @@ enum
     PROP_0,
     PROP_NAME,
     PROP_ELEMENT_NAME,
-    PROP_TEST_FUNCTION
+    PROP_TEST_FUNCTION,
+    PROP_BASE_DIRECTORY
 };
 
 enum
@@ -84,14 +86,13 @@ static void get_property   (GObject         *object,
                             GValue          *value,
                             GParamSpec      *pspec);
 
+static void         start        (CutTest  *test,
+                                  CutTestContext *test_context);
 static gdouble      get_elapsed  (CutTest  *test);
 static void         set_elapsed  (CutTest  *test, gdouble elapsed);
 static gboolean     run          (CutTest        *test,
                                   CutTestContext *test_context,
                                   CutRunContext  *run_context);
-static void         prepare      (CutTest        *test,
-                                  CutTestContext *test_context,
-                                  CutRunContext  *run_context);
 static gboolean     is_available (CutTest        *test,
                                   CutTestContext *test_context,
                                   CutRunContext  *run_context);
@@ -111,10 +112,10 @@ cut_test_class_init (CutTestClass *klass)
     gobject_class->set_property = set_property;
     gobject_class->get_property = get_property;
 
+    klass->start = start;
     klass->get_elapsed = get_elapsed;
     klass->set_elapsed = set_elapsed;
     klass->run = run;
-    klass->prepare = prepare;
     klass->is_available = is_available;
     klass->invoke = invoke;
 
@@ -138,14 +139,21 @@ cut_test_class_init (CutTestClass *klass)
                                 G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY);
     g_object_class_install_property(gobject_class, PROP_TEST_FUNCTION, spec);
 
+    spec = g_param_spec_string("base-directory",
+                               "Base directory",
+                               "The base directory of the test",
+                               NULL,
+                               G_PARAM_READWRITE);
+    g_object_class_install_property(gobject_class, PROP_BASE_DIRECTORY, spec);
+
     cut_test_signals[START]
         = g_signal_new ("start",
                         G_TYPE_FROM_CLASS (klass),
-                        G_SIGNAL_RUN_LAST,
+                        G_SIGNAL_RUN_FIRST,
                         G_STRUCT_OFFSET (CutTestClass, start),
                         NULL, NULL,
-                        g_cclosure_marshal_VOID__VOID,
-                        G_TYPE_NONE, 0);
+                        g_cclosure_marshal_VOID__OBJECT,
+                        G_TYPE_NONE, 1, CUT_TYPE_TEST_CONTEXT);
 
     cut_test_signals[PASS_ASSERTION]
         = g_signal_new ("pass-assertion",
@@ -217,22 +225,22 @@ cut_test_class_init (CutTestClass *klass)
                         CUT_TYPE_TEST_CONTEXT, CUT_TYPE_TEST_RESULT);
 
     cut_test_signals[COMPLETE]
-        = g_signal_new ("complete",
-                        G_TYPE_FROM_CLASS (klass),
-                        G_SIGNAL_RUN_LAST,
-                        G_STRUCT_OFFSET (CutTestClass, complete),
-                        NULL, NULL,
-                        g_cclosure_marshal_VOID__VOID,
-                        G_TYPE_NONE, 0);
+        = g_signal_new("complete",
+                       G_TYPE_FROM_CLASS (klass),
+                       G_SIGNAL_RUN_LAST,
+                       G_STRUCT_OFFSET (CutTestClass, complete),
+                       NULL, NULL,
+                       _cut_marshal_VOID__OBJECT_BOOLEAN,
+                       G_TYPE_NONE, 2, CUT_TYPE_TEST_CONTEXT, G_TYPE_BOOLEAN);
 
     cut_test_signals[CRASHED]
-        = g_signal_new ("crashed",
-                        G_TYPE_FROM_CLASS (klass),
-                        G_SIGNAL_RUN_LAST,
-                        G_STRUCT_OFFSET (CutTestClass, crashed),
-                        NULL, NULL,
-                        g_cclosure_marshal_VOID__STRING,
-                        G_TYPE_NONE, 1, G_TYPE_STRING);
+        = g_signal_new("crashed",
+                       G_TYPE_FROM_CLASS (klass),
+                       G_SIGNAL_RUN_LAST,
+                       G_STRUCT_OFFSET (CutTestClass, crashed),
+                       NULL, NULL,
+                       g_cclosure_marshal_VOID__STRING,
+                       G_TYPE_NONE, 1, G_TYPE_STRING);
 
     g_type_class_add_private(gobject_class, sizeof(CutTestPrivate));
 }
@@ -246,11 +254,11 @@ cut_test_init (CutTest *test)
 
     priv->test_function = NULL;
     priv->timer = NULL;
+    priv->start_time.tv_sec = 0;
+    priv->start_time.tv_usec = 0;
     priv->elapsed = -1.0;
     priv->attributes = g_hash_table_new_full(g_str_hash, g_str_equal,
                                              g_free, g_free);
-
-    priv->test_data = NULL;
 }
 
 static void
@@ -289,13 +297,13 @@ dispose (GObject *object)
         priv->attributes = NULL;
     }
 
-    if (priv->test_data) {
-        g_object_unref(priv->test_data);
-        priv->test_data = NULL;
-    }
-
     free_full_name(priv);
 
+    if (priv->base_directory) {
+        g_free(priv->base_directory);
+        priv->base_directory = NULL;
+    }
+
     G_OBJECT_CLASS(cut_test_parent_class)->dispose(object);
 }
 
@@ -321,6 +329,9 @@ set_property (GObject      *object,
       case PROP_TEST_FUNCTION:
         priv->test_function = g_value_get_pointer(value);
         break;
+      case PROP_BASE_DIRECTORY:
+        cut_test_set_base_directory(CUT_TEST(object), g_value_get_string(value));
+        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
@@ -345,6 +356,9 @@ get_property (GObject    *object,
       case PROP_TEST_FUNCTION:
         g_value_set_pointer(value, priv->test_function);
         break;
+      case PROP_BASE_DIRECTORY:
+        g_value_set_string(value, priv->base_directory);
+        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
@@ -368,9 +382,11 @@ cut_test_new_empty (void)
 }
 
 static void
-prepare (CutTest *test, CutTestContext *test_context, CutRunContext *run_context)
+start (CutTest *test, CutTestContext *test_context)
 {
-    cut_run_context_prepare_test(run_context, test);
+    GTimeVal value;
+    g_get_current_time(&value);
+    cut_test_set_start_time(test, &value);
 }
 
 static gboolean
@@ -400,9 +416,7 @@ run (CutTest *test, CutTestContext *test_context, CutRunContext *run_context)
     if (!klass->is_available(test, test_context, run_context))
         return FALSE;
 
-    klass->prepare(test, test_context, run_context);
-
-    g_signal_emit_by_name(test, "start");
+    g_signal_emit_by_name(test, "start", test_context);
 
     cut_test_context_set_jump(test_context, &jump_buffer);
     if (setjmp(jump_buffer) == 0) {
@@ -419,24 +433,22 @@ run (CutTest *test, CutTestContext *test_context, CutRunContext *run_context)
     if (success) {
         CutTestCase *test_case;
         CutTestIterator *test_iterator;
-        CutTestData *test_data = NULL;
         CutTestResult *result;
+        CutTestData *data = NULL;
 
         test_case = cut_test_context_get_test_case(test_context);
         test_iterator = cut_test_context_get_test_iterator(test_context);
-        if (cut_test_context_have_data(test_context))
-            test_data = cut_test_context_get_current_data(test_context);
+        if (CUT_IS_ITERATED_TEST(test))
+            data = cut_iterated_test_get_data(CUT_ITERATED_TEST(test));
         result = cut_test_result_new(CUT_TEST_RESULT_SUCCESS,
                                      test, test_iterator, test_case,
-                                     NULL, test_data,
-                                     NULL, NULL,
-                                     NULL, NULL, 0);
-        cut_test_result_set_elapsed(result, cut_test_get_elapsed(test));
-        g_signal_emit_by_name(test, "success", test_context, result);
+                                     NULL, data,
+                                     NULL, NULL, NULL);
+        cut_test_emit_result_signal(test, test_context, result);
         g_object_unref(result);
     }
 
-    g_signal_emit_by_name(test, "complete");
+    g_signal_emit_by_name(test, "complete", test_context, success);
 
     return success;
 }
@@ -470,12 +482,57 @@ cut_test_set_name (CutTest *test, const gchar *name)
         priv->name = g_strdup(name);
 }
 
+const gchar *
+cut_test_get_full_name (CutTest *test)
+{
+    CutTestPrivate *priv;
+    CutTestData *data = NULL;
+
+    priv = CUT_TEST_GET_PRIVATE(test);
+    if (priv->full_name)
+        return priv->full_name;
+
+    if (CUT_IS_ITERATED_TEST(test))
+        data = cut_iterated_test_get_data(CUT_ITERATED_TEST(test));
+
+    if (priv->name && data) {
+        priv->full_name = g_strconcat(priv->name,
+                                      " (",
+                                      cut_test_data_get_name(data),
+                                      ")",
+                                      NULL);
+    } else if (priv->name) {
+        priv->full_name = g_strdup(priv->name);
+    } else if (data) {
+        priv->full_name = g_strconcat(" (",
+                                      cut_test_data_get_name(data),
+                                      ")",
+                                      NULL);
+    }
+
+    return priv->full_name;
+}
+
 const gchar *
 cut_test_get_description (CutTest *test)
 {
     return cut_test_get_attribute(test, "description");
 }
 
+void
+cut_test_get_start_time (CutTest *test, GTimeVal *start_time)
+{
+    memcpy(start_time, &(CUT_TEST_GET_PRIVATE(test)->start_time),
+           sizeof(GTimeVal));
+}
+
+void
+cut_test_set_start_time (CutTest *test, GTimeVal *start_time)
+{
+    memcpy(&(CUT_TEST_GET_PRIVATE(test)->start_time), start_time,
+           sizeof(GTimeVal));
+}
+
 static gdouble
 get_elapsed (CutTest *test)
 {
@@ -528,15 +585,22 @@ cut_test_get_attributes (CutTest *test)
     return CUT_TEST_GET_PRIVATE(test)->attributes;
 }
 
-void
-cut_test_stop_timer (CutTest *test)
+const gchar *
+cut_test_get_base_directory (CutTest *test)
 {
-    CutTestPrivate *priv;
+    return CUT_TEST_GET_PRIVATE(test)->base_directory;
+}
 
-    priv = CUT_TEST_GET_PRIVATE(test);
+void
+cut_test_set_base_directory (CutTest *test, const gchar *base_directory)
+{
+    CutTestPrivate *priv = CUT_TEST_GET_PRIVATE(test);
 
-    if (priv->timer)
-        g_timer_stop(priv->timer);
+    if (priv->base_directory) {
+        g_free(priv->base_directory);
+        priv->base_directory = NULL;
+    }
+    priv->base_directory = g_strdup(base_directory);
 }
 
 gchar *
@@ -577,7 +641,7 @@ void
 cut_test_to_xml_string (CutTest *test, GString *string, guint indent)
 {
     CutTestPrivate *priv;
-    gchar *escaped, *elapsed;
+    gchar *escaped, *start_time, *elapsed;
     const gchar *description, *name;
     GHashTable *attributes;
 
@@ -597,6 +661,11 @@ cut_test_to_xml_string (CutTest *test, GString *string, guint indent)
         cut_utils_append_xml_element_with_value(string, indent + 2,
                                                 "description", description);
 
+    start_time = g_time_val_to_iso8601(&(priv->start_time));
+    cut_utils_append_xml_element_with_value(string, indent + 2,
+                                            "start-time", start_time);
+    g_free(start_time);
+
     elapsed = g_strdup_printf("%f", cut_test_get_elapsed(test));
     cut_utils_append_xml_element_with_value(string, indent + 2,
                                             "elapsed", elapsed);
@@ -616,6 +685,24 @@ cut_test_to_xml_string (CutTest *test, GString *string, guint indent)
     g_free(escaped);
 }
 
+void
+cut_test_emit_result_signal (CutTest *test,
+                             CutTestContext *test_context,
+                             CutTestResult *result)
+{
+    CutTestPrivate *priv;
+    const gchar *status_signal_name = NULL;
+    CutTestResultStatus status;
+
+    priv = CUT_TEST_GET_PRIVATE(test);
+    cut_test_result_set_start_time(result, &(priv->start_time));
+    cut_test_result_set_elapsed(result, cut_test_get_elapsed(test));
+
+    status = cut_test_result_get_status(result);
+    status_signal_name = cut_test_result_status_to_signal_name(status);
+    g_signal_emit_by_name(test, status_signal_name, test_context, result);
+}
+
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
 */
