@@ -21,9 +21,11 @@
 #define __CUTTER_H__
 
 #include <cutter/cut-version.h>
+#include <cutter/cut-features.h>
 
-#include <cutter/cut-hidden-definitions.h>
 #include <cutter/cut-assertions.h>
+#include <cutter/cut-multi-process.h>
+#include <cutter/cut-helper.h>
 #include <cutter/cut-experimental.h>
 
 #ifdef __cplusplus
@@ -251,10 +253,103 @@ void shutdown(void);
  * Since: 1.0.3
  */
 #define cut_add_data(first_data_name, ...)                      \
-    cut_test_context_add_data(get_current_test_context(),       \
+    cut_test_context_add_data(cut_get_current_test_context(),   \
                               first_data_name, ## __VA_ARGS__,  \
                               NULL)
 
+/**
+ * cut_set_attributes:
+ * @first_attribute_name: The first attribute name.
+ * @...: The value of the first attribute, followed
+ *       optionally by more name/value pairs.
+ *
+ * Sets attributes of the test.
+ *
+ * e.g.:
+ * |[
+ * #include <cutter.h>
+ *
+ * void attributes_repeat (void);
+ * void test_repeat (void);
+ *
+ * void
+ * attributes_repeat(void)
+ * {
+ *     cut_set_attributes("description", "a test for repeat function",
+ *                        "bug", "111",
+ *                        "priority", "high");
+ * }
+ *
+ * void
+ * test_repeat(void)
+ * {
+ *      cut_assert_equal_string_with_free("XXX", repeat("X", 3));
+ * }
+ * ]|
+ *
+ * Since: 1.0.4
+ */
+#define cut_set_attributes(first_attribute_name, ...)                   \
+    cut_test_context_set_attributes(cut_get_current_test_context(),     \
+                                    first_attribute_name,               \
+                                    ## __VA_ARGS__,                     \
+                                    NULL)
+
+/**
+ * cut_get_current_test_context:
+ *
+ * Returns the current test context. The current test
+ * context is a thread local object. It means that you don't
+ * need to care this if you don't create a new thread in your
+ * test. This is only needed to care if you create a new
+ * thread. You need to pass the current test context in your
+ * test thread to the current test context in a created
+ * thread.
+ *
+ * e.g.:
+ * |[
+ * int
+ * your_thread_function(void *data)
+ * {
+ *     CutTestContext *test_context = data;
+ *     cut_set_current_test_context(test_context);
+ *     ....
+ * }
+ *
+ * void
+ * run_your_thread(void)
+ * {
+ *     int result;
+ *     pthread_t your_thread;
+ *
+ *     result = pthread_create(&your_thread, NULL,
+ *                             your_thread_function,
+ *                             cut_get_current_test_context());
+ *     ...
+ * }
+ * ]|
+ *
+ * Returns: a #CutTestContext.
+ *
+ * Since: 1.0.4
+ */
+#define cut_get_current_test_context()          \
+    cut_test_context_current_get()
+
+/**
+ * cut_set_current_test_context:
+ * @test_context: the #CutTestContext to be the current test
+ * context.
+ *
+ * Set @test_context as the current test context. See
+ * cut_get_current_test_context() for more details.
+ *
+ * Since: 1.0.4
+ */
+#define cut_set_current_test_context(test_context)      \
+    cut_test_context_current_set(test_context)
+
+
 #ifdef __cplusplus
 }
 #endif
