@@ -1,5 +1,6 @@
 #!/usr/bin/env ruby
 
+require 'English'
 require 'rubygems'
 require 'mechanize'
 require 'logger'
@@ -30,7 +31,11 @@ def login(agent, user_name, password=nil)
   login_form.form_loginname = user_name
   login_form.form_pw = password || yield
 
-  page = agent.submit(login_form, login_form.buttons.first)
+  begin
+    page = agent.submit(login_form, login_form.buttons.first)
+  rescue WWW::Mechanize::ResponseCodeError
+    page = agent.get("https://sourceforge.net/my/")
+  end
   raise "login failed" unless /Personal Page/ =~ page.title
   page
 end
@@ -53,8 +58,22 @@ def go_file_releases_page(agent, project_page)
   agent.click(project_page.links.text(/\AFile Releases\z/))
 end
 
-def add_release(agent, file_releases_page, release_name)
-  add_release_link = file_releases_page.links.text(/\[Add Release\]/)
+def find_target_release_link(file_releases_page, package_name, label)
+  target_release_row = (file_releases_page / "tr").find do |row|
+    not (row / "input[@value = #{package_name}]").empty?
+  end
+  raise "can't find package ID" if /package_id=(\d+)/ !~ target_release_row.to_s
+  package_id = $1
+  release_links = file_releases_page.links.text(label)
+  release_links.find do |link|
+    /package_id=#{package_id}/ =~ link.href
+  end
+end
+
+def add_release(agent, file_releases_page, package_name, release_name)
+  add_release_link = find_target_release_link(file_releases_page,
+                                              package_name,
+                                              /\[Add Release\]/)
   add_release_page = agent.click(add_release_link)
 
   create_file_release_form = add_release_page.forms.last
@@ -64,12 +83,14 @@ def add_release(agent, file_releases_page, release_name)
                create_file_release_form.buttons.first)
 end
 
-def go_edit_release_page(agent, file_releases_page, release_name)
-  release_list_link = file_releases_page.links.text(/\[Edit Releases\]/)
-  release_list_page = agent.click(release_list_link)
+def go_edit_release_page(agent, file_releases_page, package_name, release_name)
+  edit_release_link = find_target_release_link(file_releases_page,
+                                               package_name,
+                                               /\[Edit Releases\]/)
+  release_list_page = agent.click(edit_release_link)
   edit_current_release_row = release_list_page / "td[text() ~= #{release_name}]"
   if edit_current_release_row.empty?
-    add_release(agent, file_releases_page, release_name)
+    add_release(agent, file_releases_page, package_name, release_name)
   else
     edit_release_link = (edit_current_release_row / "a")[0]
     edit_release_link = WWW::Mechanize::Page::Link.new(edit_release_link,
@@ -83,12 +104,27 @@ def extract_sections(file)
   File.read(file).gsub(/==+\n.*\n==+\n/, '').split(/.*\n^==.*\n\n\n*/)
 end
 
+def guess_target_index(file, default_index)
+  index = default_index
+  if /:(\d+)\Z/ =~ file
+    file = $PREMATCH
+    index = $1.to_i
+  end
+  [file, index]
+end
+
 def project_summary(readme)
-  extract_sections(readme)[4].split(/\n\n/)[0].chomp
+  readme, index = guess_target_index(readme, 4)
+  extract_sections(readme)[index].split(/\n\n/)[0].chomp
 end
 
 def latest_release_changes(news)
-  extract_sections(news)[1].chomp
+  news, index = guess_target_index(news, 1)
+  extract_sections(news)[index].chomp
+end
+
+def remove_rd_link_markup(text)
+  text.gsub(/\(\(<(.*?)(?:\|.*?)?>\)\)/m, '\1')
 end
 
 def update_release_info(agent, edit_release_page, news)
@@ -135,13 +171,15 @@ def submit_news(agent, submit_news_page, project_name, package_name,
   submit_news_form.summary = summary
   details = [project_summary(readme),
              latest_release_changes(news)].join("\n\n")
-  submit_news_form.details = details.gsub(/\n/, "\r\n")
+  submit_news_form.details = remove_rd_link_markup(details).gsub(/\n/, "\r\n")
   agent.submit(submit_news_form, submit_news_form.buttons.first)
 end
 
 def main(sf_user_name, project_name, package_name, release_name, file_name,
          readme, news)
-  agent = WWW::Mechanize.new
+  agent = WWW::Mechanize.new do |_agent|
+    # _agent.log = Logger.new(STDOUT)
+  end
   password = read_password("SF.net password for [#{sf_user_name}]: ")
   my_page = login(agent, sf_user_name, password)
 
@@ -149,7 +187,7 @@ def main(sf_user_name, project_name, package_name, release_name, file_name,
   file_releases_page = go_file_releases_page(agent, project_page)
   upload_file(agent, file_name, sf_user_name, password)
   edit_release_page = go_edit_release_page(agent, file_releases_page,
-                                           release_name)
+                                           package_name, release_name)
   edit_release_page = update_release_info(agent, edit_release_page, news)
   edit_release_page = register_file(agent, edit_release_page,
                                     File.basename(file_name))
