@@ -35,6 +35,7 @@
 #include <cutter/cut-test.h>
 #include <cutter/cut-test-suite.h>
 #include <cutter/cut-test-context.h>
+#include <cutter/cut-backtrace-entry.h>
 #include <cutter/cut-verbose-level.h>
 #include <cutter/cut-enum-types.h>
 
@@ -59,13 +60,6 @@
 
 #define CRASH_COLOR RED_BACK_COLOR WHITE_COLOR
 
-typedef struct _Error Error;
-struct _Error
-{
-    gchar *name;
-    gchar *detail;
-};
-
 typedef struct _CutConsoleUI CutConsoleUI;
 typedef struct _CutConsoleUIClass CutConsoleUIClass;
 
@@ -76,6 +70,8 @@ struct _CutConsoleUI
     gboolean      use_color;
     CutVerboseLevel verbose_level;
     GList        *errors;
+    gint          progress_row;
+    gint          progress_row_max;
 };
 
 struct _CutConsoleUIClass
@@ -87,7 +83,8 @@ enum
 {
     PROP_0,
     PROP_USE_COLOR,
-    PROP_VERBOSE_LEVEL
+    PROP_VERBOSE_LEVEL,
+    PROP_PROGRESS_ROW_MAX
 };
 
 static GType cut_type_console_ui = 0;
@@ -138,6 +135,13 @@ class_init (CutConsoleUIClass *klass)
                              CUT_VERBOSE_LEVEL_NORMAL,
                              G_PARAM_READWRITE);
     g_object_class_install_property(gobject_class, PROP_VERBOSE_LEVEL, spec);
+
+    spec = g_param_spec_int("progress-row-max",
+                            "Progress Row Max",
+                            "The max number of progress row",
+                            -1, G_MAXINT16, -1,
+                            G_PARAM_READWRITE);
+    g_object_class_install_property(gobject_class, PROP_PROGRESS_ROW_MAX, spec);
 }
 
 static void
@@ -146,6 +150,8 @@ init (CutConsoleUI *console)
     console->use_color = FALSE;
     console->verbose_level = CUT_VERBOSE_LEVEL_NORMAL;
     console->errors = NULL;
+    console->progress_row = 0;
+    console->progress_row_max = -1;
 }
 
 static void
@@ -232,28 +238,6 @@ CUT_MODULE_IMPL_INSTANTIATE (const gchar *first_property, va_list var_args)
     return g_object_new_valist(CUT_TYPE_CONSOLE_UI, first_property, var_args);
 }
 
-static Error *
-error_new (const gchar *name, const gchar *detail)
-{
-    Error *error;
-
-    error = g_slice_new(Error);
-    error->name = g_strdup(name);
-    error->detail = g_strdup(detail);
-
-    return error;
-}
-
-static void
-error_free (Error *error)
-{
-    if (error->name)
-        g_free(error->name);
-    if (error->detail)
-        g_free(error->detail);
-    g_slice_free(Error, error);
-}
-
 static void
 dispose (GObject *object)
 {
@@ -262,7 +246,7 @@ dispose (GObject *object)
     console = CUT_CONSOLE_UI(object);
 
     if (console->errors) {
-        g_list_foreach(console->errors, (GFunc)error_free, NULL);
+        g_list_foreach(console->errors, (GFunc)g_error_free, NULL);
         g_list_free(console->errors);
         console->errors = NULL;
     }
@@ -285,6 +269,9 @@ set_property (GObject      *object,
       case PROP_VERBOSE_LEVEL:
         console->verbose_level = g_value_get_enum(value);
         break;
+      case PROP_PROGRESS_ROW_MAX:
+        console->progress_row_max = g_value_get_int(value);
+        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
@@ -306,6 +293,9 @@ get_property (GObject    *object,
       case PROP_VERBOSE_LEVEL:
         g_value_set_enum(value, console->verbose_level);
         break;
+      case PROP_PROGRESS_ROW_MAX:
+        g_value_set_int(value, console->progress_row_max);
+        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
@@ -412,6 +402,21 @@ print_for_status (CutConsoleUI *console, CutTestResultStatus status,
     va_end(args);
 }
 
+static void
+print_progress (CutConsoleUI *console, CutTestResultStatus status,
+                gchar const *mark)
+{
+    print_with_color(console, status_to_color(status), "%s", mark);
+
+    console->progress_row += strlen(mark);
+    if (console->progress_row >= console->progress_row_max) {
+        if (console->progress_row_max != -1 &&
+            console->verbose_level < CUT_VERBOSE_LEVEL_VERBOSE)
+            g_print("\n");
+        console->progress_row = 0;
+    }
+}
+
 static void
 cb_start_test_suite (CutRunContext *run_context, CutTestSuite *test_suite,
                      CutConsoleUI *console)
@@ -509,12 +514,12 @@ cb_success_test (CutRunContext  *run_context,
 {
     if (console->verbose_level < CUT_VERBOSE_LEVEL_NORMAL)
         return;
-    print_for_status(console, CUT_TEST_RESULT_SUCCESS, ".");
+    print_progress(console, CUT_TEST_RESULT_SUCCESS, ".");
     fflush(stdout);
 }
 
 static void
-cb_notification_test (CutRunContext      *run_context,
+cb_notification_test (CutRunContext  *run_context,
                       CutTest        *test,
                       CutTestContext *test_context,
                       CutTestResult  *result,
@@ -522,12 +527,12 @@ cb_notification_test (CutRunContext      *run_context,
 {
     if (console->verbose_level < CUT_VERBOSE_LEVEL_NORMAL)
         return;
-    print_for_status(console, CUT_TEST_RESULT_NOTIFICATION, "N");
+    print_progress(console, CUT_TEST_RESULT_NOTIFICATION, "N");
     fflush(stdout);
 }
 
 static void
-cb_omission_test (CutRunContext      *run_context,
+cb_omission_test (CutRunContext  *run_context,
                   CutTest        *test,
                   CutTestContext *test_context,
                   CutTestResult  *result,
@@ -535,12 +540,12 @@ cb_omission_test (CutRunContext      *run_context,
 {
     if (console->verbose_level < CUT_VERBOSE_LEVEL_NORMAL)
         return;
-    print_for_status(console, CUT_TEST_RESULT_OMISSION, "O");
+    print_progress(console, CUT_TEST_RESULT_OMISSION, "O");
     fflush(stdout);
 }
 
 static void
-cb_pending_test (CutRunContext      *run_context,
+cb_pending_test (CutRunContext  *run_context,
                  CutTest        *test,
                  CutTestContext *test_context,
                  CutTestResult  *result,
@@ -548,12 +553,12 @@ cb_pending_test (CutRunContext      *run_context,
 {
     if (console->verbose_level < CUT_VERBOSE_LEVEL_NORMAL)
         return;
-    print_for_status(console, CUT_TEST_RESULT_PENDING, "P");
+    print_progress(console, CUT_TEST_RESULT_PENDING, "P");
     fflush(stdout);
 }
 
 static void
-cb_failure_test (CutRunContext      *run_context,
+cb_failure_test (CutRunContext  *run_context,
                  CutTest        *test,
                  CutTestContext *test_context,
                  CutTestResult  *result,
@@ -561,12 +566,12 @@ cb_failure_test (CutRunContext      *run_context,
 {
     if (console->verbose_level < CUT_VERBOSE_LEVEL_NORMAL)
         return;
-    print_for_status(console, CUT_TEST_RESULT_FAILURE, "F");
+    print_progress(console, CUT_TEST_RESULT_FAILURE, "F");
     fflush(stdout);
 }
 
 static void
-cb_error_test (CutRunContext      *run_context,
+cb_error_test (CutRunContext  *run_context,
                CutTest        *test,
                CutTestContext *test_context,
                CutTestResult  *result,
@@ -574,13 +579,14 @@ cb_error_test (CutRunContext      *run_context,
 {
     if (console->verbose_level < CUT_VERBOSE_LEVEL_NORMAL)
         return;
-    print_for_status(console, CUT_TEST_RESULT_ERROR, "E");
+    print_progress(console, CUT_TEST_RESULT_ERROR, "E");
     fflush(stdout);
 }
 
 static void
 cb_complete_test (CutRunContext *run_context, CutTest *test,
-                  CutTestContext *test_context, CutConsoleUI *console)
+                  CutTestContext *test_context, gboolean success,
+                  CutConsoleUI *console)
 {
     if (console->verbose_level < CUT_VERBOSE_LEVEL_VERBOSE)
         return;
@@ -592,16 +598,17 @@ cb_complete_test (CutRunContext *run_context, CutTest *test,
 static void
 cb_complete_iterated_test (CutRunContext *run_context,
                            CutIteratedTest *iterated_test,
-                           CutTestContext *test_context, CutConsoleUI *console)
+                           CutTestContext *test_context, gboolean success,
+                           CutConsoleUI *console)
 {
     cb_complete_test(run_context, CUT_TEST(iterated_test),
-                     test_context, console);
+                     test_context, success, console);
 }
 
 static void
 cb_complete_test_iterator (CutRunContext *run_context,
                            CutTestIterator *test_iterator,
-                           CutConsoleUI *console)
+                           gboolean success, CutConsoleUI *console)
 {
     if (console->verbose_level < CUT_VERBOSE_LEVEL_VERBOSE)
         return;
@@ -609,7 +616,15 @@ cb_complete_test_iterator (CutRunContext *run_context,
 
 static void
 cb_complete_test_case (CutRunContext *run_context, CutTestCase *test_case,
-                       CutConsoleUI *console)
+                       gboolean success, CutConsoleUI *console)
+{
+    if (console->verbose_level < CUT_VERBOSE_LEVEL_VERBOSE)
+        return;
+}
+
+static void
+cb_complete_test_suite (CutRunContext *run_context, CutTestSuite *test_suite,
+                        gboolean success, CutConsoleUI *console)
 {
     if (console->verbose_level < CUT_VERBOSE_LEVEL_VERBOSE)
         return;
@@ -650,11 +665,18 @@ print_results (CutConsoleUI *console, CutRunContext *run_context)
 
     i = 1;
     for (node = console->errors; node; node = g_list_next(node)) {
-        Error *error = node->data;
+        GError *error = node->data;
 
         g_print("\n%d) ", i);
         print_for_status(console, CUT_TEST_RESULT_ERROR,
-                         "SystemError: %s: %s", error->name, error->detail);
+                         "SystemError: %s:%d",
+                         g_quark_to_string(error->domain),
+                         error->code);
+        if (error->message) {
+            g_print("\n");
+            print_for_status(console, CUT_TEST_RESULT_ERROR,
+                             "%s", error->message);
+        }
         g_print("\n");
         i++;
     }
@@ -665,8 +687,7 @@ print_results (CutConsoleUI *console, CutRunContext *run_context)
         CutTestResult *result = node->data;
         CutTestResultStatus status;
         CutTest *test;
-        gchar *full_filename;
-        const gchar *filename;
+        const GList *node;
         const gchar *message;
         const gchar *name;
 
@@ -674,10 +695,6 @@ print_results (CutConsoleUI *console, CutRunContext *run_context)
         if (status == CUT_TEST_RESULT_SUCCESS)
             continue;
 
-        filename = cut_test_result_get_filename(result);
-        full_filename = cut_run_context_build_source_filename(run_context,
-                                                              filename);
-
         message = cut_test_result_get_message(result);
         name = cut_test_result_get_test_name(result);
         if (!name)
@@ -697,12 +714,18 @@ print_results (CutConsoleUI *console, CutRunContext *run_context)
             g_print("\n");
             print_for_status(console, status, "%s", message);
         }
-        g_print("\n%s:%d: %s()\n",
-                full_filename,
-                cut_test_result_get_line(result),
-                cut_test_result_get_function_name(result));
+        g_print("\n");
+        for (node = cut_test_result_get_backtrace(result);
+             node;
+             node = g_list_next(node)) {
+            CutBacktraceEntry *entry = node->data;
+            gchar *formatted_entry;
+
+            formatted_entry = cut_backtrace_entry_format(entry);
+            g_print("%s\n", formatted_entry);
+            g_free(formatted_entry);
+        }
         i++;
-        g_free(full_filename);
     }
 }
 
@@ -712,6 +735,7 @@ print_summary (CutConsoleUI *console, CutRunContext *run_context,
 {
     guint n_tests, n_assertions, n_successes, n_failures, n_errors;
     guint n_pendings, n_notifications, n_omissions;
+    gdouble pass_percentage;
     const gchar *color;
 
     n_tests = cut_run_context_get_n_tests(run_context);
@@ -726,21 +750,7 @@ print_summary (CutConsoleUI *console, CutRunContext *run_context,
     if (crashed) {
         color = CRASH_COLOR;
     } else {
-        CutTestResultStatus status;
-        if (n_errors > 0) {
-            status = CUT_TEST_RESULT_ERROR;
-        } else if (n_failures > 0) {
-            status = CUT_TEST_RESULT_FAILURE;
-        } else if (n_pendings > 0) {
-            status = CUT_TEST_RESULT_PENDING;
-        } else if (n_omissions > 0) {
-            status = CUT_TEST_RESULT_OMISSION;
-        } else if (n_notifications > 0) {
-            status = CUT_TEST_RESULT_NOTIFICATION;
-        } else {
-            status = CUT_TEST_RESULT_SUCCESS;
-        }
-        color = status_to_color(status);
+        color = status_to_color(cut_run_context_get_status(run_context));
     }
     print_with_color(console, color,
                      "%d test(s), %d assertion(s), %d failure(s), "
@@ -749,15 +759,19 @@ print_summary (CutConsoleUI *console, CutRunContext *run_context,
                      n_tests, n_assertions, n_failures, n_errors,
                      n_pendings, n_omissions, n_notifications);
     g_print("\n");
-    print_with_color(console, color,
-                     "%g%% passed",
-                     100.0 * ((gdouble)n_successes / (gdouble)n_tests));
+
+    if (n_tests == 0) {
+        pass_percentage = 0;
+    } else {
+        pass_percentage = 100.0 * ((gdouble)n_successes / (gdouble)n_tests);
+    }
+    print_with_color(console, color, "%g%% passed", pass_percentage);
     g_print("\n");
 }
 
 static void
-cb_complete_test_suite (CutRunContext *run_context, CutTestSuite *test_suite,
-                        CutConsoleUI *console)
+cb_complete_run (CutRunContext *run_context, gboolean success,
+                 CutConsoleUI *console)
 {
     gboolean crashed;
     CutVerboseLevel verbose_level;
@@ -783,27 +797,31 @@ cb_complete_test_suite (CutRunContext *run_context, CutTestSuite *test_suite,
     print_results(console, run_context);
 
     g_print("\n");
-    g_print("Finished in %f seconds",
-            cut_test_get_elapsed(CUT_TEST(test_suite)));
+    g_print("Finished in %f seconds (total: %f seconds)",
+            cut_run_context_get_elapsed(run_context),
+            cut_run_context_get_total_elapsed(run_context));
     g_print("\n\n");
 
     print_summary(console, run_context, crashed);
 }
 
 static void
-cb_error (CutRunContext *run_context, const gchar *name, const gchar *detail,
-          CutConsoleUI *console)
+cb_error (CutRunContext *run_context, GError *error, CutConsoleUI *console)
 {
-    print_with_color(console, status_to_color(CUT_TEST_RESULT_ERROR), "E");
-    fflush(stdout);
+    if (console->verbose_level >= CUT_VERBOSE_LEVEL_NORMAL) {
+        print_with_color(console, status_to_color(CUT_TEST_RESULT_ERROR), "E");
+        fflush(stdout);
+    }
 
-    console->errors = g_list_append(console->errors, error_new(name, detail));
+    console->errors = g_list_append(console->errors, g_error_copy(error));
 }
 
 static void
 cb_crashed (CutRunContext *run_context, const gchar *backtrace,
             CutConsoleUI *console)
 {
+    if (console->verbose_level < CUT_VERBOSE_LEVEL_NORMAL)
+        return;
     print_with_color(console, CRASH_COLOR, "!");
     fflush(stdout);
 }
@@ -827,11 +845,12 @@ connect_to_run_context (CutConsoleUI *console, CutRunContext *run_context)
     CONNECT(failure_test);
     CONNECT(error_test);
 
-    CONNECT(complete_iterated_test);
     CONNECT(complete_test);
+    CONNECT(complete_iterated_test);
     CONNECT(complete_test_iterator);
     CONNECT(complete_test_case);
     CONNECT(complete_test_suite);
+    CONNECT(complete_run);
 
     CONNECT(error);
     CONNECT(crashed);
@@ -865,6 +884,7 @@ disconnect_from_run_context (CutConsoleUI *console, CutRunContext *run_context)
     DISCONNECT(complete_test_iterator);
     DISCONNECT(complete_test_case);
     DISCONNECT(complete_test_suite);
+    DISCONNECT(complete_run);
 
     DISCONNECT(error);
     DISCONNECT(crashed);
