@@ -1,6 +1,6 @@
 /* -*- c-file-style: "gnu" -*- */
 
-#include <cutter.h>
+#include <gcutter.h>
 
 #include <stdio.h>
 #include <string.h>
@@ -157,7 +157,7 @@ error_post_parse_int (GOptionContext *context,
     error_test_post_parse_fail = TRUE;
 
   /* Set an error in the post hook */
-  g_set_error (error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE, NULL);
+  g_set_error (error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE, "XXX");
 
   return FALSE;
 }
@@ -214,7 +214,7 @@ error_test2_post_parse (GOptionContext *context,
     error_test_post_parse_fail = TRUE;
 
   /* Set an error in the post hook */
-  g_set_error (error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE, NULL);
+  g_set_error (error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE, "XXX");
 
   return FALSE;
 }
@@ -268,7 +268,7 @@ error_test3_post_parse (GOptionContext *context,
     error_test_post_parse_fail = TRUE;
 
   /* Set an error in the post hook */
-  g_set_error (error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE, NULL);
+  g_set_error (error, G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE, "XXX");
 
   return FALSE;
 }
@@ -423,8 +423,8 @@ test_arg_int64 (void)
   cut_assert (g_option_context_parse (context, &argc, &argv, &error));
 
   /* Last arg specified is the one that should be stored */
-  cut_assert_equal_int (G_GINT64_CONSTANT(4294967296), test_int64);
-  cut_assert_equal_int (G_GINT64_CONSTANT(0xfffffffff), test_int64_2);
+  gcut_assert_equal_int64 (G_GINT64_CONSTANT(4294967296), test_int64);
+  gcut_assert_equal_int64 (G_GINT64_CONSTANT(0xfffffffff), test_int64_2);
 }
 
 static gboolean
