@@ -10,8 +10,8 @@ export CUT_UI_MODULE_DIR=$BASE_DIR/../../module/ui/.libs
 export CUT_UI_FACTORY_MODULE_DIR=$BASE_DIR/../../module/ui/.libs
 export CUT_REPORT_MODULE_DIR=$BASE_DIR/../../module/report/.libs
 export CUT_REPORT_FACTORY_MODULE_DIR=$BASE_DIR/../../module/report/.libs
-export CUT_STREAMER_MODULE_DIR=$BASE_DIR/../../module/streamer/.libs
-export CUT_STREAMER_FACTORY_MODULE_DIR=$BASE_DIR/../../module/streamer/.libs
+export CUT_STREAM_MODULE_DIR=$BASE_DIR/../../module/stream/.libs
+export CUT_STREAM_FACTORY_MODULE_DIR=$BASE_DIR/../../module/stream/.libs
 
 CUTTER=$BASE_DIR/../../cutter/cutter
 if test x"$CUTTER_DEBUG" = x"yes"; then
