@@ -1,35 +1,30 @@
 #include <cutter.h>
 
 /* BUG entries */
-const char *bug_get_bug_id (void);
+const char *bug_bug_id (void);
 
 /* attribute */
-const CutTestAttribute *attributes_attribute (void);
+void attributes_attribute (void);
 const char *description_description (void);
 
 /* tests */
-void test_get_bug_id (void);
+void test_bug_id (void);
 void test_attribute (void);
 void test_description (void);
 
-const char *bug_get_bug_id (void) { return "1234567890"; }
+const char *bug_bug_id (void) { return "1234567890"; }
 void
-test_get_bug_id (void)
+test_bug_id (void)
 {
 }
 
 
-const CutTestAttribute *
+void
 attributes_attribute (void)
 {
-    static const CutTestAttribute attributes[] = {
-        {"bug", "1234"},
-        {"priority", "5678"},
-        {"bug", "9"},
-        {NULL, NULL}
-    };
-
-    return attributes;
+    cut_set_attributes("bug", "1234",
+                       "priority", "5678",
+                       "bug", "9");
 }
 
 void
