@@ -8,18 +8,23 @@
 #endif
 #include <errno.h>
 
-#include "cutter.h"
-#include "cut-test.h"
-#include "cut-test-result.h"
-#include "cut-utils.h"
-#include "cut-test-runner.h"
+#include <cutter.h>
+#include <cutter/cut-test.h>
+#include <cutter/cut-test-result.h>
+#include <cutter/cut-utils.h>
+#include <cutter/cut-test-runner.h>
 #include "lib/cuttest-assertions.h"
 
 void test_equal_int(void);
 void test_equal_string(void);
 void test_equal_string_with_diff(void);
+void test_equal_string_with_folded_diff(void);
 void test_equal_double(void);
 void test_operator(void);
+void test_operator_int(void);
+void test_operator_uint(void);
+void test_operator_double(void);
+void test_equal_memory (void);
 void test_equal_string_array (void);
 void test_null(void);
 void test_null_string(void);
@@ -52,6 +57,17 @@ static CutRunContext *run_context;
 static CutTestContext *test_context;
 static CutTestResult *test_result;
 
+static gint fail_line;
+
+#define MARK_FAIL(assertion) do                 \
+{                                               \
+    fail_line = __LINE__;                       \
+    assertion;                                  \
+} while (0)
+
+#define FAIL_LOCATION (cut_take_printf("%s:%d", __FILE__, fail_line))
+
+
 static void
 error_equal_string (void)
 {
@@ -64,44 +80,6 @@ error_equal_string_with_null (void)
     cut_assert_equal_string("", NULL);
 }
 
-static void
-stub_error_test_function (void)
-{
-    cut_error("This test should error");
-}
-
-static void
-stub_fail_test_function (void)
-{
-    cut_fail("This test should fail");
-}
-
-static void
-stub_pend_test_function (void)
-{
-    cut_pend("This test has been pending ever!");
-}
-
-static void
-stub_notify_test_function (void)
-{
-    cut_notify("This test has been notifying ever!");
-}
-
-static void
-stub_assert_message_test_function (void)
-{
-    cut_assert(FALSE, "The message of %s", "assertion");
-}
-
-static void
-stub_assert_message_with_format_string (void)
-{
-    cut_assert_equal_string("%s", "%d",
-                            "%s and %s have format string",
-                            "expected", "actual");
-}
-
 static void
 cb_collect_result (CutTest *test, CutTestContext *test_context,
                    CutTestResult *result, CutTestResult **output)
@@ -120,14 +98,15 @@ run (void)
 
     run_context = cut_test_runner_new();
 
-    test_context = cut_test_context_new(NULL, NULL, NULL, test);
+    test_context = cut_test_context_new(run_context, NULL, NULL, NULL, test);
     cut_test_context_set_fixture_data_dir(test_context,
                                           cuttest_get_base_dir(),
                                           "fixtures", "assertions", NULL);
-    original_test_context = get_current_test_context();
-    set_current_test_context(test_context);
-    success = cut_test_run(test, test_context, run_context);
-    set_current_test_context(original_test_context);
+    original_test_context = cut_get_current_test_context();
+    cut_set_current_test_context(test_context);
+    success = cut_test_runner_run_test(CUT_TEST_RUNNER(run_context),
+                                       test, test_context);
+    cut_set_current_test_context(original_test_context);
 
     return success;
 }
@@ -141,6 +120,7 @@ setup (void)
     run_context = NULL;
     test_context = NULL;
     test_result = NULL;
+    fail_line = 0;
 }
 
 void
@@ -191,10 +171,10 @@ test_equal_string_with_diff (void)
     g_signal_connect(test, "failure", G_CALLBACK(cb_collect_result),
                      &test_result);
     cut_assert_false(run());
-    cut_assert_test_result_summary(run_context, 0, 0, 0, 1, 0, 0, 0, 0);
+    cut_assert_test_result_summary(run_context, 1, 0, 0, 1, 0, 0, 0, 0);
     cut_assert_equal_string("<\"abc def ghi jkl\" == \"abc DEF ghi jkl\">\n"
                             "expected: <abc def ghi jkl>\n"
-                            " but was: <abc DEF ghi jkl>\n"
+                            "  actual: <abc DEF ghi jkl>\n"
                             "\n"
                             "diff:\n"
                             "- abc def ghi jkl\n"
@@ -204,6 +184,129 @@ test_equal_string_with_diff (void)
                             cut_test_result_get_system_message(test_result));
 }
 
+static void
+equal_string_with_folded_diff (void)
+{
+    cut_assert_equal_string("0123456789"
+                            "1123456789"
+                            "2123456789"
+                            "3123456789"
+                            "4123456789"
+                            "5123456789"
+                            "6123456789"
+                            "7123456789"
+                            "8123456789",
+
+                            "0000000000"
+                            "1123456789"
+                            "2123456789"
+                            "3123456789"
+                            "4123456789"
+                            "5123456789"
+                            "6123456789"
+                            "7123456789"
+                            "8123456789");
+}
+
+void
+test_equal_string_with_folded_diff (void)
+{
+    test = cut_test_new("assert-equal-string-with-folded-diff",
+                        equal_string_with_folded_diff);
+    g_signal_connect(test, "failure", G_CALLBACK(cb_collect_result),
+                     &test_result);
+    cut_assert_false(run());
+    cut_assert_test_result_summary(run_context, 1, 0, 0, 1, 0, 0, 0, 0);
+    cut_assert_equal_string("<"
+                            "\"0123456789\" "
+                            "\"1123456789\" "
+                            "\"2123456789\" "
+                            "\"3123456789\" "
+                            "\"4123456789\" "
+                            "\"5123456789\" "
+                            "\"6123456789\" "
+                            "\"7123456789\" "
+                            "\"8123456789\""
+                            " == "
+                            "\"0000000000\" "
+                            "\"1123456789\" "
+                            "\"2123456789\" "
+                            "\"3123456789\" "
+                            "\"4123456789\" "
+                            "\"5123456789\" "
+                            "\"6123456789\" "
+                            "\"7123456789\" "
+                            "\"8123456789\""
+                            ">\n"
+                            "expected: <"
+                            "0123456789"
+                            "1123456789"
+                            "2123456789"
+                            "3123456789"
+                            "4123456789"
+                            "5123456789"
+                            "6123456789"
+                            "7123456789"
+                            "8123456789"
+                            ">\n"
+                            "  actual: <"
+                            "0000000000"
+                            "1123456789"
+                            "2123456789"
+                            "3123456789"
+                            "4123456789"
+                            "5123456789"
+                            "6123456789"
+                            "7123456789"
+                            "8123456789"
+                            ">\n"
+                            "\n"
+                            "diff:\n"
+                            "- 0123456789"
+                            "1123456789"
+                            "2123456789"
+                            "3123456789"
+                            "4123456789"
+                            "5123456789"
+                            "6123456789"
+                            "7123456789"
+                            "8123456789\n"
+                            "?  ^^^^^^^^^\n"
+                            "+ 0000000000"
+                            "1123456789"
+                            "2123456789"
+                            "3123456789"
+                            "4123456789"
+                            "5123456789"
+                            "6123456789"
+                            "7123456789"
+                            "8123456789\n"
+                            "?  ^^^^^^^^^\n"
+                            "\n"
+                            "folded diff:\n"
+                            "- 0123456789"
+                            "1123456789"
+                            "2123456789"
+                            "3123456789"
+                            "4123456789"
+                            "5123456789"
+                            "6123456789"
+                            "71234567\n"
+                            "?  ^^^^^^^^^\n"
+                            "+ 0000000000"
+                            "1123456789"
+                            "2123456789"
+                            "3123456789"
+                            "4123456789"
+                            "5123456789"
+                            "6123456789"
+                            "71234567\n"
+                            "?  ^^^^^^^^^\n"
+                            "  89"
+                            "8123456789",
+                            cut_test_result_get_system_message(test_result));
+}
+
 void
 test_equal_double (void)
 {
@@ -223,6 +326,111 @@ test_operator (void)
     cut_assert_operator(TRUE, &&, TRUE);
 }
 
+static void
+stub_operator_int (void)
+{
+    cut_assert_operator_int(1, <, 2 + 3);
+    cut_assert_operator_int(2, ==, 1 + 1);
+    MARK_FAIL(cut_assert_operator_int(1 + 1, >=, 2 + 4));
+}
+
+void
+test_operator_int (void)
+{
+    test = cut_test_new("stub-operator-int", stub_operator_int);
+    cut_assert_not_null(test);
+
+    cut_assert_false(run());
+    cut_assert_test_result_summary(run_context, 1, 2, 0, 1, 0, 0, 0, 0);
+    cut_assert_test_result(run_context, 0, CUT_TEST_RESULT_FAILURE,
+                           "stub-operator-int", NULL,
+                           "expected: <1 + 1> >= <2 + 4>\n"
+                           "  actual: <2> >= <6>",
+                           FAIL_LOCATION,
+                           "stub_operator_int");
+}
+
+static void
+stub_operator_uint (void)
+{
+    cut_assert_operator_uint(1, <, 2 + 3);
+    cut_assert_operator_uint(2, ==, 1 + 1);
+    MARK_FAIL(cut_assert_operator_uint(1 + 1, >=, 2 + 4));
+}
+
+void
+test_operator_uint (void)
+{
+    test = cut_test_new("stub-operator-uint", stub_operator_uint);
+    cut_assert_not_null(test);
+
+    cut_assert_false(run());
+    cut_assert_test_result_summary(run_context, 1, 2, 0, 1, 0, 0, 0, 0);
+    cut_assert_test_result(run_context, 0, CUT_TEST_RESULT_FAILURE,
+                           "stub-operator-uint", NULL,
+                           "expected: <1 + 1> >= <2 + 4>\n"
+                           "  actual: <2> >= <6>",
+                           FAIL_LOCATION,
+                           "stub_operator_uint");
+}
+
+static void
+stub_operator_double (void)
+{
+    cut_assert_operator_double(1.2, <, 2.2 + 3.4);
+    MARK_FAIL(cut_assert_operator_double(1.1 + 1.1, >=, 2.2 + 4.4));
+}
+
+void
+test_operator_double (void)
+{
+    test = cut_test_new("stub-operator-double", stub_operator_double);
+    cut_assert_not_null(test);
+
+    cut_assert_false(run());
+    cut_assert_test_result_summary(run_context, 1, 1, 0, 1, 0, 0, 0, 0);
+    cut_assert_test_result(run_context, 0, CUT_TEST_RESULT_FAILURE,
+                           "stub-operator-double", NULL,
+                           "expected: <1.1 + 1.1> >= <2.2 + 4.4>\n"
+                           "  actual: <2.2> >= <6.6>",
+                           FAIL_LOCATION,
+                           "stub_operator_double");
+}
+
+static void
+stub_equal_memory (void)
+{
+    gchar expected[] = {0x00, 0x01, 0x02, 0x03, 0x04};
+    gchar actual[] = {0x00, 0x01, 0x02, 0x03, 0x04,
+                      0x12, 0x10, 0x0e, 0x0c, 0x0a};
+
+    cut_assert_equal_memory(expected, sizeof(expected),
+                            expected, sizeof(expected));
+    cut_assert_equal_memory(actual, sizeof(actual),
+                            actual, sizeof(actual));
+    cut_assert_equal_memory(expected, sizeof(expected),
+                            actual, sizeof(expected));
+    MARK_FAIL(cut_assert_equal_memory(expected, sizeof(expected),
+                                      actual, sizeof(actual)));
+}
+
+void
+test_equal_memory (void)
+{
+    test = cut_test_new("stub-equal-memory", stub_equal_memory);
+    cut_assert_false(run());
+    cut_assert_test_result_summary(run_context, 1, 3, 0, 1, 0, 0, 0, 0);
+    cut_assert_test_result(run_context, 0, CUT_TEST_RESULT_FAILURE,
+                           "stub-equal-memory", NULL,
+                           "<expected(size: sizeof(expected)) == "
+                           "actual(size: sizeof(actual))>\n"
+                           "expected: <0x00 0x01 0x02 0x03 0x04 (size: 5)>\n"
+                           "  actual: <0x00 0x01 0x02 0x03 0x04 "
+                           "0x12 0x10 0x0e 0x0c 0x0a (size: 10)>",
+                           FAIL_LOCATION,
+                           "stub_equal_memory");
+}
+
 void
 test_equal_string_array (void)
 {
@@ -255,24 +463,38 @@ test_error_equal_string (void)
     cut_assert_false(run());
 }
 
+static void
+stub_error (void)
+{
+    MARK_FAIL(cut_error("This test should error"));
+}
+
 void
 test_error (void)
 {
-    test = cut_test_new("stub-error-test", stub_error_test_function);
+    test = cut_test_new("stub-error-test", stub_error);
     cut_assert_not_null(test);
 
     cut_assert_false(run());
-    cut_assert_test_result_summary(run_context, 0, 0, 0, 0, 1, 0, 0, 0);
+    cut_assert_test_result_summary(run_context, 1, 0, 0, 0, 1, 0, 0, 0);
     cut_assert_test_result(run_context, 0, CUT_TEST_RESULT_ERROR,
                            "stub-error-test",
                            "This test should error", NULL,
-                           "stub_error_test_function");
+                           FAIL_LOCATION,
+                           "stub_error");
+}
+
+static void
+stub_pend (void)
+{
+    cut_pend("This test has been pending ever!");
 }
 
 void
 test_pending (void)
 {
-    test = cut_test_new("stub-pend-test", stub_pend_test_function);
+    /* FIXME: use cut_assert_test_result_summary() */
+    test = cut_test_new("stub-pend-test", stub_pend);
     cut_assert_not_null(test);
 
     g_signal_connect(test, "pending", G_CALLBACK(cb_collect_result),
@@ -289,15 +511,22 @@ test_pending (void)
                          cut_test_result_get_status(test_result));
 }
 
+static void
+stub_notify (void)
+{
+    cut_notify("This test has been notifying ever!");
+}
+
 void
 test_notification (void)
 {
-    test = cut_test_new("stub-notify-test", stub_notify_test_function);
+    /* FIXME: use cut_assert_test_result_summary() */
+    test = cut_test_new("stub-notify-test", stub_notify);
     cut_assert_not_null(test);
 
     g_signal_connect(test, "notification", G_CALLBACK(cb_collect_result),
                      &test_result);
-    cut_assert_true(run(), "cut_notify() did return FALSE!");
+    cut_assert_true(run(), "cut_notify() did return TRUE!");
     g_signal_handlers_disconnect_by_func(test,
                                          G_CALLBACK(cb_collect_result),
                                          &test_result);
@@ -309,10 +538,17 @@ test_notification (void)
                          cut_test_result_get_status(test_result));
 }
 
+static void
+stub_fail (void)
+{
+    cut_fail("This test should fail. %s", "Exactly!");
+}
+
 void
 test_fail (void)
 {
-    test = cut_test_new("stub-fail-test", stub_fail_test_function);
+    /* FIXME: use cut_assert_test_result_summary() */
+    test = cut_test_new("stub-fail-test", stub_fail);
     cut_assert_not_null(test);
 
     g_signal_connect(test, "failure", G_CALLBACK(cb_collect_result),
@@ -326,13 +562,23 @@ test_fail (void)
                         "since \"failure\" signal was not emitted.");
     cut_assert_equal_int(CUT_TEST_RESULT_FAILURE,
                          cut_test_result_get_status(test_result));
+
+    cut_assert_equal_string("This test should fail. Exactly!",
+                            cut_test_result_get_message(test_result));
+}
+
+static void
+stub_assert_message (void)
+{
+    cut_assert(FALSE, "The message of %s", "assertion");
 }
 
 void
 test_assert_message (void)
 {
+    /* FIXME: use cut_assert_test_result_summary() */
     test = cut_test_new("stub-assertion-message-test",
-                        stub_assert_message_test_function);
+                        stub_assert_message);
     cut_assert_not_null(test);
 
     g_signal_connect(test, "failure", G_CALLBACK(cb_collect_result),
@@ -350,9 +596,18 @@ test_assert_message (void)
                             cut_test_result_get_user_message(test_result));
 }
 
+static void
+stub_assert_message_with_format_string (void)
+{
+    cut_assert_equal_string("%s", "%d",
+                            "%s and %s have format string",
+                            "expected", "actual");
+}
+
 void
 test_assert_message_with_format_string (void)
 {
+    /* FIXME: use cut_assert_test_result_summary() */
     test = cut_test_new("stub-assert-message-with-string",
                         stub_assert_message_with_format_string);
     cut_assert_not_null(test);
@@ -370,7 +625,7 @@ test_assert_message_with_format_string (void)
                          cut_test_result_get_status(test_result));
     cut_assert_equal_string("<\"%s\" == \"%d\">\n"
                             "expected: <%s>\n"
-                            " but was: <%d>",
+                            "  actual: <%d>",
                             cut_test_result_get_system_message(test_result));
     cut_assert_equal_string("expected and actual have format string",
                             cut_test_result_get_user_message(test_result));
@@ -391,12 +646,10 @@ test_assert_equal_function (void)
     cut_assert_true(compare_function_is_called);
 }
 
-static gboolean
+static void
 fail_in_nested_function (void)
 {
     cut_fail("Fail from nested function");
-
-    return FALSE;
 }
 
 static void
@@ -409,6 +662,7 @@ just_call_fail_in_nested_function (void)
 void
 test_failure_from_nested_function (void)
 {
+    /* FIXME: use cut_assert_test_result_summary() */
     CutTestResult *result;
 
     test = cut_test_new("fail from nested function",
@@ -418,7 +672,7 @@ test_failure_from_nested_function (void)
     cut_assert_false(run());
 
     cut_assert_not_null(run_context);
-    cut_assert_test_result_summary(run_context, 0, 0, 0, 1, 0, 0, 0, 0);
+    cut_assert_test_result_summary(run_context, 1, 0, 0, 1, 0, 0, 0, 0);
 
     result = cut_run_context_get_results(run_context)->data;
     cut_assert_equal_string("Fail from nested function",
@@ -437,7 +691,7 @@ test_null_string (void)
 {
     test = cut_test_new("assert-null-string", null_string_assertions);
     cut_assert_false(run());
-    cut_assert_test_result_summary(run_context, 0, 1, 0, 1, 0, 0, 0, 0);
+    cut_assert_test_result_summary(run_context, 1, 1, 0, 1, 0, 0, 0, 0);
 }
 
 static void
@@ -455,7 +709,7 @@ test_equal_string_with_free (void)
     test = cut_test_new("assert-string-equal-string-with-free",
                         equal_string_with_free_assertions);
     cut_assert_false(run());
-    cut_assert_test_result_summary(run_context, 0, 3, 0, 1, 0, 0, 0, 0);
+    cut_assert_test_result_summary(run_context, 1, 3, 0, 1, 0, 0, 0, 0);
 }
 
 static void
@@ -473,7 +727,7 @@ test_assert_errno (void)
 {
     test = cut_test_new("assert-errno-for-eacces", assert_errno_for_eacces);
     cut_assert_false(run());
-    cut_assert_test_result_summary(run_context, 0, 1, 0, 1, 0, 0, 0, 0);
+    cut_assert_test_result_summary(run_context, 1, 1, 0, 1, 0, 0, 0, 0);
 }
 
 static void
@@ -489,7 +743,7 @@ test_omit (void)
 {
     test = cut_test_new("omit-test", omit_test);
     cut_assert_true(run());
-    cut_assert_test_result_summary(run_context, 0, 1, 1, 0, 0, 0, 0, 1);
+    cut_assert_test_result_summary(run_context, 1, 1, 1, 0, 0, 0, 0, 1);
 }
 
 static void
@@ -501,7 +755,7 @@ path_exist_test (void)
     fd = g_file_open_tmp(NULL, &tmp_file_name, &error);
     if (fd == -1) {
         const gchar *message;
-        message = cut_take_string(g_strdup(error->message));
+        message = cut_take_string(gcut_error_inspect(error));
         g_error_free(error);
         cut_error("can't create temporary file: %s", message);
     }
@@ -517,7 +771,7 @@ test_path_exist (void)
 {
     test = cut_test_new("path-exist-test", path_exist_test);
     cut_assert_false(run());
-    cut_assert_test_result_summary(run_context, 0, 1, 0, 1, 0, 0, 0, 0);
+    cut_assert_test_result_summary(run_context, 1, 1, 0, 1, 0, 0, 0, 0);
 }
 
 static void
@@ -526,16 +780,15 @@ path_not_exist_test (void)
     gint fd;
     GError *error = NULL;
 
+    fd = g_file_open_tmp(NULL, &tmp_file_name, &error);
+    cut_assert_g_error(error);
+    close(fd);
+
+    g_remove(tmp_file_name);
     cut_assert_path_not_exist(tmp_file_name);
 
-    fd = g_file_open_tmp(NULL, &tmp_file_name, &error);
-    if (fd == -1) {
-        const gchar *message;
-        message = cut_take_string(g_strdup(error->message));
-        g_error_free(error);
-        cut_error("can't create temporary file: %s", message);
-    }
-    close(fd);
+    g_file_set_contents(tmp_file_name, "XXX", -1, &error);
+    cut_assert_g_error(error);
     cut_assert_path_not_exist(tmp_file_name);
 }
 
@@ -544,7 +797,7 @@ test_path_not_exist (void)
 {
     test = cut_test_new("path-not-exist-test", path_not_exist_test);
     cut_assert_false(run());
-    cut_assert_test_result_summary(run_context, 0, 1, 0, 1, 0, 0, 0, 0);
+    cut_assert_test_result_summary(run_context, 1, 3, 0, 1, 0, 0, 0, 0);
 }
 
 static void
@@ -559,7 +812,7 @@ test_match (void)
 {
     test = cut_test_new("match-test", match_test);
     cut_assert_false(run());
-    cut_assert_test_result_summary(run_context, 0, 1, 0, 1, 0, 0, 0, 0);
+    cut_assert_test_result_summary(run_context, 1, 1, 0, 1, 0, 0, 0, 0);
 }
 
 static void
@@ -576,7 +829,7 @@ test_equal_pointer (void)
 {
     test = cut_test_new("equal-pointer-test", equal_pointer_test);
     cut_assert_false(run());
-    cut_assert_test_result_summary(run_context, 0, 1, 0, 1, 0, 0, 0, 0);
+    cut_assert_test_result_summary(run_context, 1, 1, 0, 1, 0, 0, 0, 0);
 }
 
 static void
@@ -593,7 +846,7 @@ test_equal_fixture_data_string (void)
     test = cut_test_new("equal-fixture-data-sting-test",
                         equal_fixture_data_string_test);
     cut_assert_false(run());
-    cut_assert_test_result_summary(run_context, 0, 2, 0, 1, 0, 0, 0, 0);
+    cut_assert_test_result_summary(run_context, 1, 2, 0, 1, 0, 0, 0, 0);
 }
 
 static void
@@ -608,7 +861,7 @@ test_equal_fixture_data_string_without_file (void)
     test = cut_test_new("equal-fixture-data-sting-test-without-file",
                         equal_fixture_data_string_test_without_file);
     cut_assert_false(run());
-    cut_assert_test_result_summary(run_context, 0, 0, 0, 0, 1, 0, 0, 0);
+    cut_assert_test_result_summary(run_context, 1, 0, 0, 0, 1, 0, 0, 0);
 }
 
 static void
@@ -624,11 +877,12 @@ error_errno (void)
 void
 test_error_errno (void)
 {
+    /* FIXME: use cut_assert_test_result_summary() */
     CutTestResult *result;
 
     test = cut_test_new("error-errno", error_errno);
     cut_assert_false(run());
-    cut_assert_test_result_summary(run_context, 0, 0, 0, 0, 1, 0, 0, 0);
+    cut_assert_test_result_summary(run_context, 1, 0, 0, 0, 1, 0, 0, 0);
 
     result = cut_run_context_get_results(run_context)->data;
     cut_assert_equal_string("Should error",
@@ -636,5 +890,5 @@ test_error_errno (void)
 }
 
 /*
-vi:nowrap:ai:expandtab:sw=4
+vi:ts=4:nowrap:ai:expandtab:sw=4
 */
