@@ -1,3 +1,7 @@
+/* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+
+#include <string.h>
+
 #include <cutter.h>
 #include <cutter/cut-loader.h>
 #include <cutter/cut-test-runner.h>
@@ -61,7 +65,8 @@ run (void)
     cut_assert_not_null(test_case);
 
     run_context = cut_test_runner_new();
-    return cut_test_case_run(test_case, run_context);
+    return cut_test_runner_run_test_case(CUT_TEST_RUNNER(run_context),
+                                         test_case);
 }
 
 static gchar *expected_functions[] = {
@@ -90,14 +95,19 @@ test_load_function (void)
     const gchar *target_test_names[] = {"/.*/", NULL};
     gint i;
 
-    loader = loader_new("test", "stub_loader_test." G_MODULE_SUFFIX);
+    loader = loader_new("test", "stub-loader-test." G_MODULE_SUFFIX);
     test_case = cut_loader_load_test_case(loader);
     cut_assert(test_case);
-    cut_assert_equal_int(4, cut_test_case_get_n_tests(test_case, NULL));
-    cut_assert_equal_int(4, cut_test_case_get_n_tests(test_case,
-                                                      target_test_names));
 
     container = CUT_TEST_CONTAINER(test_case);
+    cut_assert_equal_int(4,
+                         cut_test_container_get_n_tests(container, NULL));
+
+    run_context = cut_test_runner_new();
+    cut_run_context_set_target_test_names(run_context, target_test_names);
+    cut_assert_equal_int(4,
+                         cut_test_container_get_n_tests(container, run_context));
+
     tests = (GList *)cut_test_container_get_children(container);
     cut_assert(tests);
 
@@ -121,7 +131,7 @@ test_load_startup_and_shutdown_function (void)
     CutStartupFunction startup_function = NULL;
     CutShutdownFunction shutdown_function = NULL;
 
-    loader = loader_new("test", "stub_loader_test." G_MODULE_SUFFIX);
+    loader = loader_new("test", "stub-loader-test." G_MODULE_SUFFIX);
     test_case = cut_loader_load_test_case(loader);
     cut_assert(test_case);
 
@@ -136,7 +146,7 @@ test_load_startup_and_shutdown_function (void)
 void
 test_fail_to_load (void)
 {
-    loader = loader_new("module", "cannot_load_module." G_MODULE_SUFFIX);
+    loader = loader_new("module", "cannot-load-module." G_MODULE_SUFFIX);
     cut_assert_not_null(loader);
 
     test_case = cut_loader_load_test_case(loader);
@@ -147,18 +157,20 @@ void
 test_load_test_iterator (void)
 {
     const GList *tests;
+    CutTestContainer *container;
     CutTestIterator *test_iterator;
     CutIteratedTestFunction iterated_test_function = NULL;
     CutDataSetupFunction data_setup_function = NULL;
 
-    loader = loader_new("iterator", "success_iterated_test." G_MODULE_SUFFIX);
+    loader = loader_new("iterator", "success-iterated-test." G_MODULE_SUFFIX);
     cut_assert_not_null(loader);
 
     test_case = cut_loader_load_test_case(loader);
     cut_assert_not_null(test_case);
-    cut_assert_equal_int(1, cut_test_case_get_n_tests(test_case, NULL));
+    container = CUT_TEST_CONTAINER(test_case);
+    cut_assert_equal_int(1, cut_test_container_get_n_tests(container, NULL));
 
-    tests = cut_test_container_get_children(CUT_TEST_CONTAINER(test_case));
+    tests = cut_test_container_get_children(container);
     cut_assert_not_null(tests);
 
     test_iterator = tests->data;
