@@ -6,30 +6,38 @@ const char   *cut_get_cutter_command_path (void);
 
 void test_cutter_command_path (void);
 
-static const gchar *original_cutter_command_path = NULL;
+static gchar *original_cutter_command_path;
 
 void
 setup (void)
 {
-    original_cutter_command_path = cut_get_cutter_command_path();
+    original_cutter_command_path = g_strdup(cut_get_cutter_command_path());
 }
 
 void
 teardown (void)
 {
-    if (original_cutter_command_path)
+    if (original_cutter_command_path) {
         cut_set_cutter_command_path(original_cutter_command_path);
+        g_free(original_cutter_command_path);
+    }
 }
 
 void
 test_cutter_command_path (void)
 {
+    const gchar *exist_command;
     const gchar *absolute_path, *relative_cutter, *absolute_cutter;
 
     cut_assert_not_null(cut_get_cutter_command_path());
 
-    cut_set_cutter_command_path("dir");
-    cut_assert_equal_string(cut_take_string(g_find_program_in_path("dir")),
+#ifdef G_OS_WIN32
+    exist_command = "notepad";
+#else
+    exist_command = "ls";
+#endif
+    cut_set_cutter_command_path(exist_command);
+    cut_assert_equal_string(cut_take_string(g_find_program_in_path(exist_command)),
                             cut_get_cutter_command_path());
 
     absolute_path =
