@@ -1,12 +1,15 @@
+/* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
 #include <cutter.h>
+#include <gcutter.h>
 #include <cutter/cut-pipeline.h>
 #include <cutter/cut-runner.h>
 
-#include "lib/cuttest-utils.h"
+#include "lib/cuttest-assertions.h"
 
 #ifdef HAVE_UNISTD_H
 #  include <unistd.h>
@@ -17,37 +20,74 @@ void test_signal (gconstpointer data);
 void data_count (void);
 void test_count (gconstpointer data);
 
-static CutRunContext *pipeline;
+static GHashTable *pipelines;
+static GMutex *fixture_mutex, *run_mutex;
 
-static gchar *
-build_test_dir (const gchar *result)
+#define build_test_dir(dir, ...)                \
+    g_build_filename(cuttest_get_base_dir(),    \
+                     "fixtures",                \
+                     "pipeline",                \
+                     dir,                       \
+                     ## __VA_ARGS__,            \
+                     NULL)
+
+void
+startup (void)
 {
-    return g_build_filename(cuttest_get_base_dir(),
-                            "fixtures",
-                            "pipeline",
-                            result,
-                            NULL);
+    pipelines = g_hash_table_new_full(g_direct_hash, g_direct_equal,
+                                      NULL, g_object_unref);
+    fixture_mutex = g_mutex_new();
+    run_mutex = g_mutex_new();
 }
 
+void
+shutdown (void)
+{
+    g_mutex_free(run_mutex);
+    g_mutex_free(fixture_mutex);
+    g_hash_table_unref(pipelines);
+}
+
+#define pipeline (g_hash_table_lookup(pipelines, g_thread_self()))
+
 void
 setup (void)
 {
-    pipeline = cut_pipeline_new();
+    g_mutex_lock(fixture_mutex);
+    g_hash_table_insert(pipelines, g_thread_self(), cut_pipeline_new());
+    g_mutex_unlock(fixture_mutex);
 }
 
 void
 teardown (void)
 {
-    if (pipeline)
-        g_object_unref(pipeline);
+    g_mutex_lock(fixture_mutex);
+    g_hash_table_remove(pipelines, g_thread_self());
+    g_mutex_unlock(fixture_mutex);
+}
+
+static void
+report_error (CutRunContext *context, GError *error, gpointer user_data)
+{
+    g_mutex_unlock(run_mutex);
+    gcut_assert_error(error, "Pipeline Error");
 }
 
 static gboolean
 run (const gchar *test_dir)
 {
-    cut_run_context_set_test_directory(pipeline, test_dir);
+    gboolean success;
 
-    return cut_runner_run(CUT_RUNNER(pipeline));
+    cut_run_context_set_test_directory(pipeline, test_dir);
+    cut_run_context_set_source_directory(pipeline, test_dir);
+
+    g_signal_connect(pipeline, "error", G_CALLBACK(report_error), NULL);
+
+    g_mutex_lock(run_mutex);
+    success = cut_run_context_start(pipeline);
+    g_mutex_unlock(run_mutex);
+
+    return success;
 }
 
 typedef struct _SignalTestData
@@ -176,6 +216,8 @@ data_count (void)
     ADD_DATA(pending, pendings, FALSE);
     ADD_DATA(omission, omissions, TRUE);
 
+    ADD_DATA(setup-error, errors, FALSE);
+
 #undef ADD_DATA
 }
 
