@@ -1,7 +1,16 @@
-#include "cutter.h"
-#include <cutter/cut-process.h>
+/* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 
-#include <unistd.h>
+#ifdef HAVE_CONFIG_H
+#  include "../cutter/config.h"
+#endif
+
+#include <errno.h>
+#ifdef HAVE_UNISTD_H
+#  include <unistd.h>
+#endif
+
+#include <cutter.h>
+#include <cutter/cut-process.h>
 
 void test_fork (void);
 
