@@ -1,7 +1,8 @@
-#include "cutter.h"
-#include "cut-test-runner.h"
-#include "cut-listener.h"
-#include "cut-report.h"
+#include <cutter.h>
+#include <cutter/cut-test-runner.h>
+#include <cutter/cut-listener.h>
+#include <cutter/cut-report.h>
+#include "lib/cuttest-utils.h"
 
 void test_report_success (void);
 void test_report_failure (void);
@@ -48,7 +49,7 @@ stub_notification_test (void)
 void
 setup (void)
 {
-    gchar *test_names[] = {"/.*/", NULL};
+    const gchar *test_names[] = {"/.*/", NULL};
     test = NULL;
     test_context = NULL;
 
@@ -59,10 +60,7 @@ setup (void)
     cut_listener_attach_to_run_context(CUT_LISTENER(report), run_context);
 
     test_case = cut_test_case_new("stub test case",
-                                  NULL, NULL,
-                                  get_current_test_context,
-                                  set_current_test_context,
-                                  NULL, NULL);
+                                  NULL, NULL, NULL, NULL);
 }
 
 void
@@ -90,11 +88,13 @@ run (void)
     gboolean success;
     CutTestContext *original_test_context;
 
-    test_context = cut_test_context_new(NULL, test_case, NULL, test);
-    original_test_context = get_current_test_context();
-    set_current_test_context(test_context);
-    success = cut_test_run(test, test_context, run_context);
-    set_current_test_context(original_test_context);
+    test_context = cut_test_context_new(run_context,
+                                        NULL, test_case, NULL, test);
+    original_test_context = cut_get_current_test_context();
+    cut_set_current_test_context(test_context);
+    success = cut_test_runner_run_test(CUT_TEST_RUNNER(run_context),
+                                       test, test_context);
+    cut_set_current_test_context(original_test_context);
 
     g_object_unref(test_context);
     test_context = NULL;
@@ -108,11 +108,13 @@ test_report_success (void)
         "  <result>\n"
         "    <test-case>\n"
         "      <name>stub test case</name>\n"
+        "      <start-time>1970-01-01T00:00:00Z</start-time>\n"
         "      <elapsed>.+?</elapsed>\n"
         "    </test-case>\n"
         "    <test>\n"
         "      <name>stub-success-test</name>\n"
         "      <description>A success test</description>\n"
+        "      <start-time>" ISO8601_PATTERN "</start-time>\n"
         "      <elapsed>.+?</elapsed>\n"
         "      <option>\n"
         "        <name>price</name>\n"
@@ -124,6 +126,7 @@ test_report_success (void)
         "      </option>\n"
         "    </test>\n"
         "    <status>success</status>\n"
+        "    <start-time>" ISO8601_PATTERN "</start-time>\n"
         "    <elapsed>.+?</elapsed>\n"
         "  </result>\n";
 
@@ -149,10 +152,12 @@ test_report_failure (void)
         "  <result>\n"
         "    <test-case>\n"
         "      <name>stub test case</name>\n"
+        "      <start-time>1970-01-01T00:00:00Z</start-time>\n"
         "      <elapsed>.+?</elapsed>\n"
         "    </test-case>\n"
         "    <test>\n"
         "      <name>stub-failure-test</name>\n"
+        "      <start-time>" ISO8601_PATTERN "</start-time>\n"
         "      <elapsed>.+?</elapsed>\n"
         "    </test>\n"
         "    <status>failure</status>\n"
@@ -160,10 +165,12 @@ test_report_failure (void)
         "    <backtrace>\n"
         "      <entry>\n"
         "        <file>test-cut-report-xml\\.c</file>\n"
-        "        <line>27</line>\n"
-        "        <info>stub_failure_test\\(\\)</info>\n"
+        "        <line>28</line>\n"
+        "        <info>stub_failure_test\\(\\): "
+                      "cut_fail\\(&quot;This test should fail&quot;\\)</info>\n"
         "      </entry>\n"
         "    </backtrace>\n"
+        "    <start-time>" ISO8601_PATTERN "</start-time>\n"
         "    <elapsed>.+?</elapsed>\n"
         "  </result>\n";
 
@@ -187,10 +194,12 @@ test_report_pending (void)
         "  <result>\n"
         "    <test-case>\n"
         "      <name>stub test case</name>\n"
+        "      <start-time>1970-01-01T00:00:00Z</start-time>\n"
         "      <elapsed>.+?</elapsed>\n"
         "    </test-case>\n"
         "    <test>\n"
         "      <name>stub-pending-test</name>\n"
+        "      <start-time>" ISO8601_PATTERN "</start-time>\n"
         "      <elapsed>.+?</elapsed>\n"
         "    </test>\n"
         "    <status>pending</status>\n"
@@ -198,10 +207,13 @@ test_report_pending (void)
         "    <backtrace>\n"
         "      <entry>\n"
         "        <file>test-cut-report-xml\\.c</file>\n"
-        "        <line>39</line>\n"
-        "        <info>stub_pending_test\\(\\)</info>\n"
+        "        <line>40</line>\n"
+        "        <info>stub_pending_test\\(\\): "
+                  "cut_pend\\(&quot;This test has been pending ever!&quot;\\)"
+                "</info>\n"
         "      </entry>\n"
         "    </backtrace>\n"
+        "    <start-time>" ISO8601_PATTERN "</start-time>\n"
         "    <elapsed>.+?</elapsed>\n"
         "  </result>\n";
 
@@ -225,10 +237,12 @@ test_report_notification (void)
         "  <result>\n"
         "    <test-case>\n"
         "      <name>stub test case</name>\n"
+        "      <start-time>1970-01-01T00:00:00Z</start-time>\n"
         "      <elapsed>.+?</elapsed>\n"
         "    </test-case>\n"
         "    <test>\n"
         "      <name>stub-notification-test</name>\n"
+        "      <start-time>" ISO8601_PATTERN "</start-time>\n"
         "      <elapsed>.+?</elapsed>\n"
         "    </test>\n"
         "    <status>notification</status>\n"
@@ -236,10 +250,14 @@ test_report_notification (void)
         "    <backtrace>\n"
         "      <entry>\n"
         "        <file>test-cut-report-xml\\.c</file>\n"
-        "        <line>45</line>\n"
-        "        <info>stub_notification_test\\(\\)</info>\n"
+        "        <line>46</line>\n"
+        "        <info>stub_notification_test\\(\\): "
+                  "cut_notify\\(&quot;This test has been "
+                                     "notifable ever!&quot;\\)"
+                "</info>\n"
         "      </entry>\n"
         "    </backtrace>\n"
+        "    <start-time>" ISO8601_PATTERN "</start-time>\n"
         "    <elapsed>.+?</elapsed>\n"
         "  </result>\n";
 
@@ -263,10 +281,12 @@ test_report_error (void)
         "  <result>\n"
         "    <test-case>\n"
         "      <name>stub test case</name>\n"
+        "      <start-time>1970-01-01T00:00:00Z</start-time>\n"
         "      <elapsed>.+?</elapsed>\n"
         "    </test-case>\n"
         "    <test>\n"
         "      <name>stub-error-test</name>\n"
+        "      <start-time>" ISO8601_PATTERN "</start-time>\n"
         "      <elapsed>.+?</elapsed>\n"
         "    </test>\n"
         "    <status>error</status>\n"
@@ -274,10 +294,13 @@ test_report_error (void)
         "    <backtrace>\n"
         "      <entry>\n"
         "        <file>test-cut-report-xml\\.c</file>\n"
-        "        <line>33</line>\n"
-        "        <info>stub_error_test\\(\\)</info>\n"
+        "        <line>34</line>\n"
+        "        <info>stub_error_test\\(\\): "
+                      "cut_error\\(&quot;This test should error&quot;\\)"
+                "</info>\n"
         "      </entry>\n"
         "    </backtrace>\n"
+        "    <start-time>" ISO8601_PATTERN "</start-time>\n"
         "    <elapsed>.+?</elapsed>\n"
         "  </result>\n";
 
@@ -300,22 +323,27 @@ test_plural_reports (void)
         "  <result>\n"
         "    <test-case>\n"
         "      <name>stub test case</name>\n"
+        "      <start-time>" ISO8601_PATTERN "</start-time>\n"
         "      <elapsed>.+?</elapsed>\n"
         "    </test-case>\n"
         "    <test>\n"
         "      <name>stub-success-test</name>\n"
+        "      <start-time>" ISO8601_PATTERN "</start-time>\n"
         "      <elapsed>.+?</elapsed>\n"
         "    </test>\n"
         "    <status>success</status>\n"
+        "    <start-time>" ISO8601_PATTERN "</start-time>\n"
         "    <elapsed>.+?</elapsed>\n"
         "  </result>\n"
         "  <result>\n"
         "    <test-case>\n"
         "      <name>stub test case</name>\n"
+        "      <start-time>" ISO8601_PATTERN "</start-time>\n"
         "      <elapsed>.+?</elapsed>\n"
         "    </test-case>\n"
         "    <test>\n"
         "      <name>stub-error-test</name>\n"
+        "      <start-time>" ISO8601_PATTERN "</start-time>\n"
         "      <elapsed>.+?</elapsed>\n"
         "    </test>\n"
         "    <status>error</status>\n"
@@ -323,10 +351,13 @@ test_plural_reports (void)
         "    <backtrace>\n"
         "      <entry>\n"
         "        <file>test-cut-report-xml\\.c</file>\n"
-        "        <line>33</line>\n"
-        "        <info>stub_error_test\\(\\)</info>\n"
+        "        <line>34</line>\n"
+        "        <info>stub_error_test\\(\\): "
+                  "cut_error\\(&quot;This test should error&quot;\\)"
+                "</info>\n"
         "      </entry>\n"
         "    </backtrace>\n"
+        "    <start-time>" ISO8601_PATTERN "</start-time>\n"
         "    <elapsed>.+?</elapsed>\n"
         "  </result>\n";
 
@@ -339,7 +370,8 @@ test_plural_reports (void)
     g_signal_connect_after(test, "error",
                            G_CALLBACK(cb_test_signal), NULL);
     cut_test_case_add_test(test_case, test);
-    cut_assert_false(cut_test_case_run(test_case, run_context));
+    cut_assert_false(cut_test_runner_run_test_case(CUT_TEST_RUNNER(run_context),
+                                                   test_case));
 
     g_signal_handlers_disconnect_by_func(test,
                                          G_CALLBACK(cb_test_signal),
