@@ -125,8 +125,7 @@ equal_match (const CutSequenceMatchInfo *info1,
 }
 
 static gboolean
-equal_match_with_data (gconstpointer info1, gconstpointer info2,
-                       gpointer user_data)
+equal_match_with_data (gconstpointer info1, gconstpointer info2)
 {
     return equal_match(info1, info2);
 }
@@ -134,7 +133,7 @@ equal_match_with_data (gconstpointer info1, gconstpointer info2,
 static gboolean
 equal_matches (const GList *matches1, const GList *matches2)
 {
-    return gcut_list_equal(matches1, matches2, equal_match_with_data, NULL);
+    return gcut_list_equal(matches1, matches2, equal_match_with_data);
 }
 
 static gchar *
@@ -329,9 +328,8 @@ test_to_indexes_complex_for_char_sequence (void)
     if (equal_match(&info, actual_info)) {                              \
         cut_test_pass();                                                \
     } else {                                                            \
-        cut_test_fail(FAILURE,                                          \
-                      cut_take_printf("expected: %s\n"                  \
-                                      " but was: %s",                   \
+        cut_test_fail(cut_take_printf("expected: %s\n"                  \
+                                      "  actual: %s",                   \
                                       inspect_match(&info),             \
                                       inspect_match(actual_info)),      \
                       "cut_sequence_matcher_get_longest_match"          \
@@ -454,9 +452,8 @@ append_match_info (GList *list, gint begin, gint end, gint size)
                                                                         \
         inspected_expected = inspect_matches(_expected);                \
         inspected_actual = inspect_matches(_actual);                    \
-        cut_test_fail(FAILURE,                                          \
-                      cut_take_printf("expected: <%s>\n"                \
-                                      " but was: <%s>",                 \
+        cut_test_fail(cut_take_printf("expected: <%s>\n"                \
+                                      "  actual: <%s>",                 \
                                       inspected_expected,               \
                                       inspected_actual),                \
                       ## __VA_ARGS__);                                  \
@@ -654,8 +651,7 @@ test_get_blocks_complex_for_char_sequence (void)
 }
 
 static gboolean
-equal_operation_with_data (gconstpointer data1, gconstpointer data2,
-                           gpointer user_data)
+equal_operation_with_data (gconstpointer data1, gconstpointer data2)
 {
     const CutSequenceMatchOperation *operation1, *operation2;
 
@@ -672,7 +668,7 @@ static gboolean
 equal_operations (const GList *operations1, const GList *operations2)
 {
     return gcut_list_equal(operations1, operations2,
-                           equal_operation_with_data, NULL);
+                           equal_operation_with_data);
 }
 
 static const gchar *
@@ -731,9 +727,8 @@ inspect_operations (const GList *operations)
                                                                         \
         inspected_expected = inspect_operations(_expected);             \
         inspected_actual = inspect_operations(_actual);                 \
-        cut_test_fail(FAILURE,                                          \
-                      cut_take_printf("expected: <%s>\n"                \
-                                      " but was: <%s>",                 \
+        cut_test_fail(cut_take_printf("expected: <%s>\n"                \
+                                      "  actual: <%s>",                 \
                                       inspected_expected,               \
                                       inspected_actual),                \
                       "cut_sequence_matcher_get_operations(%s)",        \
