@@ -104,7 +104,7 @@ stub_teardown_function (void)
 void
 setup (void)
 {
-    gchar *test_names[] = {"/.*/", NULL};
+    const gchar *test_names[] = {"/.*/", NULL};
 
     set_error_on_setup = FALSE;
 
@@ -120,8 +120,6 @@ setup (void)
     test_object = cut_test_case_new("stub test case",
                                     stub_setup_function,
                                     stub_teardown_function,
-                                    get_current_test_context,
-                                    set_current_test_context,
                                     NULL, NULL);
     cuttest_add_test(test_object, "stub_test_1", stub_test_function1);
     cuttest_add_test(test_object, "stub_test_2", stub_test_function2);
@@ -136,8 +134,17 @@ teardown (void)
 }
 
 static void
-cb_count_around_test (CutTestCase *test_case, CutTest *test,
-                      CutTestContext *test_context, gpointer data)
+cb_count_start_test (CutTestCase *test_case, CutTest *test,
+                     CutTestContext *test_context, gpointer data)
+{
+    gint *count = data;
+    *count += 1;
+}
+
+static void
+cb_count_complete_test (CutTestCase *test_case, CutTest *test,
+                        CutTestContext *test_context, gboolean success,
+                        gpointer data)
 {
     gint *count = data;
     *count += 1;
@@ -174,9 +181,16 @@ test_teardown (void)
 void
 test_test_case_count (void)
 {
+    CutTestContainer *container;
     const gchar *test_names[] = {"/.*/", NULL};
-    cut_assert_equal_int(3, cut_test_case_get_n_tests(test_object, NULL));
-    cut_assert_equal_int(3, cut_test_case_get_n_tests(test_object, test_names));
+
+    container = CUT_TEST_CONTAINER(test_object);
+    cut_assert_equal_int(3,
+                         cut_test_container_get_n_tests(container, NULL));
+
+    cut_run_context_set_target_test_names(run_context, test_names);
+    cut_assert_equal_int(3,
+                         cut_test_container_get_n_tests(container, run_context));
 }
 
 void
@@ -262,10 +276,10 @@ test_start_signal (void)
 {
     gint n_start_tests = 0;
     g_signal_connect(test_object, "start-test",
-                     G_CALLBACK(cb_count_around_test), &n_start_tests);
+                     G_CALLBACK(cb_count_start_test), &n_start_tests);
     cut_assert(cut_test_case_run(test_object, run_context));
     g_signal_handlers_disconnect_by_func(test_object,
-                                         G_CALLBACK(cb_count_around_test),
+                                         G_CALLBACK(cb_count_start_test),
                                          &n_start_tests);
     cut_assert_equal_int(3, n_start_tests);
 }
@@ -345,10 +359,10 @@ test_complete_signal (void)
 {
     gint n_complete_tests = 0;
     g_signal_connect(test_object, "complete-test",
-                     G_CALLBACK(cb_count_around_test), &n_complete_tests);
+                     G_CALLBACK(cb_count_complete_test), &n_complete_tests);
     cut_assert(cut_test_case_run(test_object, run_context));
     g_signal_handlers_disconnect_by_func(test_object,
-                                         G_CALLBACK(cb_count_around_test),
+                                         G_CALLBACK(cb_count_complete_test),
                                          &n_complete_tests);
     cut_assert_equal_int(3, n_complete_tests);
 }
