@@ -29,64 +29,95 @@ static guint n_pass_assertion_signals = 0;
 static guint n_notification_signals = 0;
 static guint n_omission_signals = 0;
 
-#define PREPEND_GUINT(list, guint_value)                         \
-    list = g_list_prepend(list,                                  \
-                          GUINT_TO_POINTER(guint_value))
+static gint fail_line;
 
-#define cut_assert_n_signals(_n_start_signals, _n_complete_signals,     \
-                             _n_setup_calls, _n_teardown_calls,         \
-                             _n_tests,                                  \
-                             _n_assertions, _n_successes,               \
-                             _n_failures, _n_errors, _n_pendings,       \
-                             _n_notifications, _n_omissions) do         \
-{                                                                       \
-    GList *_result_summary;                                             \
-    const GList *_expected_result_summary;                              \
-    const GList *_actual_result_summary;                                \
-    guint __n_start_signals, __n_complete_signals;                      \
-    guint __n_setup_calls, __n_teardown_calls;                          \
-                                                                        \
-    __n_start_signals = (_n_start_signals);                             \
-    __n_complete_signals = (_n_complete_signals);                       \
-    __n_setup_calls = (_n_setup_calls);                                 \
-    __n_teardown_calls = (_n_teardown_calls);                           \
-                                                                        \
-    _result_summary =                                                   \
-        cuttest_result_summary_list_new((_n_tests),                     \
-                                        (_n_assertions),                \
-                                        (_n_successes),                 \
-                                        (_n_failures),                  \
-                                        (_n_errors),                    \
-                                        (_n_pendings),                  \
-                                        (_n_notifications),             \
-                                        (_n_omissions));                \
-    PREPEND_GUINT(_result_summary, __n_teardown_calls);                 \
-    PREPEND_GUINT(_result_summary, __n_setup_calls);                    \
-    PREPEND_GUINT(_result_summary, __n_complete_signals);               \
-    PREPEND_GUINT(_result_summary, __n_start_signals);                  \
-    _expected_result_summary =                                          \
-        cut_take_result_summary_list(_result_summary);                  \
-                                                                        \
-    _result_summary =                                                   \
-        cuttest_result_summary_list_new(n_tests,                        \
-                                        n_pass_assertion_signals,       \
-                                        n_success_signals,              \
-                                        n_failure_signals,              \
-                                        n_error_signals,                \
-                                        n_pending_signals,              \
-                                        n_notification_signals,         \
-                                        n_omission_signals);            \
-    PREPEND_GUINT(_result_summary, n_teardown_calls);                   \
-    PREPEND_GUINT(_result_summary, n_setup_calls);                      \
-    PREPEND_GUINT(_result_summary, n_complete_signals);                 \
-    PREPEND_GUINT(_result_summary, n_start_signals);                    \
-    _actual_result_summary =                                            \
-        cut_take_result_summary_list(_result_summary);                  \
-                                                                        \
-    gcut_assert_equal_list_uint(_expected_result_summary,               \
-                                _actual_result_summary);                \
+#define MARK_FAIL(assertion) do                 \
+{                                               \
+    fail_line = __LINE__;                       \
+    assertion;                                  \
 } while (0)
 
+#define FAIL_LOCATION (cut_take_printf("%s:%d", __FILE__, fail_line))
+
+
+static void
+cut_assert_n_signals_helper(guint expected_n_start_signals,
+                            guint expected_n_complete_signals,
+                            guint expected_n_setup_calls,
+                            guint expected_n_teardown_calls,
+                            guint expected_n_tests,
+                            guint expected_n_assertions,
+                            guint expected_n_successes,
+                            guint expected_n_failures,
+                            guint expected_n_errors,
+                            guint expected_n_pendings,
+                            guint expected_n_notifications,
+                            guint expected_n_omissions)
+{
+    GList *result_summary;
+    const GList *expected_result_summary;
+    const GList *actual_result_summary;
+
+#define APPEND(value)                                                   \
+    result_summary = g_list_append(result_summary, GUINT_TO_POINTER(value))
+
+    result_summary = NULL;
+    APPEND(expected_n_start_signals);
+    APPEND(expected_n_complete_signals);
+    APPEND(expected_n_setup_calls);
+    APPEND(expected_n_teardown_calls);
+    APPEND(expected_n_tests);
+    APPEND(expected_n_assertions);
+    APPEND(expected_n_successes);
+    APPEND(expected_n_failures);
+    APPEND(expected_n_errors);
+    APPEND(expected_n_pendings);
+    APPEND(expected_n_notifications);
+    APPEND(expected_n_omissions);
+    expected_result_summary = gcut_take_list(result_summary, NULL);
+
+    result_summary = NULL;
+    APPEND(n_start_signals);
+    APPEND(n_complete_signals);
+    APPEND(n_setup_calls);
+    APPEND(n_teardown_calls);
+    APPEND(n_tests);
+    APPEND(n_pass_assertion_signals);
+    APPEND(n_success_signals);
+    APPEND(n_failure_signals);
+    APPEND(n_error_signals);
+    APPEND(n_pending_signals);
+    APPEND(n_notification_signals);
+    APPEND(n_omission_signals);
+    actual_result_summary = gcut_take_list(result_summary, NULL);
+#undef APPEND
+
+    gcut_assert_equal_list_uint(expected_result_summary,
+                                actual_result_summary);
+}
+
+#define cut_assert_n_signals(n_start_signals, n_complete_signals,       \
+                             n_setup_calls, n_teardown_calls,           \
+                             n_tests,                                   \
+                             n_assertions, n_successes,                 \
+                             n_failures, n_errors, n_pendings,          \
+                             n_notifications, n_omissions)              \
+    cut_trace_with_info_expression(                                     \
+        cut_assert_n_signals_helper(n_start_signals,                    \
+                                    n_complete_signals,                 \
+                                    n_setup_calls, n_teardown_calls,    \
+                                    n_tests, n_assertions,              \
+                                    n_successes, n_failures,            \
+                                    n_errors, n_pendings,               \
+                                    n_notifications, n_omissions),      \
+        cut_assert_n_signals(n_start_signals,                           \
+                             n_complete_signals,                        \
+                             n_setup_calls, n_teardown_calls,           \
+                             n_tests, n_assertions,                     \
+                             n_successes, n_failures,                   \
+                             n_errors, n_pendings,                      \
+                             n_notifications, n_omissions))
+
 static void
 stub_setup (void)
 {
@@ -119,10 +150,10 @@ setup (void)
     run_context = CUT_RUN_CONTEXT(cut_test_runner_new());
     test_case = cut_test_case_new("driver test case",
                                   stub_setup, stub_teardown,
-                                  get_current_test_context,
-                                  set_current_test_context,
                                   NULL, NULL);
     test_iterator = NULL;
+
+    fail_line = 0;
 }
 
 void
@@ -247,11 +278,13 @@ run (void)
 
     test = CUT_TEST(test_iterator);
     connect_signals(test);
-    test_context = cut_test_context_new(NULL, test_case, NULL, NULL);
-    original_test_context = get_current_test_context();
-    set_current_test_context(test_context);
-    success = cut_test_run(test, test_context, run_context);
-    set_current_test_context(original_test_context);
+    test_context = cut_test_context_new(run_context,
+                                        NULL, test_case, NULL, NULL);
+    original_test_context = cut_get_current_test_context();
+    cut_set_current_test_context(test_context);
+    success = cut_test_runner_run_test_iterator(CUT_TEST_RUNNER(run_context),
+                                                test_iterator, test_context);
+    cut_set_current_test_context(original_test_context);
     disconnect_signals(test);
 
     g_object_unref(test_context);
@@ -280,12 +313,15 @@ stub_success_iterated_test (gconstpointer data)
 void
 test_success (void)
 {
+    CutTestContainer *container;
+
     test_iterator = cut_test_iterator_new("success test iterator",
                                           stub_success_iterated_test,
                                           stub_iterated_data);
-    cut_assert_equal_uint(0, cut_test_iterator_get_n_tests(test_iterator));
+    container = CUT_TEST_CONTAINER(test_iterator);
+    cut_assert_equal_uint(0, cut_test_container_get_n_tests(container, NULL));
     cut_assert_true(run());
-    cut_assert_equal_uint(3, cut_test_iterator_get_n_tests(test_iterator));
+    cut_assert_equal_uint(3, cut_test_container_get_n_tests(container, NULL));
 
     cut_assert_n_signals(1, 1, 3, 3, 3, 0, 1, 0, 0, 0, 0, 0);
     cut_assert_test_result_summary(run_context, 3, 6, 3, 0, 0, 0, 0, 0);
@@ -295,19 +331,22 @@ static void
 stub_failure_iterated_test (gconstpointer data)
 {
     cut_assert_true(TRUE, "always pass");
-    cut_assert_equal_int(2, GPOINTER_TO_INT(data));
+    MARK_FAIL(cut_assert_equal_int(2, GPOINTER_TO_INT(data)));
     cut_assert_true(TRUE, "always pass if come here");
 }
 
 void
 test_failure (void)
 {
+    CutTestContainer *container;
+
     test_iterator = cut_test_iterator_new("failure test iterator",
                                           stub_failure_iterated_test,
                                           stub_iterated_data);
-    cut_assert_equal_uint(0, cut_test_iterator_get_n_tests(test_iterator));
+    container = CUT_TEST_CONTAINER(test_iterator);
+    cut_assert_equal_uint(0, cut_test_container_get_n_tests(container, NULL));
     cut_assert_false(run());
-    cut_assert_equal_uint(3, cut_test_iterator_get_n_tests(test_iterator));
+    cut_assert_equal_uint(3, cut_test_container_get_n_tests(container, NULL));
 
     cut_assert_n_signals(1, 1, 3, 3, 3, 0, 0, 1, 0, 0, 0, 0);
     cut_assert_test_result_summary(run_context, 3, 5, 1, 2, 0, 0, 0, 0);
@@ -316,7 +355,8 @@ test_failure (void)
                            NULL,
                            "<2 == GPOINTER_TO_INT(data)>\n"
                            "expected: <2>\n"
-                           " but was: <1>",
+                           "  actual: <1>",
+                           FAIL_LOCATION,
                            "stub_failure_iterated_test");
     cut_assert_test_result(run_context, 1, CUT_TEST_RESULT_SUCCESS,
                            "failure test iterator (Second)",
@@ -326,7 +366,8 @@ test_failure (void)
                            NULL,
                            "<2 == GPOINTER_TO_INT(data)>\n"
                            "expected: <2>\n"
-                           " but was: <3>",
+                           "  actual: <3>",
+                           FAIL_LOCATION,
                            "stub_failure_iterated_test");
 }
 
@@ -335,19 +376,22 @@ stub_error_iterated_test (gconstpointer data)
 {
     cut_assert_true(TRUE, "always pass");
     if (GPOINTER_TO_INT(data) == 2)
-        cut_error("ERROR!");
+        MARK_FAIL(cut_error("ERROR!"));
     cut_assert_true(TRUE, "always pass if come here");
 }
 
 void
 test_error (void)
 {
+    CutTestContainer *container;
+
     test_iterator = cut_test_iterator_new("error test iterator",
                                           stub_error_iterated_test,
                                           stub_iterated_data);
-    cut_assert_equal_uint(0, cut_test_iterator_get_n_tests(test_iterator));
+    container = CUT_TEST_CONTAINER(test_iterator);
+    cut_assert_equal_uint(0, cut_test_container_get_n_tests(container, NULL));
     cut_assert_false(run());
-    cut_assert_equal_uint(3, cut_test_iterator_get_n_tests(test_iterator));
+    cut_assert_equal_uint(3, cut_test_container_get_n_tests(container, NULL));
 
     cut_assert_n_signals(1, 1, 3, 3, 3, 0, 0, 0, 1, 0, 0, 0);
     cut_assert_test_result_summary(run_context, 3, 5, 2, 0, 1, 0, 0, 0);
@@ -358,6 +402,7 @@ test_error (void)
                            "error test iterator (Second)",
                            "ERROR!",
                            NULL,
+                           FAIL_LOCATION,
                            "stub_error_iterated_test");
     cut_assert_test_result(run_context, 2, CUT_TEST_RESULT_SUCCESS,
                            "error test iterator (Third)",
@@ -369,19 +414,22 @@ stub_pending_iterated_test (gconstpointer data)
 {
     cut_assert_true(TRUE, "always pass");
     if (GPOINTER_TO_INT(data) == 2)
-        cut_pend("PENDING!");
+        MARK_FAIL(cut_pend("PENDING!"));
     cut_assert_true(TRUE, "always pass if come here");
 }
 
 void
 test_pending (void)
 {
+    CutTestContainer *container;
+
     test_iterator = cut_test_iterator_new("pending test iterator",
                                           stub_pending_iterated_test,
                                           stub_iterated_data);
-    cut_assert_equal_uint(0, cut_test_iterator_get_n_tests(test_iterator));
+    container = CUT_TEST_CONTAINER(test_iterator);
+    cut_assert_equal_uint(0, cut_test_container_get_n_tests(container, NULL));
     cut_assert_false(run());
-    cut_assert_equal_uint(3, cut_test_iterator_get_n_tests(test_iterator));
+    cut_assert_equal_uint(3, cut_test_container_get_n_tests(container, NULL));
 
     cut_assert_n_signals(1, 1, 3, 3, 3, 0, 0, 0, 0, 1, 0, 0);
     cut_assert_test_result_summary(run_context, 3, 5, 2, 0, 0, 1, 0, 0);
@@ -392,6 +440,7 @@ test_pending (void)
                            "pending test iterator (Second)",
                            "PENDING!",
                            NULL,
+                           FAIL_LOCATION,
                            "stub_pending_iterated_test");
     cut_assert_test_result(run_context, 2, CUT_TEST_RESULT_SUCCESS,
                            "pending test iterator (Third)",
@@ -403,19 +452,22 @@ stub_notification_iterated_test (gconstpointer data)
 {
     cut_assert_true(TRUE, "always pass");
     if (GPOINTER_TO_INT(data) == 2)
-        cut_notify("NOTIFICATION!");
+        MARK_FAIL(cut_notify("NOTIFICATION!"));
     cut_assert_true(TRUE, "always pass if come here");
 }
 
 void
 test_notification (void)
 {
+    CutTestContainer *container;
+
     test_iterator = cut_test_iterator_new("notification test iterator",
                                           stub_notification_iterated_test,
                                           stub_iterated_data);
-    cut_assert_equal_uint(0, cut_test_iterator_get_n_tests(test_iterator));
+    container = CUT_TEST_CONTAINER(test_iterator);
+    cut_assert_equal_uint(0, cut_test_container_get_n_tests(container, NULL));
     cut_assert_true(run());
-    cut_assert_equal_uint(3, cut_test_iterator_get_n_tests(test_iterator));
+    cut_assert_equal_uint(3, cut_test_container_get_n_tests(container, NULL));
 
     cut_assert_n_signals(1, 1, 3, 3, 3, 0, 0, 0, 0, 0, 1, 0);
     cut_assert_test_result_summary(run_context, 3, 6, 3, 0, 0, 0, 1, 0);
@@ -426,6 +478,7 @@ test_notification (void)
                            "notification test iterator (Second)",
                            "NOTIFICATION!",
                            NULL,
+                           FAIL_LOCATION,
                            "stub_notification_iterated_test");
     cut_assert_test_result(run_context, 2, CUT_TEST_RESULT_SUCCESS,
                            "notification test iterator (Second)",
@@ -440,19 +493,22 @@ stub_omission_iterated_test (gconstpointer data)
 {
     cut_assert_true(TRUE, "always pass");
     if (GPOINTER_TO_INT(data) == 2)
-        cut_omit("OMISSION!");
+        MARK_FAIL(cut_omit("OMISSION!"));
     cut_assert_true(TRUE, "always pass if come here");
 }
 
 void
 test_omission (void)
 {
+    CutTestContainer *container;
+
     test_iterator = cut_test_iterator_new("omission test iterator",
                                           stub_omission_iterated_test,
                                           stub_iterated_data);
-    cut_assert_equal_uint(0, cut_test_iterator_get_n_tests(test_iterator));
+    container = CUT_TEST_CONTAINER(test_iterator);
+    cut_assert_equal_uint(0, cut_test_container_get_n_tests(container, NULL));
     cut_assert_true(run());
-    cut_assert_equal_uint(3, cut_test_iterator_get_n_tests(test_iterator));
+    cut_assert_equal_uint(3, cut_test_container_get_n_tests(container, NULL));
 
     cut_assert_n_signals(1, 1, 3, 3, 3, 0, 0, 0, 0, 0, 0, 1);
     cut_assert_test_result_summary(run_context, 3, 5, 3, 0, 0, 0, 0, 1);
@@ -463,6 +519,7 @@ test_omission (void)
                            "omission test iterator (Second)",
                            "OMISSION!",
                            NULL,
+                           FAIL_LOCATION,
                            "stub_omission_iterated_test");
     cut_assert_test_result(run_context, 2, CUT_TEST_RESULT_SUCCESS,
                            "omission test iterator (Second)",
@@ -490,13 +547,16 @@ stub_error_in_data_setup_iterated_test (gconstpointer data)
 void
 test_error_in_data_setup (void)
 {
+    CutTestContainer *container;
+
     test_iterator =
         cut_test_iterator_new("error in data setup test iterator",
                               stub_error_in_data_setup_iterated_test,
                               stub_error_in_data_setup_iterated_data);
-    cut_assert_equal_uint(0, cut_test_iterator_get_n_tests(test_iterator));
+    container = CUT_TEST_CONTAINER(test_iterator);
+    cut_assert_equal_uint(0, cut_test_container_get_n_tests(container, NULL));
     cut_assert_false(run());
-    cut_assert_equal_uint(0, cut_test_iterator_get_n_tests(test_iterator));
+    cut_assert_equal_uint(0, cut_test_container_get_n_tests(container, NULL));
 
     cut_assert_n_signals(0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0);
     cut_assert_test_result_summary(run_context, 0, 0, 0, 0, 0, 0, 0, 0);
