@@ -1,4 +1,4 @@
-#include <cutter.h>
+#include <gcutter.h>
 #include <cutter/cut-test.h>
 #include <cutter/cut-test-runner.h>
 
@@ -14,8 +14,9 @@ void test_failure_signal(void);
 void test_pending_signal(void);
 void test_notification_signal(void);
 void test_omission_signal(void);
-void test_test_function (void);
+void test_test_function(void);
 void test_set_elapsed(void);
+void test_start_time(void);
 
 static CutRunContext *run_context;
 static CutTest *test;
@@ -151,11 +152,11 @@ run (void)
     CutTestContext *original_test_context;
     CutTestContext *test_context;
 
-    test_context = cut_test_context_new(NULL, NULL, NULL, test);
-    original_test_context = get_current_test_context();
-    set_current_test_context(test_context);
+    test_context = cut_test_context_new(run_context, NULL, NULL, NULL, test);
+    original_test_context = cut_get_current_test_context();
+    cut_set_current_test_context(test_context);
     success = cut_test_run(test, test_context, run_context);
-    set_current_test_context(original_test_context);
+    cut_set_current_test_context(original_test_context);
 
     g_object_unref(test_context);
 
@@ -328,6 +329,31 @@ test_set_elapsed (void)
     cut_assert_equal_double(0.0, 0.1, cut_test_get_elapsed(test));
 }
 
+void
+test_start_time (void)
+{
+    GTimeVal expected, actual;
+
+    test = cut_test_new("stub-test", stub_test_function);
+
+    expected.tv_sec = 0;
+    expected.tv_usec = 0;
+    cut_test_get_start_time(test, &actual);
+    gcut_assert_equal_time_val(&expected, &actual);
+
+    g_get_current_time(&expected);
+    g_signal_emit_by_name(test, "start", NULL);
+    cut_test_get_start_time(test, &actual);
+    expected.tv_usec = actual.tv_usec; /* don't compare usec */
+    gcut_assert_equal_time_val(&expected, &actual);
+
+    expected.tv_sec = 100000;
+    expected.tv_usec = 0;
+    cut_test_set_start_time(test, &expected);
+    cut_test_get_start_time(test, &actual);
+    gcut_assert_equal_time_val(&expected, &actual);
+}
+
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
 */
